@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,10 +7,9 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: daemon.c 8588 2009-06-02 01:48:48Z charles $
+ * $Id: daemon.c 10281 2010-02-24 18:14:40Z charles $
  */
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h> /* printf */
 #include <stdlib.h> /* exit, atoi */
@@ -21,6 +20,9 @@
 
 #include <fcntl.h> /* open */
 #include <signal.h>
+#ifdef HAVE_SYSLOG
+#include <syslog.h>
+#endif
 #include <unistd.h> /* daemon */
 
 #include <event.h>
@@ -54,7 +56,8 @@ getUsage( void )
            "A fast and easy BitTorrent client\n"
            "\n"
            MY_NAME " is a headless Transmission session\n"
-           "that can be controlled via transmission-remote or Clutch.\n"
+           "that can be controlled via transmission-remote\n"
+           "or the web interface.\n"
            "\n"
            "Usage: " MY_NAME " [options]";
 }
@@ -66,7 +69,10 @@ static const struct tr_option options[] =
     { 'B', "no-blocklist", "Disable peer blocklists", "B", 0, NULL },
     { 'c', "watch-dir", "Directory to watch for new .torrent files", "c", 1, "<directory>" },
     { 'C', "no-watch-dir", "Disable the watch-dir", "C", 0, NULL },
+    { 941, "incomplete-dir", "Where to store new torrents until they're complete", NULL, 1, "<directory>" },
+    { 942, "no-incomplete-dir", "Don't store incomplete torrents in a different location", NULL, 0, NULL },
     { 'd', "dump-settings", "Dump the settings and exit", "d", 0, NULL },
+    { 'e', "logfile", "Dump the log messages to this filename", "e", 1, "<filename>" },
     { 'f', "foreground", "Run in the foreground instead of daemonizing", "f", 0, NULL },
     { 'g', "config-dir", "Where to look for configuration files", "g", 1, "<path>" },
     { 'p', "port", "RPC port (Default: " TR_DEFAULT_RPC_PORT_STR ")", "p", 1, "<port>" },
@@ -77,6 +83,8 @@ static const struct tr_option options[] =
     { 'V', "version", "Show version number and exit", "V", 0, NULL },
     { 'w', "download-dir", "Where to save downloaded data", "w", 1, "<path>" },
     { 800, "paused", "Pause all torrents on startup", NULL, 0, NULL },
+    { 'o', "dht", "Enable distributed hash tables (DHT)", "o", 0, NULL },
+    { 'O', "no-dht", "Disable distributed hash tables (DHT)", "O", 0, NULL },
     { 'P', "peerport", "Port for incoming peers (Default: " TR_DEFAULT_PEER_PORT_STR ")", "P", 1, "<port>" },
     { 'm', "portmap", "Enable portmapping via NAT-PMP or UPnP", "m", 0, NULL },
     { 'M', "no-portmap", "Disable portmapping", "M", 0, NULL },
@@ -86,8 +94,10 @@ static const struct tr_option options[] =
     { 911, "encryption-preferred", "Prefer encrypted peer connections", "ep", 0, NULL },
     { 912, "encryption-tolerated", "Prefer unencrypted peer connections", "et", 0, NULL },
     { 'i', "bind-address-ipv4", "Where to listen for peer connections", "i", 1, "<ipv4 address>" },
-    { 'I', "bind-address-ipv6", "Where to listen for peer connections", "I", 1, "<ipv6 address" },
+    { 'I', "bind-address-ipv6", "Where to listen for peer connections", "I", 1, "<ipv6 address>" },
     { 'r', "rpc-bind-address", "Where to listen for RPC connections", "r", 1, "<ipv4 address>" },
+    { 953, "global-seedratio", "All torrents, unless overridden by a per-torrent setting, should seed until a specific ratio", "gsr", 1, "ratio" },
+    { 954, "no-global-seedratio", "All torrents, unless overridden by a per-torrent setting, should seed regardless of ratio", "GSR", 0, NULL },
     { 0, NULL, NULL, NULL, 0, NULL }
 };
 
@@ -99,9 +109,27 @@ showUsage( void )
 }
 
 static void
-gotsig( int sig UNUSED )
+gotsig( int sig )
 {
-    closing = TRUE;
+    switch( sig )
+    {
+        case SIGHUP:
+        {
+            tr_benc settings;
+            const char * configDir = tr_sessionGetConfigDir( mySession );
+            tr_inf( "Reloading settings from \"%s\"", configDir );
+            tr_bencInitDict( &settings, 0 );
+            tr_sessionLoadSettings( &settings, configDir, MY_NAME );
+            tr_sessionSet( mySession, &settings );
+            tr_bencFree( &settings );
+            tr_sessionReloadBlocklists( mySession );
+            break;
+        }
+
+        default:
+            closing = TRUE;
+            break;
+    }
 }
 
 #if defined(WIN32)
@@ -145,6 +173,8 @@ tr_daemon( int nochdir, int noclose )
                 /* send stdin, stdout, and stderr to /dev/null */
                 int i;
                 int fd = open( "/dev/null", O_RDWR, 0 );
+                if( fd < 0 )
+                    fprintf( stderr, "unable to open /dev/null: %s\n", tr_strerror(errno) );
                 for( i=0; i<3; ++i ) {
                     if( close( i ) )
                         return -1;
@@ -201,6 +231,50 @@ onFileAdded( tr_session * session, const char * dir, const char * file )
     }
 }
 
+static void
+printMessage( FILE * logfile, int level, const char * name, const char * message, const char * file, int line )
+{
+    if( logfile != NULL )
+    {
+        char timestr[64];
+        tr_getLogTimeStr( timestr, sizeof( timestr ) );
+        if( name )
+            fprintf( logfile, "[%s] %s %s (%s:%d)\n", timestr, name, message, file, line );
+        else
+            fprintf( logfile, "[%s] %s (%s:%d)\n", timestr, message, file, line );
+    }
+#ifdef HAVE_SYSLOG
+    else /* daemon... write to syslog */
+    {
+        int priority;
+
+        /* figure out the syslog priority */
+        switch( level ) {
+            case TR_MSG_ERR: priority = LOG_ERR; break;
+            case TR_MSG_DBG: priority = LOG_DEBUG; break;
+            default: priority = LOG_INFO; break;
+        }
+
+        if( name )
+            syslog( priority, "%s %s (%s:%d)", name, message, file, line );
+        else
+            syslog( priority, "%s (%s:%d)", message, file, line );
+    }
+#endif
+}
+
+static void
+pumpLogMessages( FILE * logfile )
+{
+    const tr_msg_list * l;
+    tr_msg_list * list = tr_getQueuedMessages( );
+
+    for( l=list; l!=NULL; l=l->next )
+        printMessage( logfile, l->level, l->name, l->message, l->file, l->line );
+
+    tr_freeMessageList( list );
+}
+
 int
 main( int argc, char ** argv )
 {
@@ -208,24 +282,23 @@ main( int argc, char ** argv )
     const char * optarg;
     tr_benc settings;
     tr_bool boolVal;
+    tr_bool loaded;
     tr_bool foreground = FALSE;
     tr_bool dumpSettings = FALSE;
     const char * configDir = NULL;
     dtr_watchdir * watchdir = NULL;
+    FILE * logfile = NULL;
 
     signal( SIGINT, gotsig );
     signal( SIGTERM, gotsig );
-#ifndef WIN32 
-    signal( SIGQUIT, gotsig );
-    signal( SIGPIPE, SIG_IGN );
-    signal( SIGHUP, SIG_IGN );
+#ifndef WIN32
+    signal( SIGHUP, gotsig );
 #endif
 
     /* load settings from defaults + config file */
     tr_bencInitDict( &settings, 0 );
     configDir = getConfigDir( argc, (const char**)argv );
-    tr_sessionLoadSettings( &settings, configDir, MY_NAME );
-    tr_bencDictAddBool( &settings, TR_PREFS_KEY_RPC_ENABLED, TRUE );
+    loaded = tr_sessionLoadSettings( &settings, configDir, MY_NAME );
 
     /* overwrite settings from the comamndline */
     tr_optind = 1;
@@ -243,8 +316,19 @@ main( int argc, char ** argv )
                       break;
             case 'C': tr_bencDictAddBool( &settings, PREF_KEY_DIR_WATCH_ENABLED, FALSE );
                       break;
+	    case 941:
+        	      tr_bencDictAddStr( &settings, TR_PREFS_KEY_INCOMPLETE_DIR, optarg );
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, TRUE );
+		      break;
+	    case 942:
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, FALSE );
+		      break;
             case 'd': dumpSettings = TRUE;
                       break;
+            case 'e': logfile = fopen( optarg, "a+" );
+                      if( logfile == NULL )
+                          fprintf( stderr, "Couldn't open \"%s\": %s\n", optarg, tr_strerror( errno ) );
+                      break;
             case 'f': foreground = TRUE;
                       break;
             case 'g': /* handled above */
@@ -252,6 +336,12 @@ main( int argc, char ** argv )
 	    case 'V': /* version */
 		      fprintf(stderr, "Transmission %s\n", LONG_VERSION_STRING);
 		      exit( 0 );
+	    case 'o':
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, TRUE );
+		      break;
+	    case 'O':
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, FALSE );
+		      break;
             case 'p': tr_bencDictAddInt( &settings, TR_PREFS_KEY_RPC_PORT, atoi( optarg ) );
                       break;
             case 't': tr_bencDictAddBool( &settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, TRUE );
@@ -291,11 +381,27 @@ main( int argc, char ** argv )
             case 'r':
                       tr_bencDictAddStr( &settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, optarg );
                       break;
+	    case 953:
+		      tr_bencDictAddReal( &settings, TR_PREFS_KEY_RATIO, atof(optarg) );
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
+		      break;
+	    case 954:
+		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, FALSE );
+		      break;
             default:  showUsage( );
                       break;
         }
     }
 
+    if( foreground && !logfile )
+        logfile = stderr;
+
+    if( !loaded )
+    {
+        printMessage( logfile, TR_MSG_ERR, MY_NAME, "Error loading config file -- exiting.", __FILE__, __LINE__ );
+        return -1;
+    }
+
     if( dumpSettings )
     {
         char * str = tr_bencToStr( &settings, TR_FMT_JSON, NULL );
@@ -306,12 +412,16 @@ main( int argc, char ** argv )
 
     if( !foreground && tr_daemon( TRUE, FALSE ) < 0 )
     {
-        fprintf( stderr, "failed to daemonize: %s\n", strerror( errno ) );
+        char buf[256];
+        tr_snprintf( buf, sizeof( buf ), "Failed to daemonize: %s", tr_strerror( errno ) );
+        printMessage( logfile, TR_MSG_ERR, MY_NAME, buf, __FILE__, __LINE__ );
         exit( 1 );
     }
 
     /* start the session */
-    mySession = tr_sessionInit( "daemon", configDir, FALSE, &settings );
+    mySession = tr_sessionInit( "daemon", configDir, TRUE, &settings );
+    tr_ninf( NULL, "Using settings from \"%s\"", configDir );
+    tr_sessionSaveSettings( mySession, configDir, &settings );
 
     if( tr_bencDictFindBool( &settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &boolVal ) && boolVal )
         tr_ninf( MY_NAME, "requiring authentication" );
@@ -342,13 +452,26 @@ main( int argc, char ** argv )
         tr_ctorFree( ctor );
     }
 
-    while( !closing )
-    {
-        tr_wait( 1000 ); /* sleep one second */
+#ifdef HAVE_SYSLOG
+    if( !foreground )
+        openlog( MY_NAME, LOG_CONS, LOG_DAEMON );
+#endif
+
+    while( !closing ) {
+        tr_wait_msec( 1000 ); /* sleep one second */
         dtr_watchdir_update( watchdir );
+        pumpLogMessages( logfile );
     }
 
     /* shutdown */
+#if HAVE_SYSLOG
+    if( !foreground )
+    {
+        syslog( LOG_INFO, "%s", "Closing session" );
+        closelog( );
+    }
+#endif
+
     printf( "Closing transmission session..." );
     tr_sessionSaveSettings( mySession, configDir, &settings );
     dtr_watchdir_free( watchdir );
