@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: watch.c 8630 2009-06-05 17:52:02Z charles $
+ * $Id: watch.c 10295 2010-02-27 22:13:29Z charles $
  */
 #ifdef WITH_INOTIFY
   #include <sys/inotify.h>
@@ -16,13 +16,14 @@
 #else
   #include <sys/types.h> /* stat */
   #include <sys/stat.h> /* stat */
-  #include <dirent.h> /* readdir */
   #include <event.h> /* evbuffer */
 #endif
 
 #include <errno.h>
 #include <string.h> /* strstr */
 
+#include <dirent.h> /* readdir */
+
 #include <libtransmission/transmission.h>
 #include <libtransmission/utils.h> /* tr_buildPath(), tr_inf() */
 #include "watch.h"
@@ -40,6 +41,18 @@ struct dtr_watchdir
 #endif
 };
 
+static tr_bool
+str_has_suffix( const char *str, const char *suffix )
+{
+    const size_t str_len = strlen( str );
+    const size_t suffix_len = strlen( suffix );
+    
+    if( str_len < suffix_len )
+        return FALSE;
+
+    return !strncasecmp( str + str_len - suffix_len, suffix, suffix_len );
+}
+
 /***
 ****  INOTIFY IMPLEMENTATION
 ***/
@@ -53,17 +66,41 @@ struct dtr_watchdir
 /* reasonable guess as to size of 50 events */
 #define BUF_LEN (EVENT_BATCH_COUNT * (EVENT_SIZE + 16) + 2048)
 
-#define DTR_INOTIFY_MASK (IN_CREATE|IN_CLOSE_WRITE|IN_MOVED_TO)
+#define DTR_INOTIFY_MASK (IN_CLOSE_WRITE|IN_MOVED_TO|IN_ONLYDIR)
 
 static void
 watchdir_new_impl( dtr_watchdir * w )
 {
     int i;
+    DIR * odir;
     w->inotify_fd = inotify_init( );
     tr_inf( "Using inotify to watch directory \"%s\"", w->dir );
     i = inotify_add_watch( w->inotify_fd, w->dir, DTR_INOTIFY_MASK );
+
     if( i < 0 )
+    {
         tr_err( "Unable to watch \"%s\": %s", w->dir, strerror (errno) );
+    }
+    else if(( odir = opendir( w->dir )))
+    {
+        struct dirent * d;
+
+        while(( d = readdir( odir )))
+        {
+            const char * name = d->d_name;
+
+            if( !name || *name=='.' ) /* skip dotfiles */
+                continue;
+            if( !str_has_suffix( name, ".torrent" ) ) /* skip non-torrents */
+                continue;
+
+            tr_inf( "Found new .torrent file \"%s\" in watchdir \"%s\"", name, w->dir );
+            w->callback( w->session, w->dir, name );
+        }
+
+        closedir( odir );
+    }
+
 }
 static void
 watchdir_free_impl( dtr_watchdir * w )
@@ -99,8 +136,12 @@ watchdir_update_impl( dtr_watchdir * w )
         int len = read( fd, buf, sizeof( buf ) );
         while (i < len) {
             struct inotify_event * event = (struct inotify_event *) &buf[i];
-            tr_inf( "Found new .torrent file \"%s\" in watchdir \"%s\"", event->name, w->dir );
-            w->callback( w->session, w->dir, event->name );
+            const char * name = event->name;
+            if( str_has_suffix( name, ".torrent" ) )
+            {
+                tr_inf( "Found new .torrent file \"%s\" in watchdir \"%s\"", name, w->dir );
+                w->callback( w->session, w->dir, name );
+            }
             i += EVENT_SIZE +  event->len;
         }
     }
@@ -141,7 +182,7 @@ is_file_in_list( struct evbuffer * buf, const char * filename, size_t len )
     tr_bool in_list;
     struct evbuffer * test = evbuffer_new( );
     add_file_to_list( test, filename, len );
-    in_list = evbuffer_find( buf, EVBUFFER_DATA( test ), EVBUFFER_LENGTH( test ) );
+    in_list = evbuffer_find( buf, EVBUFFER_DATA( test ), EVBUFFER_LENGTH( test ) ) != NULL;
     evbuffer_free( test );
     return in_list;
 }
@@ -164,19 +205,20 @@ watchdir_update_impl( dtr_watchdir * w )
         for( d = readdir( odir ); d != NULL; d = readdir( odir ) )
         {
             size_t len;
+            const char * name = d->d_name;
 
-            if( !d->d_name || *d->d_name=='.' ) /* skip dotfiles */
+            if( !name || *name=='.' ) /* skip dotfiles */
                 continue;
-            if( !strstr( d->d_name, ".torrent" ) ) /* skip non-torrents */
+            if( !str_has_suffix( name, ".torrent" ) ) /* skip non-torrents */
                 continue;
 
-            len = strlen( d->d_name );
-            add_file_to_list( curFiles, d->d_name, len );
+            len = strlen( name );
+            add_file_to_list( curFiles, name, len );
 
             /* if this file wasn't here last time, try adding it */
-            if( !is_file_in_list( w->lastFiles, d->d_name, len ) ) {
-                tr_inf( "Found new .torrent file \"%s\" in watchdir \"%s\"", d->d_name, w->dir );
-                w->callback( w->session, w->dir, d->d_name );
+            if( !is_file_in_list( w->lastFiles, name, len ) ) {
+                tr_inf( "Found new .torrent file \"%s\" in watchdir \"%s\"", name, w->dir );
+                w->callback( w->session, w->dir, name );
             }
         }
 
