@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,19 +7,24 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 8420 2009-05-16 19:41:41Z charles $
+ * $Id: actions.c 9989 2010-01-21 20:51:48Z charles $
  */
 
 #include <string.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+
 #include <libtransmission/transmission.h>
+
 #include "actions.h"
 #include "conf.h"
 #include "tr-core.h"
 #include "tr-prefs.h"
 #include "lock.h"
+
 #include "logo.h"
+#include "ratio-icon.h"
 #include "options-icon.h"
 #include "turtles.h"
 
@@ -46,15 +51,15 @@ action_cb( GtkAction * a,
 
 static GtkRadioActionEntry sort_radio_entries[] =
 {
-    { "sort-by-activity", NULL, N_( "Sort by _Activity" ), NULL, NULL, 0 },
-    { "sort-by-name",     NULL, N_( "Sort by _Name" ),     NULL, NULL, 1 },
-    { "sort-by-progress", NULL, N_( "Sort by _Progress" ), NULL, NULL, 2 },
-    { "sort-by-ratio",    NULL, N_( "Sort by _Ratio" ),    NULL, NULL, 3 },
-    { "sort-by-state",    NULL, N_( "Sort by _State" ),    NULL, NULL, 4 },
-    { "sort-by-tracker",  NULL, N_( "Sort by _Tracker" ),  NULL, NULL, 5 },
-    { "sort-by-age",      NULL, N_( "Sort by A_ge" ),      NULL, NULL, 6 },
-    { "sort-by-eta",      NULL, N_( "Sort by _ETA" ),      NULL, NULL, 7 },
-    { "sort-by-size",     NULL, N_( "Sort by Si_ze" ),     NULL, NULL, 8 }
+    { "sort-by-activity",  NULL, N_( "Sort by _Activity" ),  NULL, NULL, 0 },
+    { "sort-by-name",      NULL, N_( "Sort by _Name" ),      NULL, NULL, 1 },
+    { "sort-by-progress",  NULL, N_( "Sort by _Progress" ),  NULL, NULL, 2 },
+    { "sort-by-ratio",     NULL, N_( "Sort by Rati_o" ),     NULL, NULL, 3 },
+    { "sort-by-state",     NULL, N_( "Sort by Stat_e" ),     NULL, NULL, 4 },
+    { "sort-by-tracker",   NULL, N_( "Sort by T_racker" ),   NULL, NULL, 5 },
+    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 6 },
+    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 7 },
+    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 8 }
 };
 
 static void
@@ -71,7 +76,7 @@ sort_changed_cb( GtkAction            * action UNUSED,
 
 static GtkToggleActionEntry show_toggle_entries[] =
 {
-    { "toggle-main-window", NULL, N_( "_Main Window" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
+    { "toggle-main-window", NULL, N_( "_Show Transmission" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
     { "toggle-message-log", NULL, N_( "Message _Log" ), NULL, NULL, G_CALLBACK( action_cb ), FALSE }
 };
 
@@ -87,9 +92,9 @@ toggle_pref_cb( GtkToggleAction *  action,
 
 static GtkToggleActionEntry  pref_toggle_entries[] =
 {
-    { "alt-speed-enabled", NULL, N_( "Speed _Limit Mode" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "alt-speed-enabled", NULL, N_( "Enable Temporary Speed _Limits" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "minimal-view",      NULL, N_( "_Minimal View" ), "<alt>M", NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
-    { "sort-reversed",     NULL, N_( "_Reverse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "sort-reversed",     NULL, N_( "Re_verse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "show-filterbar",    NULL, N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "show-statusbar",    NULL, N_( "_Statusbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "show-toolbar",      NULL, N_( "_Toolbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE }
@@ -97,16 +102,20 @@ static GtkToggleActionEntry  pref_toggle_entries[] =
 
 static GtkActionEntry entries[] =
 {
+    { "file-menu", NULL, N_( "_File" ), NULL, NULL, NULL  },
     { "torrent-menu", NULL, N_( "_Torrent" ), NULL, NULL, NULL  },
     { "view-menu", NULL, N_( "_View" ), NULL, NULL, NULL  },
     { "sort-menu", NULL, N_( "_Sort Torrents By" ), NULL, NULL, NULL },
     { "edit-menu", NULL, N_( "_Edit" ), NULL, NULL, NULL },
     { "help-menu", NULL, N_( "_Help" ), NULL, NULL, NULL },
+    { "copy-magnet-link-to-clipboard",  GTK_STOCK_COPY, N_("Copy _Magnet Link to Clipboard" ), "<control>M", NULL,  G_CALLBACK( action_cb ) },
+    { "add-torrent-from-url",  GTK_STOCK_ADD, N_("Add _URL..." ), NULL, N_( "Add URL..." ),  G_CALLBACK( action_cb ) },
     { "add-torrent-toolbar",  GTK_STOCK_ADD, NULL, NULL, N_( "Add a torrent" ),  G_CALLBACK( action_cb ) },
-    { "add-torrent-menu", GTK_STOCK_ADD, N_( "_Add..." ), "<control>D", N_( "Add a torrent" ), G_CALLBACK( action_cb ) },
+    { "add-torrent-menu", GTK_STOCK_ADD, N_( "_Add File..." ), "<control>D", N_( "Add a torrent" ), G_CALLBACK( action_cb ) },
     { "start-torrent", GTK_STOCK_MEDIA_PLAY, N_( "_Start" ), "<control>S", N_( "Start torrent" ), G_CALLBACK( action_cb ) },
     { "show-stats", NULL, N_( "_Statistics" ), NULL, NULL, G_CALLBACK( action_cb ) },
-    { "verify-torrent", NULL, N_( "_Verify Local Data" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "donate", NULL, N_( "_Donate" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "verify-torrent", NULL, N_( "_Verify Local Data" ), "<control>V", NULL, G_CALLBACK( action_cb ) },
     { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
     { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
@@ -135,11 +144,14 @@ BuiltinIconInfo;
 
 static const BuiltinIconInfo my_fallback_icons[] =
 {
-    { tr_icon_logo, "transmission"      },
+    { tr_icon_logo, WINDOW_ICON         },
+    { tr_icon_logo, TRAY_ICON           },
+    { tr_icon_logo, NOTIFICATION_ICON   },
     { tr_icon_lock, "transmission-lock" },
     { options_icon, "options"           },
     { blue_turtle,  "alt-speed-on"      },
-    { grey_turtle,  "alt-speed-off"     }
+    { grey_turtle,  "alt-speed-off"     },
+    { ratio_icon,   "ratio"             }
 };
 
 static void
@@ -299,6 +311,15 @@ action_sensitize( const char * name,
     g_object_set( action, "sensitive", b, NULL );
 }
 
+void
+action_set_important( const char * name, gboolean b )
+{
+    GtkAction * action = get_action( name );
+
+    g_assert( action != NULL );
+    g_object_set( action, "is-important", b, NULL );
+}
+
 void
 action_toggle( const char * name,
                gboolean     b )
