@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: file-list.c 8668 2009-06-11 02:16:42Z charles $
+ * $Id: file-list.c 10173 2010-02-11 14:55:00Z charles $
  */
 
 #include <stddef.h>
@@ -18,11 +18,11 @@
 #include <gtk/gtk.h>
 
 #include <libtransmission/transmission.h>
-#include <libtransmission/utils.h> /* tr_getRatio */
 
 #include "file-list.h"
 #include "hig.h"
 #include "icons.h"
+#include "tr-prefs.h"
 
 enum
 {
@@ -109,10 +109,11 @@ refreshFilesForeach( GtkTreeModel * model,
     if( is_file )
     {
         tr_torrent * tor = data->tor;
-        const int enabled = tr_torrentGetFileDL( tor, index );
-        const int priority = tr_torrentGetFilePriority( tor, index );
+        const tr_info * inf = tr_torrentInfo( tor );
+        const int enabled = !inf->files[index].dnd;
+        const int priority = inf->files[index].priority;
         const uint64_t have = data->refresh_file_stat[index].bytesCompleted;
-        const int prog = (int)((100.0*have)/size);
+        const int prog = size ? (int)((100.0*have)/size) : 1;
 
         if( (priority!=old_priority) || (enabled!=old_enabled) || (have!=old_have) || (prog!=old_prog) )
             gtk_tree_store_set( data->store, iter, FC_PRIORITY, priority,
@@ -160,7 +161,7 @@ refreshFilesForeach( GtkTreeModel * model,
         }
         while( gtk_tree_model_iter_next( model, &child ) );
 
-        prog = (int)((100.0*have)/sub_size);
+        prog = sub_size ? (int)((100.0*have)/sub_size) : 1;
 
         if( (size!=sub_size) || (have!=old_have)
                              || (priority!=old_priority)
@@ -205,7 +206,11 @@ gtr_tree_model_foreach_postorder( GtkTreeModel            * model,
 static void
 refresh( FileData * data )
 {
-    tr_torrent * tor = tr_torrentFindFromId( tr_core_session( data->core ), data->torrentId );
+    tr_torrent * tor = NULL;
+    tr_session * session = tr_core_session( data->core );
+
+    if( session != NULL )
+        tor = tr_torrentFindFromId( session, data->torrentId );
 
     if( tor == NULL )
     {
@@ -214,7 +219,7 @@ refresh( FileData * data )
     else
     {
         tr_file_index_t fileCount;
-        data->tor = tr_torrentFindFromId( tr_core_session( data->core ), data->torrentId );
+        data->tor = tr_torrentFindFromId( session, data->torrentId );
         data->refresh_file_stat = tr_torrentFiles( tor, &fileCount );
 
         gtr_tree_model_foreach_postorder( data->model, refreshFilesForeach, data );
@@ -382,9 +387,10 @@ buildTree( GNode * node, gpointer gdata )
     const gboolean isLeaf = node->children == NULL;
 
     const char * mime_type = isLeaf ? get_mime_type_from_filename( child_data->name ) : DIRECTORY_MIME_TYPE;
-    GdkPixbuf * icon = get_mime_type_icon( mime_type, GTK_ICON_SIZE_MENU, build->w ); 
-    const int priority = isLeaf ? tr_torrentGetFilePriority( build->tor, child_data->index ) : 0;
-    const gboolean enabled = isLeaf ? tr_torrentGetFileDL( build->tor, child_data->index ) : TRUE;
+    GdkPixbuf * icon = get_mime_type_icon( mime_type, GTK_ICON_SIZE_MENU, build->w );
+    const tr_info * inf = tr_torrentInfo( build->tor );
+    const int priority = isLeaf ? inf->files[ child_data->index ].priority : 0;
+    const gboolean enabled = isLeaf ? !inf->files[ child_data->index ].dnd : TRUE;
 #if GTK_CHECK_VERSION(2,10,0)
     gtk_tree_store_insert_with_values( build->store, &child_iter, build->iter, INT_MAX,
                                        FC_INDEX, child_data->index,
@@ -416,7 +422,7 @@ buildTree( GNode * node, gpointer gdata )
     }
 
     g_object_unref( icon );
- 
+
     /* we're done with this node */
     g_free( child_data->name );
     g_free( child_data );
@@ -518,7 +524,7 @@ file_list_set_torrent( GtkWidget * w, int torrentId )
         }
 
         refresh( data );
-        data->timeout_tag = gtr_timeout_add_seconds( 2, refreshModel, data );
+        data->timeout_tag = gtr_timeout_add_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, refreshModel, data );
     }
 
     gtk_tree_view_set_model( GTK_TREE_VIEW( data->view ), data->model );
@@ -693,7 +699,6 @@ file_list_new( TrCore * core, int torrentId )
     /* create the view */
     view = gtk_tree_view_new( );
     tree_view = GTK_TREE_VIEW( view );
-    gtk_tree_view_set_fixed_height_mode( tree_view, TRUE );
     gtk_tree_view_set_rules_hint( tree_view, TRUE );
     gtk_container_set_border_width( GTK_CONTAINER( view ), GUI_PAD_BIG );
     g_signal_connect( view, "button-press-event",
@@ -714,7 +719,7 @@ file_list_new( TrCore * core, int torrentId )
                                                 "expand", TRUE,
                                                 "title", _( "File" ),
                                                 NULL ) );
-    gtk_tree_view_column_set_sizing( col, GTK_TREE_VIEW_COLUMN_FIXED );
+    gtk_tree_view_column_set_resizable( col, TRUE );
     rend = gtk_cell_renderer_pixbuf_new( );
     gtk_tree_view_column_pack_start( col, rend, FALSE );
     gtk_tree_view_column_add_attribute( col, rend, "pixbuf", FC_ICON );
@@ -730,20 +735,20 @@ file_list_new( TrCore * core, int torrentId )
     title = _( "Progress" );
     pango_layout = gtk_widget_create_pango_layout( view, title );
     pango_layout_get_pixel_size( pango_layout, &width, NULL );
-    width += GUI_PAD * 2;
+    width += 30; /* room for the sort indicator */
     g_object_unref( G_OBJECT( pango_layout ) );
     rend = gtk_cell_renderer_progress_new( );
     col = gtk_tree_view_column_new_with_attributes( title, rend, "value", FC_PROG, NULL );
     gtk_tree_view_column_set_fixed_width( col, width );
     gtk_tree_view_column_set_sizing( col, GTK_TREE_VIEW_COLUMN_FIXED );
     gtk_tree_view_column_set_sort_column_id( col, FC_PROG );
-    gtk_tree_view_append_column ( tree_view, col );
+    gtk_tree_view_append_column( tree_view, col );
 
     /* add "enabled" column */
     title = _( "Download" );
     pango_layout = gtk_widget_create_pango_layout( view, title );
     pango_layout_get_pixel_size( pango_layout, &width, NULL );
-    width += GUI_PAD * 2;
+    width += 30; /* room for the sort indicator */
     g_object_unref( G_OBJECT( pango_layout ) );
     rend = gtk_cell_renderer_toggle_new( );
     col = gtk_tree_view_column_new_with_attributes( title, rend, NULL );
@@ -751,13 +756,13 @@ file_list_new( TrCore * core, int torrentId )
     gtk_tree_view_column_set_sizing( col, GTK_TREE_VIEW_COLUMN_FIXED );
     gtk_tree_view_column_set_cell_data_func( col, rend, renderDownload, NULL, NULL );
     gtk_tree_view_column_set_sort_column_id( col, FC_ENABLED );
-    gtk_tree_view_append_column ( tree_view, col );
+    gtk_tree_view_append_column( tree_view, col );
 
     /* add priority column */
     title = _( "Priority" );
     pango_layout = gtk_widget_create_pango_layout( view, title );
     pango_layout_get_pixel_size( pango_layout, &width, NULL );
-    width += GUI_PAD * 2;
+    width += 30; /* room for the sort indicator */
     g_object_unref( G_OBJECT( pango_layout ) );
     rend = gtk_cell_renderer_text_new( );
     g_object_set( rend, "xalign", (gfloat)0.5, "yalign", (gfloat)0.5, NULL );
@@ -766,7 +771,7 @@ file_list_new( TrCore * core, int torrentId )
     gtk_tree_view_column_set_sizing( col, GTK_TREE_VIEW_COLUMN_FIXED );
     gtk_tree_view_column_set_sort_column_id( col, FC_PRIORITY );
     gtk_tree_view_column_set_cell_data_func( col, rend, renderPriority, NULL, NULL );
-    gtk_tree_view_append_column ( tree_view, col );
+    gtk_tree_view_append_column( tree_view, col );
 
     /* create the scrolled window and stick the view in it */
     scroll = gtk_scrolled_window_new( NULL, NULL );
