@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 9358 2009-10-22 02:25:26Z charles $
+ * $Id: main.c 10292 2010-02-27 18:00:24Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -61,8 +61,6 @@
 
 #define MY_NAME "transmission"
 
-#define REFRESH_INTERVAL_SECONDS 2
-
 #if GTK_CHECK_VERSION( 2, 8, 0 )
  #define SHOW_LICENSE
 static const char * LICENSE =
@@ -88,6 +86,108 @@ struct cbdata
     GtkTreeSelection  * sel;
 };
 
+/**
+***
+**/
+
+static int
+compareInts( const void * a, const void * b )
+{
+    return *(int*)a - *(int*)b;
+}
+
+static char*
+getDetailsDialogKey( GSList * id_list )
+{
+    int i;
+    int n;
+    int * ids;
+    GSList * l;
+    GString * gstr = g_string_new( NULL );
+
+    n = g_slist_length( id_list );
+    ids = g_new( int, n );
+    i = 0;
+    for( l=id_list; l!=NULL; l=l->next )
+        ids[i++] = GPOINTER_TO_INT( l->data );
+    g_assert( i == n );
+    qsort( ids, n, sizeof(int), compareInts );
+
+    for( i=0; i<n; ++i )
+        g_string_append_printf( gstr, "%d ", ids[i] );
+
+    g_free( ids );
+    return g_string_free( gstr, FALSE );
+}
+
+struct DetailsDialogHandle
+{
+    char * key;
+    GtkWidget * dialog;
+};
+
+static GSList*
+getSelectedTorrentIds( struct cbdata * data )
+{
+    GtkTreeSelection * s;
+    GtkTreeModel * model;
+    GSList * ids = NULL;
+    GList * paths = NULL;
+    GList * l;
+
+    /* build a list of the selected torrents' ids */
+    s = tr_window_get_selection( data->wind );
+    for( paths=l=gtk_tree_selection_get_selected_rows(s,&model); l; l=l->next ) {
+        GtkTreeIter iter;
+        if( gtk_tree_model_get_iter( model, &iter, l->data ) ) {
+            tr_torrent * tor;
+            gtk_tree_model_get( model, &iter, MC_TORRENT_RAW, &tor, -1 );
+            ids = g_slist_append( ids, GINT_TO_POINTER( tr_torrentId( tor ) ) );
+        }
+    }
+
+    /* cleanup */
+    g_list_foreach( paths, (GFunc)gtk_tree_path_free, NULL );
+    g_list_free( paths );
+    return ids;
+}
+
+static struct DetailsDialogHandle*
+findDetailsDialogFromIds( struct cbdata * cbdata, GSList * ids )
+{
+    GSList * l;
+    struct DetailsDialogHandle * ret = NULL;
+    char * key = getDetailsDialogKey( ids );
+
+    for( l=cbdata->details; l!=NULL && ret==NULL; l=l->next ) {
+        struct DetailsDialogHandle * h = l->data;
+        if( !strcmp( h->key, key ) )
+            ret = h;
+    }
+
+    g_free( key );
+    return ret;
+}
+
+static struct DetailsDialogHandle*
+findDetailsDialogFromWidget( struct cbdata * cbdata, gpointer w )
+{
+    GSList * l;
+    struct DetailsDialogHandle * ret = NULL;
+
+    for( l=cbdata->details; l!=NULL && ret==NULL; l=l->next ) {
+        struct DetailsDialogHandle * h = l->data;
+        if( h->dialog == w )
+            ret = h;
+    }
+
+    return ret;
+}
+
+/***
+****
+***/
+
 static void           appsetup( TrWindow * wind,
                                 GSList *   args,
                                 struct     cbdata *,
@@ -123,6 +223,51 @@ static void           prefschanged( TrCore *     core,
 
 static gboolean       updatemodel( gpointer gdata );
 
+/***
+****
+***/
+
+#ifdef HAVE_LIBGCONF
+ #include <gconf/gconf.h>
+ #include <gconf/gconf-client.h>
+#endif
+
+static void
+registerMagnetLinkHandler( void )
+{
+#ifdef HAVE_LIBGCONF
+    GError * err;
+    GConfValue * value;
+    GConfClient * client = gconf_client_get_default( );
+    const char * key = "/desktop/gnome/url-handlers/magnet/command";
+
+    /* if there's already a manget handler registered, don't do anything */
+    value = gconf_client_get( client, key, NULL );
+    if( value != NULL )
+    {
+        gconf_value_free( value );
+        return;
+    }
+
+    err = NULL;
+    if( !gconf_client_set_string( client, key, "transmission '%s'", &err ) )
+    {
+        tr_inf( "Unable to register Transmission as default magnet link handler: \"%s\"", err->message );
+        g_clear_error( &err );
+    }
+    else
+    {
+        gconf_client_set_bool( client, "/desktop/gnome/url-handlers/magnet/needs_terminal", FALSE, NULL );
+        gconf_client_set_bool( client, "/desktop/gnome/url-handlers/magnet/enabled", TRUE, NULL );
+        tr_inf( "Transmission registered as default magnet link handler" );
+    }
+#endif
+}
+
+/***
+****
+***/
+
 struct counts_data
 {
     int    totalCount;
@@ -179,6 +324,7 @@ refreshActions( struct cbdata * data )
     action_sensitize( "show-torrent-properties", counts.totalCount != 0 );
     action_sensitize( "open-torrent-folder", counts.totalCount == 1 );
     action_sensitize( "relocate-torrent", counts.totalCount == 1 );
+    action_sensitize( "copy-magnet-link-to-clipboard", counts.totalCount == 1 );
 
     canUpdate = 0;
     gtk_tree_selection_selected_foreach( s, accumulateCanUpdateForeach, &canUpdate );
@@ -201,34 +347,6 @@ refreshActions( struct cbdata * data )
     }
 }
 
-static void
-refreshDetailsDialog( struct cbdata * data, GtkWidget * details )
-{
-    GtkTreeSelection * s = tr_window_get_selection( data->wind );
-    GtkTreeModel * model;
-    GSList * ids = NULL;
-    GList * selrows = NULL;
-    GList * l;
-
-    /* build a list of the selected torrents' ids */
-    s = tr_window_get_selection( data->wind );
-    for( selrows=l=gtk_tree_selection_get_selected_rows(s,&model); l; l=l->next ) {
-        GtkTreeIter iter;
-        if( gtk_tree_model_get_iter( model, &iter, l->data ) ) {
-            tr_torrent * tor;
-            gtk_tree_model_get( model, &iter, MC_TORRENT_RAW, &tor, -1 );
-            ids = g_slist_append( ids, GINT_TO_POINTER( tr_torrentId( tor ) ) );
-        }
-    }
-
-    torrent_inspector_set_torrents( details, ids );
-
-    /* cleanup */
-    g_slist_free( ids );
-    g_list_foreach( selrows, (GFunc)gtk_tree_path_free, NULL );
-    g_list_free( selrows );
-}
-
 static void
 selectionChangedCB( GtkTreeSelection * s UNUSED, gpointer data )
 {
@@ -272,9 +390,14 @@ signal_handler( int sig )
     else switch( sig )
     {
         case SIGINT:
+        case SIGTERM:
             g_message( _( "Got signal %d; trying to shut down cleanly.  Do it again if it gets stuck." ), sig );
             doAction( "quit", sighandler_cbdata );
             break;
+
+        default:
+            g_message( "unhandled signal" );
+            break;
     }
 }
 
@@ -282,10 +405,11 @@ static void
 setupsighandlers( void )
 {
     signal( SIGINT, signal_handler );
+    signal( SIGKILL, signal_handler );
 }
 
 static tr_rpc_callback_status
-onRPCChanged( tr_session            * session UNUSED,
+onRPCChanged( tr_session            * session,
               tr_rpc_callback_type    type,
               struct tr_torrent     * tor,
               void                  * gdata )
@@ -299,20 +423,18 @@ onRPCChanged( tr_session            * session UNUSED,
             tr_core_add_torrent( cbdata->core, tr_torrent_new_preexisting( tor ), TRUE );
             break;
 
-        case TR_RPC_TORRENT_STARTED:
-            /* this should be automatic */
-            break;
-
-        case TR_RPC_TORRENT_STOPPED:
-            /* this should be automatic */
-            break;
-
         case TR_RPC_TORRENT_REMOVING:
             tr_core_torrent_destroyed( cbdata->core, tr_torrentId( tor ) );
             break;
 
-        case TR_RPC_TORRENT_CHANGED:
         case TR_RPC_SESSION_CHANGED:
+            tr_sessionGetSettings( session, pref_get_all( ) );
+            break;
+
+        case TR_RPC_TORRENT_CHANGED:
+        case TR_RPC_TORRENT_MOVED:
+        case TR_RPC_TORRENT_STARTED:
+        case TR_RPC_TORRENT_STOPPED:
             /* nothing interesting to do here */
             break;
     }
@@ -321,6 +443,39 @@ onRPCChanged( tr_session            * session UNUSED,
     return TR_RPC_OK;
 }
 
+static GSList *
+checkfilenames( int argc, char **argv )
+{
+    int i;
+    GSList * ret = NULL;
+    char * pwd = g_get_current_dir( );
+
+    for( i=0; i<argc; ++i )
+    {
+        if( gtr_is_supported_url( argv[i] ) || gtr_is_magnet_link( argv[i] ) )
+        {
+            ret = g_slist_prepend( ret, g_strdup( argv[i] ) );
+        }
+        else /* local file */
+        {
+            char * filename = g_path_is_absolute( argv[i] )
+                            ? g_strdup ( argv[i] )
+                            : g_build_filename( pwd, argv[i], NULL );
+
+            if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
+                ret = g_slist_prepend( ret, filename );
+            else {
+                if( gtr_is_hex_hashcode( argv[i] ) )
+                    ret = g_slist_prepend( ret, g_strdup_printf( "magnet:?xt=urn:btih:%s", argv[i] ) );
+                g_free( filename );
+            }
+        }
+    }
+
+    g_free( pwd );
+    return g_slist_reverse( ret );
+}
+
 int
 main( int argc, char ** argv )
 {
@@ -344,7 +499,7 @@ main( int argc, char ** argv )
 #ifdef STATUS_ICON_SUPPORTED
         { "minimized",  'm', 0, G_OPTION_ARG_NONE,
           &startminimized,
-          _( "Start minimized in system tray" ), NULL },
+          _( "Start minimized in notification area" ), NULL },
 #endif
         { "config-dir", 'g', 0, G_OPTION_ARG_FILENAME, &configDir,
           _( "Where to look for configuration files" ), NULL },
@@ -363,7 +518,7 @@ main( int argc, char ** argv )
         g_thread_init( NULL );
 
     gerr = NULL;
-    if( !gtk_init_with_args( &argc, &argv, (char*)_( "[torrent files]" ), entries,
+    if( !gtk_init_with_args( &argc, &argv, (char*)_( "[torrent files or urls]" ), entries,
                              (char*)domain, &gerr ) )
     {
         fprintf( stderr, "%s\n", gerr->message );
@@ -434,10 +589,10 @@ main( int argc, char ** argv )
         sighandler_cbdata = cbdata;
 
         /* ensure the directories are created */
-        if(( str = pref_string_get( PREF_KEY_DIR_WATCH )))
-            gtr_mkdir_with_parents( str, 0777 );
         if(( str = pref_string_get( TR_PREFS_KEY_DOWNLOAD_DIR )))
             gtr_mkdir_with_parents( str, 0777 );
+        if(( str = pref_string_get( TR_PREFS_KEY_INCOMPLETE_DIR )))
+            gtr_mkdir_with_parents( str, 0777 );
 
         /* initialize the libtransmission session */
         session = tr_sessionInit( "gtk", configDir, TRUE, pref_get_all( ) );
@@ -464,6 +619,9 @@ main( int argc, char ** argv )
             && ( time( NULL ) - pref_int_get( "blocklist-date" ) > ( 60 * 60 * 24 * 7 ) ) )
                 tr_core_blocklist_update( cbdata->core );
 
+        /* if there's no magnet link handler registered, register us */
+        registerMagnetLinkHandler( );
+
         gtk_main( );
     }
     else if( err )
@@ -529,7 +687,7 @@ appsetup( TrWindow *      wind,
     prefschanged( cbdata->core, PREF_KEY_SHOW_TRAY_ICON, cbdata );
 
     /* start model update timer */
-    cbdata->timer = gtr_timeout_add_seconds( REFRESH_INTERVAL_SECONDS, updatemodel, cbdata );
+    cbdata->timer = gtr_timeout_add_seconds( MAIN_WINDOW_REFRESH_INTERVAL_SECONDS, updatemodel, cbdata );
     updatemodel( cbdata );
 
     /* either show the window or iconify it */
@@ -542,6 +700,28 @@ appsetup( TrWindow *      wind,
         cbdata->isIconified = FALSE; // ensure that the next toggle iconifies
         action_toggle( "toggle-main-window", FALSE );
     }
+
+    if( !pref_flag_get( PREF_KEY_USER_HAS_GIVEN_INFORMED_CONSENT ) )
+    {
+        GtkWidget * w = gtk_message_dialog_new( GTK_WINDOW( wind ),
+                                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                GTK_MESSAGE_INFO,
+                                                GTK_BUTTONS_NONE,
+                                                "%s",
+            _( "Transmission is a file sharing program.  When you run a torrent, its data will be made available to others by means of upload.  And of course, any content you share is your sole responsibility.\n\nYou probably knew this, so we won't tell you again." ) );
+        gtk_dialog_add_button( GTK_DIALOG( w ), GTK_STOCK_QUIT, GTK_RESPONSE_REJECT );
+        gtk_dialog_add_button( GTK_DIALOG( w ), _( "I _Accept" ), GTK_RESPONSE_ACCEPT );
+        gtk_dialog_set_default_response( GTK_DIALOG( w ), GTK_RESPONSE_ACCEPT );
+        switch( gtk_dialog_run( GTK_DIALOG( w ) ) ) {
+            case GTK_RESPONSE_ACCEPT:
+                /* only show it once */
+                pref_flag_set( PREF_KEY_USER_HAS_GIVEN_INFORMED_CONSENT, TRUE );
+                gtk_widget_destroy( w );
+                break;
+            default:
+                exit( 0 );
+        }
+    }
 }
 
 static void
@@ -627,19 +807,17 @@ winsetup( struct cbdata * cbdata,
     setupdrag( GTK_WIDGET( wind ), cbdata );
 }
 
-static gpointer
-quitThreadFunc( gpointer gdata )
+static gboolean
+onSessionClosed( gpointer gdata )
 {
     struct cbdata * cbdata = gdata;
-    gdk_threads_enter( );
-
-    tr_core_close( cbdata->core );
 
     /* shutdown the gui */
-    if( cbdata->details ) {
-        g_slist_foreach( cbdata->details, (GFunc)gtk_widget_destroy, NULL );
-        g_slist_free( cbdata->details );
+    while( cbdata->details != NULL ) {
+        struct DetailsDialogHandle * h = cbdata->details->data;
+        gtk_widget_destroy( h->dialog );
     }
+
     if( cbdata->prefs )
         gtk_widget_destroy( GTK_WIDGET( cbdata->prefs ) );
     if( cbdata->wind )
@@ -658,8 +836,21 @@ quitThreadFunc( gpointer gdata )
     g_free( cbdata );
 
     gtk_main_quit( );
-    gdk_threads_leave( );
 
+    return FALSE;
+}
+
+static gpointer
+sessionCloseThreadFunc( gpointer gdata )
+{
+    /* since tr_sessionClose() is a blocking function,
+     * call it from another thread... when it's done,
+     * punt the GUI teardown back to the GTK+ thread */
+    struct cbdata * cbdata = gdata;
+    gdk_threads_enter( );
+    tr_core_close( cbdata->core );
+    gtr_idle_add( onSessionClosed, gdata );
+    gdk_threads_leave( );
     return NULL;
 }
 
@@ -726,7 +917,7 @@ wannaquit( gpointer vdata )
                                    pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
 
     /* shut down libT */
-    g_thread_create( quitThreadFunc, vdata, TRUE, NULL );
+    g_thread_create( sessionCloseThreadFunc, vdata, TRUE, NULL );
 }
 
 static void
@@ -781,25 +972,18 @@ gotdrag( GtkWidget         * widget UNUSED,
                 continue;
 
             /* walk past "file://", if present */
-            if( g_str_has_prefix( filename, "file:" ) )
-            {
+            if( g_str_has_prefix( filename, "file:" ) ) {
                 filename += 5;
                 while( g_str_has_prefix( filename, "//" ) )
                     ++filename;
             }
 
-            /* if the file doesn't exist, the first part
-               might be a hostname ... walk past it. */
-            if( !g_file_test( filename, G_FILE_TEST_EXISTS ) )
-            {
-                char * pch = strchr( filename + 1, '/' );
-                if( pch != NULL )
-                    filename = pch;
-            }
+            g_debug( "got from drag: [%s]", filename );
 
-            /* finally, add it to the list of torrents to try adding */
             if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
                 paths = g_slist_prepend( paths, g_strdup( filename ) );
+            else
+                tr_core_add_from_url( data->core, filename );
         }
 
         /* try to add any torrents we found */
@@ -1010,6 +1194,10 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_sessionSetPexEnabled( tr, pref_flag_get( key ) );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_RENAME_PARTIAL_FILES ) )
+    {
+        tr_sessionSetIncompleteFileNamingEnabled( tr, pref_flag_get( key ) );
+    }
     else if( !strcmp( key, TR_PREFS_KEY_DHT_ENABLED ) )
     {
         tr_sessionSetDHTEnabled( tr, pref_flag_get( key ) );
@@ -1104,6 +1292,14 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_sessionSetPeerPortRandomOnStart( tr, pref_flag_get( key ) );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_INCOMPLETE_DIR ) )
+    {
+        tr_sessionSetIncompleteDir( tr, pref_string_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED ) )
+    {
+        tr_sessionSetIncompleteDirEnabled( tr, pref_flag_get( key ) );
+    }
 }
 
 static gboolean
@@ -1118,11 +1314,15 @@ updatemodel( gpointer gdata )
         tr_core_update( data->core );
 
         /* update the main window's statusbar and toolbar buttons */
-        if( data->wind )
+        if( data->wind != NULL )
             tr_window_update( data->wind );
 
         /* update the actions */
         refreshActions( data );
+
+        /* update the status tray icon */
+        if( data->icon != NULL )
+            tr_icon_refresh( data->icon );
     }
 
     return !done;
@@ -1175,39 +1375,39 @@ about( GtkWindow * parent )
 }
 
 static void
-startTorrentForeach( GtkTreeModel *      model,
-                     GtkTreePath  * path UNUSED,
-                     GtkTreeIter *       iter,
-                     gpointer       data UNUSED )
+appendIdToBencList( GtkTreeModel * m, GtkTreePath * path UNUSED,
+                    GtkTreeIter * iter, gpointer list )
 {
     tr_torrent * tor = NULL;
-
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
-    tr_torrentStart( tor );
+    gtk_tree_model_get( m, iter, MC_TORRENT_RAW, &tor, -1 );
+    tr_bencListAddInt( list, tr_torrentId( tor ) );
 }
 
-static void
-stopTorrentForeach( GtkTreeModel *      model,
-                    GtkTreePath  * path UNUSED,
-                    GtkTreeIter *       iter,
-                    gpointer       data UNUSED )
+static gboolean
+rpcOnSelectedTorrents( struct cbdata * data, const char * method )
 {
-    tr_torrent * tor = NULL;
+    tr_benc top, *args, *ids;
+    gboolean invoked = FALSE;
+    tr_session * session = tr_core_session( data->core );
+    GtkTreeSelection * s = tr_window_get_selection( data->wind );
 
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
-    tr_torrentStop( tor );
-}
+    tr_bencInitDict( &top, 2 );
+    tr_bencDictAddStr( &top, "method", method );
+    args = tr_bencDictAddDict( &top, "arguments", 1 );
+    ids = tr_bencDictAddList( args, "ids", 0 );
+    gtk_tree_selection_selected_foreach( s, appendIdToBencList, ids );
 
-static void
-updateTrackerForeach( GtkTreeModel *      model,
-                      GtkTreePath  * path UNUSED,
-                      GtkTreeIter *       iter,
-                      gpointer       data UNUSED )
-{
-    tr_torrent * tor = NULL;
+    if( tr_bencListSize( ids ) != 0 )
+    {
+        int json_len;
+        char * json = tr_bencToStr( &top, TR_FMT_JSON_LEAN, &json_len );
+        tr_rpc_request_exec_json( session, json, json_len, NULL, NULL );
+        g_free( json );
+        invoked = TRUE;
+    }
 
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
-    tr_torrentManualUpdate( tor );
+    tr_bencFree( &top );
+    return invoked;
 }
 
 static void
@@ -1223,19 +1423,6 @@ openFolderForeach( GtkTreeModel *           model,
     g_object_unref( G_OBJECT( gtor ) );
 }
 
-static void
-recheckTorrentForeach( GtkTreeModel *      model,
-                       GtkTreePath  * path UNUSED,
-                       GtkTreeIter *       iter,
-                       gpointer       data UNUSED )
-{
-    TrTorrent * gtor = NULL;
-
-    gtk_tree_model_get( model, iter, MC_TORRENT, &gtor, -1 );
-    tr_torrentVerify( tr_torrent_handle( gtor ) );
-    g_object_unref( G_OBJECT( gtor ) );
-}
-
 static gboolean
 msgwinclosed( void )
 {
@@ -1310,7 +1497,36 @@ static void
 detailsClosed( gpointer gdata, GObject * dead )
 {
     struct cbdata * data = gdata;
-    data->details = g_slist_remove( data->details, dead );
+    struct DetailsDialogHandle * h = findDetailsDialogFromWidget( data, dead );
+
+    if( h != NULL )
+    {
+        data->details = g_slist_remove( data->details, h );
+        g_free( h->key );
+        g_free( h );
+    }
+}
+
+static void
+copyMagnetLinkToClipboard( GtkWidget * w, tr_torrent * tor )
+{
+    char * magnet = tr_torrentGetMagnetLink( tor );
+    GdkDisplay * display = gtk_widget_get_display( w );
+    GdkAtom selection;
+    GtkClipboard * clipboard;
+
+    /* this is The Right Thing for copy/paste... */
+    selection = GDK_SELECTION_CLIPBOARD;
+    clipboard = gtk_clipboard_get_for_display( display, selection );
+    gtk_clipboard_set_text( clipboard, magnet, -1 );
+
+    /* ...but people using plain ol' X need this instead */
+    selection = GDK_SELECTION_PRIMARY;
+    clipboard = gtk_clipboard_get_for_display( display, selection );
+    gtk_clipboard_set_text( clipboard, magnet, -1 );
+
+    /* cleanup */
+    tr_free( magnet );
 }
 
 void
@@ -1319,7 +1535,11 @@ doAction( const char * action_name, gpointer user_data )
     struct cbdata * data = user_data;
     gboolean        changed = FALSE;
 
-    if(  !strcmp( action_name, "add-torrent-menu" )
+    if( !strcmp( action_name, "add-torrent-from-url" ) )
+    {
+        addURLDialog( data->wind, data->core );
+    }
+    else if(  !strcmp( action_name, "add-torrent-menu" )
       || !strcmp( action_name, "add-torrent-toolbar" ) )
     {
         addDialog( data->wind, data->core );
@@ -1333,12 +1553,6 @@ doAction( const char * action_name, gpointer user_data )
     {
         gtr_open_file( "http://www.transmissionbt.com/donate.php" );
     }
-    else if( !strcmp( action_name, "start-torrent" ) )
-    {
-        GtkTreeSelection * s = tr_window_get_selection( data->wind );
-        gtk_tree_selection_selected_foreach( s, startTorrentForeach, NULL );
-        changed |= gtk_tree_selection_count_selected_rows( s ) != 0;
-    }
     else if( !strcmp( action_name, "pause-all-torrents" ) )
     {
         pauseAllTorrents( data );
@@ -1347,27 +1561,39 @@ doAction( const char * action_name, gpointer user_data )
     {
         startAllTorrents( data );
     }
+    else if( !strcmp( action_name, "copy-magnet-link-to-clipboard" ) )
+    {
+        tr_torrent * tor = getFirstSelectedTorrent( data );
+        if( tor != NULL )
+        {
+            copyMagnetLinkToClipboard( GTK_WIDGET( data->wind ), tor );
+        }
+    }
     else if( !strcmp( action_name, "relocate-torrent" ) )
     {
         tr_torrent * tor = getFirstSelectedTorrent( data );
-        if( tor )
+        if( tor != NULL )
         {
             GtkWindow * parent = GTK_WINDOW( data->wind );
             GtkWidget * w = gtr_relocate_dialog_new( parent, tor );
             gtk_widget_show( w );
         }
     }
+    else if( !strcmp( action_name, "start-torrent" ) )
+    {
+        changed |= rpcOnSelectedTorrents( data, "torrent-start" );
+    }
     else if( !strcmp( action_name, "pause-torrent" ) )
     {
-        GtkTreeSelection * s = tr_window_get_selection( data->wind );
-        gtk_tree_selection_selected_foreach( s, stopTorrentForeach, NULL );
-        changed |= gtk_tree_selection_count_selected_rows( s ) != 0;
+        changed |= rpcOnSelectedTorrents( data, "torrent-stop" );
     }
     else if( !strcmp( action_name, "verify-torrent" ) )
     {
-        GtkTreeSelection * s = tr_window_get_selection( data->wind );
-        gtk_tree_selection_selected_foreach( s, recheckTorrentForeach, NULL );
-        changed |= gtk_tree_selection_count_selected_rows( s ) != 0;
+        changed |= rpcOnSelectedTorrents( data, "torrent-verify" );
+    }
+    else if( !strcmp( action_name, "update-tracker" ) )
+    {
+        changed |= rpcOnSelectedTorrents( data, "torrent-reannounce" );
     }
     else if( !strcmp( action_name, "open-torrent-folder" ) )
     {
@@ -1376,22 +1602,25 @@ doAction( const char * action_name, gpointer user_data )
     }
     else if( !strcmp( action_name, "show-torrent-properties" ) )
     {
-        GtkWidget * w = torrent_inspector_new( GTK_WINDOW( data->wind ), data->core );
-        data->details = g_slist_prepend( data->details, w );
-        g_object_weak_ref( G_OBJECT( w ), detailsClosed, data );
-        refreshDetailsDialog( data, w );
-        gtk_widget_show( w );
-    }
-    else if( !strcmp( action_name, "update-tracker" ) )
-    {
-        GtkTreeSelection * s = tr_window_get_selection( data->wind );
-        gtk_tree_selection_selected_foreach( s, updateTrackerForeach,
-                                             data->wind );
+        GtkWidget * w;
+        GSList * ids = getSelectedTorrentIds( data );
+        struct DetailsDialogHandle * h = findDetailsDialogFromIds( data, ids );
+        if( h != NULL )
+            w = h->dialog;
+        else {
+            h = g_new( struct DetailsDialogHandle, 1 );
+            h->key = getDetailsDialogKey( ids );
+            h->dialog = w = torrent_inspector_new( GTK_WINDOW( data->wind ), data->core );
+            torrent_inspector_set_torrents( w, ids );
+            data->details = g_slist_append( data->details, h );
+            g_object_weak_ref( G_OBJECT( w ), detailsClosed, data );
+        }
+        gtk_window_present( GTK_WINDOW( w ) );
+        g_slist_free( ids );
     }
     else if( !strcmp( action_name, "new-torrent" ) )
     {
-        GtkWidget * w = make_meta_ui( GTK_WINDOW( data->wind ),
-                                     tr_core_session( data->core ) );
+        GtkWidget * w = make_meta_ui( GTK_WINDOW( data->wind ), data->core );
         gtk_widget_show_all( w );
     }
     else if( !strcmp( action_name, "remove-torrent" ) )
