@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: stats.c 7463 2008-12-22 05:39:02Z charles $
+ * $Id: stats.c 10066 2010-02-01 04:54:10Z charles $
  */
 
 #include <glib/gi18n.h>
@@ -15,10 +15,11 @@
 #include "hig.h"
 #include "stats.h"
 #include "tr-core.h"
+#include "tr-prefs.h"
 
 enum
 {
-    TR_RESPONSE_CLEAR = 1
+    TR_RESPONSE_RESET = 1
 };
 
 struct stat_ui
@@ -101,10 +102,29 @@ dialogResponse( GtkDialog * dialog,
 {
     struct stat_ui * ui = gdata;
 
-    if( response == TR_RESPONSE_CLEAR )
+    if( response == TR_RESPONSE_RESET )
     {
-        tr_sessionClearStats( tr_core_session( ui->core ) );
-        updateStats( ui );
+        const char * primary = _( "Reset your statistics?" );
+        const char * secondary = _( "These statistics are for your information only.  "
+                                    "Resetting them doesn't affect the statistics logged by your BitTorrent trackers." );
+        const int flags = GTK_DIALOG_DESTROY_WITH_PARENT
+                        | GTK_DIALOG_MODAL;
+        GtkWidget * w = gtk_message_dialog_new( GTK_WINDOW( dialog ),
+                                                flags,
+                                                GTK_MESSAGE_QUESTION,
+                                                GTK_BUTTONS_NONE,
+                                                "%s", primary );
+        gtk_dialog_add_buttons( GTK_DIALOG( w ),
+                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                _( "_Reset" ), TR_RESPONSE_RESET,
+                                NULL );
+        gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ), "%s", secondary );
+        if( gtk_dialog_run( GTK_DIALOG( w ) ) == TR_RESPONSE_RESET )
+        {
+            tr_sessionClearStats( tr_core_session( ui->core ) );
+            updateStats( ui );
+        }
+        gtk_widget_destroy( w );
     }
 
     if( response == GTK_RESPONSE_CLOSE )
@@ -124,19 +144,18 @@ stats_dialog_create( GtkWindow * parent,
     GtkWidget *      l;
     struct stat_ui * ui = g_new0( struct stat_ui, 1 );
 
-    d = gtk_dialog_new_with_buttons( _(
-                                         "Statistics" ),
+    d = gtk_dialog_new_with_buttons( _( "Statistics" ),
                                      parent,
                                      GTK_DIALOG_DESTROY_WITH_PARENT |
                                      GTK_DIALOG_NO_SEPARATOR,
-                                     GTK_STOCK_DELETE, TR_RESPONSE_CLEAR,
+                                     _( "_Reset" ), TR_RESPONSE_RESET,
                                      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                      NULL );
     gtk_dialog_set_default_response( GTK_DIALOG( d ),
                                      GTK_RESPONSE_CLOSE );
     gtk_dialog_set_alternative_button_order( GTK_DIALOG( d ),
                                              GTK_RESPONSE_CLOSE,
-                                             TR_RESPONSE_CLEAR,
+                                             TR_RESPONSE_RESET,
                                              -1 );
     t = hig_workarea_create( );
     gtk_box_pack_start( GTK_BOX( GTK_DIALOG( d )->vbox ), t, TRUE, TRUE, 0 );
@@ -169,7 +188,7 @@ stats_dialog_create( GtkWindow * parent,
     updateStats( ui );
     g_object_set_data_full( G_OBJECT( d ), "data", ui, g_free );
     g_signal_connect( d, "response", G_CALLBACK( dialogResponse ), ui );
-    i = gtr_timeout_add_seconds( 1, updateStats, ui );
+    i = gtr_timeout_add_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, updateStats, ui );
     g_object_weak_ref( G_OBJECT( d ), dialogDestroyed, GUINT_TO_POINTER( i ) );
     return d;
 }
