@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.c 8703 2009-06-15 20:29:26Z charles $
+ * $Id: tr-core.c 10108 2010-02-06 05:21:25Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -38,6 +38,7 @@
 #include <libtransmission/rpcimpl.h>
 #include <libtransmission/json.h>
 #include <libtransmission/utils.h> /* tr_free */
+#include <libtransmission/web.h>
 
 #include "conf.h"
 #include "notify.h"
@@ -251,25 +252,30 @@ compareByRatio( GtkTreeModel  * model,
 }
 
 static int
-compareByActivity( GtkTreeModel *           model,
-                   GtkTreeIter *            a,
-                   GtkTreeIter *            b,
+compareByActivity( GtkTreeModel * model,
+                   GtkTreeIter  * a,
+                   GtkTreeIter  * b,
                    gpointer       user_data UNUSED )
 {
-    int            i;
-    tr_torrent *   ta, *tb;
+    int i;
+    tr_torrent *ta, *tb;
     const tr_stat *sa, *sb;
+    double aUp, aDown, bUp, bDown;
 
-    gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
-    gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
+    gtk_tree_model_get( model, a, MC_SPEED_UP, &aUp,
+                                  MC_SPEED_DOWN, &aDown,
+                                  MC_TORRENT_RAW, &ta,
+                                  -1 );
+    gtk_tree_model_get( model, b, MC_SPEED_UP, &bUp,
+                                  MC_SPEED_DOWN, &bDown,
+                                  MC_TORRENT_RAW, &tb,
+                                  -1 );
+
+    if(( i = compareDouble( aUp+aDown, bUp+bDown )))
+        return i;
 
     sa = tr_torrentStatCached( ta );
     sb = tr_torrentStatCached( tb );
-
-    if( ( i = compareDouble( sa->pieceUploadSpeed + sa->pieceDownloadSpeed,
-                             sb->pieceUploadSpeed + sb->pieceDownloadSpeed ) ) )
-        return i;
-
     if( sa->uploadedEver != sb->uploadedEver )
         return sa->uploadedEver < sa->uploadedEver ? -1 : 1;
 
@@ -387,12 +393,25 @@ compareByTracker( GtkTreeModel * model,
                   GtkTreeIter  * b,
                   gpointer       user_data UNUSED )
 {
-    const tr_torrent *ta, *tb;
+    const tr_torrent * ta;
+    const tr_torrent * tb;
+    const tr_info * aInf;
+    const tr_info * bInf;
+    const char * aTracker;
+    const char * bTracker;
 
     gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
     gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
-    return strcmp( tr_torrentInfo( ta )->trackers[0].announce,
-                   tr_torrentInfo( tb )->trackers[0].announce );
+
+    aInf = tr_torrentInfo( ta );
+    bInf = tr_torrentInfo( tb );
+    aTracker = aInf->trackerCount > 0 ? aInf->trackers[0].announce : NULL;
+    bTracker = bInf->trackerCount > 0 ? bInf->trackers[0].announce : NULL;
+
+    if( !aTracker && !bTracker ) return 0;
+    if( !aTracker ) return -1;
+    if( !bTracker ) return 1;
+    return strcmp( aTracker, bTracker );
 }
 
 static void
@@ -413,7 +432,7 @@ setSort( TrCore *     core,
         sort_func = compareByAge;
     else if( !strcmp( mode, "sort-by-progress" ) )
         sort_func = compareByProgress;
-    else if( !strcmp( mode, "sort-by-eta" ) )
+    else if( !strcmp( mode, "sort-by-time-left" ) )
         sort_func = compareByETA;
     else if( !strcmp( mode, "sort-by-ratio" ) )
         sort_func = compareByRatio;
@@ -464,24 +483,92 @@ tr_strcmp( const void * a,
 }
 
 #ifdef HAVE_GIO
-static gboolean
-watchFolderIdle( gpointer gcore )
+
+struct watchdir_file
 {
-    TrCore * core = TR_CORE( gcore );
+    char * filename;
+    time_t mtime;
+};
 
-    core->priv->adding_from_watch_dir = TRUE;
-    tr_core_add_list_defaults( core, core->priv->monitor_files, TRUE );
-    core->priv->adding_from_watch_dir = FALSE;
-
-    /* cleanup */
-    core->priv->monitor_files = NULL;
-    core->priv->monitor_idle_tag = 0;
-    return FALSE;
+static int
+compare_watchdir_file_to_filename( const void * a, const void * filename )
+{
+    return strcmp( ((const struct watchdir_file*)a)->filename, filename );
 }
 
 static void
-maybeAddTorrent( TrCore *     core,
-                 const char * filename )
+watchdir_file_update_mtime( struct watchdir_file * file )
+{
+    GFile * gfile = g_file_new_for_path( file->filename );
+    GFileInfo * info = g_file_query_info( gfile, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL );
+
+    file->mtime = g_file_info_get_attribute_uint64( info, G_FILE_ATTRIBUTE_TIME_MODIFIED );
+
+    g_object_unref( G_OBJECT( info ) );
+    g_object_unref( G_OBJECT( gfile ) );
+}
+
+static struct watchdir_file*
+watchdir_file_new( const char * filename )
+{
+    struct watchdir_file * f;
+
+    f = g_new( struct watchdir_file, 1 );
+    f->filename = g_strdup( filename );
+    watchdir_file_update_mtime( f );
+
+    return f;
+}
+
+static void
+watchdir_file_free( struct watchdir_file * f )
+{
+    g_free( f->filename );
+    g_free( f );
+}
+
+static gboolean
+watchFolderIdle( gpointer gcore )
+{
+    GSList * l;
+    GSList * addme = NULL;
+    GSList * monitor_files = NULL;
+    TrCore * core = TR_CORE( gcore );
+    const time_t now = time( NULL );
+    struct TrCorePrivate * p = core->priv;
+
+    /* of the monitor_files, make a list of those that haven't
+     * changed lately, since they should be ready to add */
+    for( l=p->monitor_files; l!=NULL; l=l->next ) {
+        struct watchdir_file * f = l->data;
+        watchdir_file_update_mtime( f );
+        if( f->mtime + 2 >= now )
+            monitor_files = g_slist_prepend( monitor_files, f );
+        else {
+            addme = g_slist_prepend( addme, g_strdup( f->filename ) );
+            watchdir_file_free( f );
+        }
+    }
+
+    /* add the torrents from that list */
+    core->priv->adding_from_watch_dir = TRUE;
+    tr_core_add_list_defaults( core, addme, TRUE );
+    core->priv->adding_from_watch_dir = FALSE;
+
+    /* update the monitor_files list */
+    g_slist_free( p->monitor_files );
+    p->monitor_files = monitor_files;
+
+    /* if monitor_files is nonempty, keep checking every second */
+    if( core->priv->monitor_files )
+        return TRUE;
+    core->priv->monitor_idle_tag = 0;
+    return FALSE;
+
+}
+
+static void
+maybeAddTorrent( TrCore * core, const char * filename )
 {
     const gboolean isTorrent = g_str_has_suffix( filename, ".torrent" );
 
@@ -489,10 +576,9 @@ maybeAddTorrent( TrCore *     core,
     {
         struct TrCorePrivate * p = core->priv;
 
-        if( !g_slist_find_custom( p->monitor_files, filename,
-                                  (GCompareFunc)strcmp ) )
-            p->monitor_files =
-                g_slist_append( p->monitor_files, g_strdup( filename ) );
+        if( !g_slist_find_custom( p->monitor_files, filename, (GCompareFunc)compare_watchdir_file_to_filename ) )
+            p->monitor_files = g_slist_append( p->monitor_files, watchdir_file_new( filename ) );
+
         if( !p->monitor_idle_tag )
             p->monitor_idle_tag = gtr_timeout_add_seconds( 1, watchFolderIdle, core );
     }
@@ -520,18 +606,21 @@ scanWatchDir( TrCore * core )
 
     if( isEnabled )
     {
-        const char * basename;
         const char * dirname = pref_string_get( PREF_KEY_DIR_WATCH );
         GDir * dir = g_dir_open( dirname, 0, NULL );
 
-        while(( basename = g_dir_read_name( dir )))
+        if( dir != NULL )
         {
-            char * filename = g_build_filename( dirname, basename, NULL );
-            maybeAddTorrent( core, filename );
-            g_free( filename );
-        }
+            const char * basename;
+            while(( basename = g_dir_read_name( dir )))
+            {
+                char * filename = g_build_filename( dirname, basename, NULL );
+                maybeAddTorrent( core, filename );
+                g_free( filename );
+            }
 
-        g_dir_close( dir );
+            g_dir_close( dir );
+        }
     }
 }
 
@@ -608,26 +697,26 @@ static void
 tr_core_init( GTypeInstance *  instance,
               gpointer g_class UNUSED )
 {
-    TrCore *               self = (TrCore *) instance;
-    GtkListStore *         store;
+    GtkListStore * store;
     struct TrCorePrivate * p;
+    TrCore * self = (TrCore *) instance;
 
     /* column types for the model used to store torrent information */
     /* keep this in sync with the enum near the bottom of tr_core.h */
-    GType                  types[] = {
-        G_TYPE_STRING,    /* name */
-        G_TYPE_STRING,    /* collated name */
-        TR_TORRENT_TYPE,  /* TrTorrent object */
-        G_TYPE_POINTER,   /* tr_torrent* */
-        G_TYPE_INT        /* tr_stat()->status */
-    };
+    GType types[] = { G_TYPE_STRING,    /* name */
+                      G_TYPE_STRING,    /* collated name */
+                      TR_TORRENT_TYPE,  /* TrTorrent object */
+                      G_TYPE_POINTER,   /* tr_torrent* */
+                      G_TYPE_DOUBLE,    /* tr_stat.pieceUploadSpeed */
+                      G_TYPE_DOUBLE,    /* tr_stat.pieceDownloadSpeed */
+                      G_TYPE_INT };     /* tr_stat.status */
 
     p = self->priv = G_TYPE_INSTANCE_GET_PRIVATE( self,
                                                   TR_CORE_TYPE,
                                                   struct TrCorePrivate );
 
     /* create the model used to store torrent data */
-    g_assert( ALEN( types ) == MC_ROW_COUNT );
+    g_assert( G_N_ELEMENTS( types ) == MC_ROW_COUNT );
     store = gtk_list_store_newv( MC_ROW_COUNT, types );
 
     p->model    = GTK_TREE_MODEL( store );
@@ -718,51 +807,12 @@ tr_core_session( TrCore * core )
     return isDisposed( core ) ? NULL : core->priv->session;
 }
 
-static gboolean
-statsForeach( GtkTreeModel * model,
-              GtkTreePath  * path UNUSED,
-              GtkTreeIter  * iter,
-              gpointer       gstats )
-{
-    tr_torrent *        tor;
-    struct core_stats * stats = gstats;
-    int                 activity;
-
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
-    activity = tr_torrentGetActivity( tor );
-
-    if( activity == TR_STATUS_DOWNLOAD )
-        ++stats->downloadCount;
-    else if( activity == TR_STATUS_SEED )
-        ++stats->seedingCount;
-
-    return FALSE;
-}
-
-void
-tr_core_get_stats( const TrCore *      core,
-                   struct core_stats * setme )
-{
-    memset( setme, 0, sizeof( struct core_stats ) );
-
-    if( !isDisposed( core ) )
-    {
-        tr_session * session = core->priv->session;
-
-        setme->clientDownloadSpeed = tr_sessionGetPieceSpeed( session, TR_DOWN );
-
-        setme->clientUploadSpeed = tr_sessionGetPieceSpeed( session, TR_UP );
-
-        gtk_tree_model_foreach( core->priv->model, statsForeach, setme );
-    }
-}
-
 static char*
 doCollate( const char * in )
 {
-    const char * end = in + strlen( in );
-    char *       casefold;
-    char *       ret;
+    char * ret;
+    char * casefold;
+    const char * end = in ? in + strlen( in ) : NULL;
 
     while( in < end )
     {
@@ -790,7 +840,7 @@ tr_core_add_torrent( TrCore     * self,
                      gboolean     doNotify )
 {
     const tr_info * inf = tr_torrent_info( gtor );
-    const tr_stat * torStat = tr_torrent_stat( gtor );
+    const tr_stat * st = tr_torrent_stat( gtor );
     tr_torrent *    tor = tr_torrent_handle( gtor );
     char *          collated = doCollate( inf->name );
     GtkListStore *  store = GTK_LIST_STORE( tr_core_model( self ) );
@@ -801,7 +851,9 @@ tr_core_add_torrent( TrCore     * self,
                                        MC_NAME_COLLATED, collated,
                                        MC_TORRENT,       gtor,
                                        MC_TORRENT_RAW,   tor,
-                                       MC_ACTIVITY,      torStat->activity,
+                                       MC_SPEED_UP,      st->pieceUploadSpeed,
+                                       MC_SPEED_DOWN,    st->pieceDownloadSpeed,
+                                       MC_ACTIVITY,      st->activity,
                                        -1 );
 
     if( doNotify )
@@ -865,11 +917,10 @@ add_ctor( TrCore * core, tr_ctor * ctor, gboolean doPrompt, gboolean doNotify )
 
     switch( err )
     {
-        case TR_EINVALID:
+        case TR_PARSE_ERR:
             break;
 
-        case TR_EDUPLICATE:
-            g_message( "it's a duplicate" );
+        case TR_PARSE_DUPLICATE:
             /* don't complain about .torrent files in the watch directory
              * that have already been added... that gets annoying and we
              * don't want to be nagging users to clean up their watch dirs */
@@ -894,22 +945,35 @@ add_ctor( TrCore * core, tr_ctor * ctor, gboolean doPrompt, gboolean doNotify )
     return err;
 }
 
+void
+tr_core_add_ctor( TrCore * core, tr_ctor * ctor )
+{
+    const gboolean doStart = pref_flag_get( PREF_KEY_START );
+    const gboolean doPrompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
+    tr_core_apply_defaults( ctor );
+    add_ctor( core, ctor, doStart, doPrompt );
+}
+
 /* invoked remotely via dbus. */
 gboolean
 tr_core_add_metainfo( TrCore      * core,
-                      const char  * base64_metainfo,
-                      gboolean    * setme_success,
+                      const char  * payload,
+                      gboolean    * setme_handled,
                       GError     ** gerr UNUSED )
 {
     tr_session * session = tr_core_session( core );
 
     if( !session )
     {
-        *setme_success = FALSE;
+        *setme_handled = FALSE;
     }
-    else
+    else if( gtr_is_supported_url( payload ) || gtr_is_magnet_link( payload ) )
+    {
+        tr_core_add_from_url( core, payload );
+        *setme_handled = TRUE;
+    }
+    else /* base64-encoded metainfo */
     {
-        int err;
         int file_length;
         tr_ctor * ctor;
         char * file_contents;
@@ -918,20 +982,108 @@ tr_core_add_metainfo( TrCore      * core,
         ctor = tr_ctorNew( session );
         tr_core_apply_defaults( ctor );
 
-        file_contents = tr_base64_decode( base64_metainfo, -1, &file_length );
-        err = tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
-
-        if( !err )
-            err = add_ctor( core, ctor, do_prompt, TRUE );
+        file_contents = tr_base64_decode( payload, -1, &file_length );
+        tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
+        add_ctor( core, ctor, do_prompt, TRUE );
 
         tr_free( file_contents );
         tr_core_torrents_added( core );
-        *setme_success = TRUE;
+        *setme_handled = TRUE;
     }
 
     return TRUE;
 }
 
+/***
+****
+***/
+
+struct url_dialog_data
+{
+    TrCore * core;
+    tr_ctor * ctor;
+    GtkDialog * dialog;
+};
+
+static gboolean
+onURLDoneIdle( gpointer vdata )
+{
+    struct url_dialog_data * data = vdata;
+    tr_core_add_ctor( data->core, data->ctor );
+    g_free( data );
+    return FALSE;
+}
+
+static void
+onURLDone( tr_session       * session,
+           long               response_code UNUSED,
+           const void       * response,
+           size_t             response_byte_count,
+           void             * vdata )
+{
+    struct url_dialog_data * data = vdata;
+    tr_ctor * ctor = tr_ctorNew( session );
+
+    /* FIME: error dialog */
+
+    if( tr_ctorSetMetainfo( ctor, response, response_byte_count ) )
+    {
+        tr_ctorFree( ctor );
+        g_free( data );
+    }
+    else /* move the work back to the gtk thread */
+    {
+        data->ctor = ctor;
+        gtr_idle_add( onURLDoneIdle, data );
+    }
+}
+
+void
+tr_core_add_from_url( TrCore * core, const char * url )
+{
+    tr_session * session = tr_core_session( core );
+    const gboolean is_magnet_link = gtr_is_magnet_link( url );
+
+    if( is_magnet_link || gtr_is_hex_hashcode( url ) )
+    {
+        int err;
+        char * tmp = NULL;
+        tr_ctor * ctor = tr_ctorNew( session );
+
+        if( gtr_is_hex_hashcode( url ) )
+            url = tmp = g_strdup_printf( "magnet:?xt=urn:btih:%s", url );
+
+        err = tr_ctorSetMetainfoFromMagnetLink( ctor, url );
+
+        if( err )
+        {
+            gtr_unrecognized_url_dialog( NULL, url );
+            tr_ctorFree( ctor );
+        }
+        else
+        {
+            tr_session * session = tr_core_session( core );
+            TrTorrent * gtor = tr_torrent_new_ctor( session, ctor, &err );
+            if( !err )
+                tr_core_add_torrent( core, gtor, FALSE );
+            else
+                g_message( "tr_torrent_new_ctor err %d", err );
+        }
+
+        g_free( tmp );
+    }
+    else
+    {
+        struct url_dialog_data * data = g_new( struct url_dialog_data, 1 );
+        data->core = core;
+        tr_webRun( session, url, NULL, onURLDone, data );
+    }
+}
+
+/***
+****
+***/
+
 static void
 add_filename( TrCore      * core,
               const char  * filename,
@@ -940,17 +1092,30 @@ add_filename( TrCore      * core,
               gboolean      doNotify )
 {
     tr_session * session = tr_core_session( core );
-    if( filename && session )
+
+    if( session == NULL )
+        return;
+
+    if( gtr_is_supported_url( filename ) || gtr_is_magnet_link( filename ) )
+    {
+        tr_core_add_from_url( core, filename );
+    }
+    else if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
     {
         int err;
+
         tr_ctor * ctor = tr_ctorNew( session );
+        tr_ctorSetMetainfoFromFile( ctor, filename );
         tr_core_apply_defaults( ctor );
         tr_ctorSetPaused( ctor, TR_FORCE, !doStart );
-        tr_ctorSetMetainfoFromFile( ctor, filename );
 
         err = add_ctor( core, ctor, doPrompt, doNotify );
-        if( err == TR_EINVALID )
-            tr_core_errsig( core, TR_EINVALID, filename );
+        if( err == TR_PARSE_ERR )
+            tr_core_errsig( core, TR_PARSE_ERR, filename );
+    }
+    else if( gtr_is_hex_hashcode( filename ) )
+    {
+        tr_core_add_from_url( core, filename );
     }
 }
 
@@ -1063,25 +1228,43 @@ tr_core_remove_torrent( TrCore *    core,
 ***/
 
 static gboolean
-update_foreach( GtkTreeModel *      model,
+update_foreach( GtkTreeModel * model,
                 GtkTreePath  * path UNUSED,
-                GtkTreeIter *       iter,
+                GtkTreeIter  * iter,
                 gpointer       data UNUSED )
 {
-    int         oldActivity;
-    int         newActivity;
+    int oldActivity, newActivity;
+    double oldUpSpeed, newUpSpeed;
+    double oldDownSpeed, newDownSpeed;
+    const tr_stat * st;
     TrTorrent * gtor;
 
-    /* maybe update the status column in the model */
+    /* get the old states */
     gtk_tree_model_get( model, iter,
                         MC_TORRENT, &gtor,
                         MC_ACTIVITY, &oldActivity,
+                        MC_SPEED_UP, &oldUpSpeed,
+                        MC_SPEED_DOWN, &oldDownSpeed,
                         -1 );
-    newActivity = tr_torrentGetActivity( tr_torrent_handle( gtor ) );
-    if( newActivity != oldActivity )
+
+    /* get the new states */
+    st = tr_torrentStat( tr_torrent_handle( gtor ) );
+    newActivity = st->activity;
+    newUpSpeed = st->pieceUploadSpeed;
+    newDownSpeed = st->pieceDownloadSpeed;
+
+    /* updating the model triggers off resort/refresh,
+       so don't do it unless something's actually changed... */
+    if( ( newActivity != oldActivity ) ||
+        ( (int)(newUpSpeed*10.0) != (int)(oldUpSpeed*10.0) ) ||
+        ( (int)(newDownSpeed*10.0) != (int)(oldDownSpeed*10.0) ) )
+    {
         gtk_list_store_set( GTK_LIST_STORE( model ), iter,
                             MC_ACTIVITY, newActivity,
+                            MC_SPEED_UP, newUpSpeed,
+                            MC_SPEED_DOWN, newDownSpeed,
                             -1 );
+    }
 
     /* cleanup */
     g_object_unref( gtor );
@@ -1127,8 +1310,11 @@ tr_core_quit( TrCore * core )
 static DBusGProxy*
 get_hibernation_inhibit_proxy( void )
 {
-    GError *          error = NULL;
     DBusGConnection * conn;
+    GError * error = NULL;
+    const char * name = "org.gnome.SessionManager";
+    const char * path = "/org/gnome/SessionManager";
+    const char * interface = "org.gnome.SessionManager";
 
     conn = dbus_g_bus_get( DBUS_BUS_SESSION, &error );
     if( error )
@@ -1138,11 +1324,7 @@ get_hibernation_inhibit_proxy( void )
         return NULL;
     }
 
-    return dbus_g_proxy_new_for_name (
-               conn,
-               "org.freedesktop.PowerManagement",
-               "/org/freedesktop/PowerManagement/Inhibit",
-               "org.freedesktop.PowerManagement.Inhibit" );
+    return dbus_g_proxy_new_for_name ( conn, name, path, interface );
 }
 
 static gboolean
@@ -1153,21 +1335,25 @@ gtr_inhibit_hibernation( guint * cookie )
 
     if( proxy )
     {
-        GError *     error = NULL;
+        GError * error = NULL;
+        const int toplevel_xid = 0;
         const char * application = _( "Transmission Bittorrent Client" );
         const char * reason = _( "BitTorrent Activity" );
+        const int flags = 4; /* Inhibit suspending the session or computer */
+
         success = dbus_g_proxy_call( proxy, "Inhibit", &error,
                                      G_TYPE_STRING, application,
+                                     G_TYPE_UINT, toplevel_xid,
                                      G_TYPE_STRING, reason,
-                                     G_TYPE_INVALID,
+                                     G_TYPE_UINT, flags,
+                                     G_TYPE_INVALID, /* sentinel - end of input args */
                                      G_TYPE_UINT, cookie,
-                                     G_TYPE_INVALID );
+                                     G_TYPE_INVALID /* senitnel - end of output args */ );
         if( success )
-            tr_inf( _( "Disallowing desktop hibernation" ) );
+            tr_inf( "%s", _( "Disallowing desktop hibernation" ) );
         else
         {
-            tr_err( _(
-                        "Couldn't disable desktop hibernation: %s" ),
+            tr_err( _( "Couldn't disable desktop hibernation: %s" ),
                     error->message );
             g_error_free( error );
         }
@@ -1186,12 +1372,12 @@ gtr_uninhibit_hibernation( guint inhibit_cookie )
     if( proxy )
     {
         GError * error = NULL;
-        gboolean success = dbus_g_proxy_call( proxy, "UnInhibit", &error,
+        gboolean success = dbus_g_proxy_call( proxy, "Uninhibit", &error,
                                               G_TYPE_UINT, inhibit_cookie,
                                               G_TYPE_INVALID,
                                               G_TYPE_INVALID );
         if( success )
-            tr_inf( _( "Allowing desktop hibernation" ) );
+            tr_inf( "%s", _( "Allowing desktop hibernation" ) );
         else
         {
             g_warning( "Couldn't uninhibit the system from suspending: %s.",
@@ -1373,7 +1559,7 @@ readResponse( tr_session  * session UNUSED,
     g_message( "response: [%*.*s]", (int)response_len, (int)response_len, response );
 #endif
     g_byte_array_append( bytes, (const uint8_t*)response, response_len );
-    g_idle_add( readResponseIdle, bytes );
+    gtr_idle_add( readResponseIdle, bytes );
 }
 
 static void
