@@ -1,33 +1,24 @@
-/******************************************************************************
- * $Id: util.c 8669 2009-06-11 14:51:21Z charles $
+/*
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: util.c 10101 2010-02-05 13:59:44Z charles $
+ */
 
 #include <ctype.h> /* isxdigit() */
 #include <errno.h>
-#include <stdarg.h>
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcmp() */
 
+#include <sys/types.h> /* for gtr_lockfile()'s open() */
+#include <sys/stat.h> /* for gtr_lockfile()'s open() */
+#include <fcntl.h> /* for gtr_lockfile()'s open() */
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h> /* g_unlink() */
@@ -49,6 +40,55 @@
 #include "tr-prefs.h"
 #include "util.h"
 
+gtr_lockfile_state_t
+gtr_lockfile( const char * filename )
+{
+    gtr_lockfile_state_t ret;
+
+#ifdef WIN32
+
+    HANDLE file = CreateFile( filename,
+                              GENERIC_READ | GENERIC_WRITE,
+                              FILE_SHARE_READ | FILE_SHARE_WRITE,
+                              NULL,
+                              OPEN_ALWAYS,
+                              FILE_ATTRIBUTE_NORMAL,
+                              NULL );
+    if( file == INVALID_HANDLE_VALUE )
+        ret = GTR_LOCKFILE_EOPEN;
+    else if( !LockFile( file, 0, 0, 1, 1 ) )
+        ret = GTR_LOCKFILE_ELOCK;
+    else
+        ret = GTR_LOCKFILE_SUCCESS;
+
+#else
+
+    int fd = open( filename, O_RDWR | O_CREAT, 0666 );
+    if( fd < 0 )
+        ret = GTR_LOCKFILE_EOPEN;
+    else {
+        struct flock lk;
+        memset( &lk, 0,  sizeof( lk ) );
+        lk.l_start = 0;
+        lk.l_len = 0;
+        lk.l_type = F_WRLCK;
+        lk.l_whence = SEEK_SET;
+        if( -1 == fcntl( fd, F_SETLK, &lk ) )
+            ret = GTR_LOCKFILE_ELOCK;
+        else
+            ret = GTR_LOCKFILE_SUCCESS;
+    }
+
+#endif
+
+    return ret;
+}
+
+/***
+****
+***/
+
+
 char*
 tr_strlratio( char * buf, double ratio, size_t buflen )
 {
@@ -209,8 +249,7 @@ gtr_localtime2( char * buf, time_t time, size_t buflen )
 }
 
 int
-mkdir_p( const char * path,
-         mode_t       mode )
+gtr_mkdir_with_parents( const char * path, int mode )
 {
 #if GLIB_CHECK_VERSION( 2, 8, 0 )
     return !g_mkdir_with_parents( path, mode );
@@ -282,82 +321,46 @@ decode_uri( const char * uri )
     return ret;
 }
 
-GSList *
-checkfilenames( int    argc,
-                char **argv )
+
+gboolean
+gtr_is_supported_url( const char * str )
 {
-    int      i;
-    GSList * ret = NULL;
-    char *   pwd = g_get_current_dir( );
-
-    for( i = 0; i < argc; ++i )
-    {
-        char * filename = g_path_is_absolute( argv[i] )
-                          ? g_strdup ( argv[i] )
-                          : g_build_filename( pwd, argv[i], NULL );
-
-        if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
-            ret = g_slist_prepend( ret, filename );
-        else
-            g_free( filename );
-    }
-
-    g_free( pwd );
-    return g_slist_reverse( ret );
+    return !strncmp( str, "ftp://", 6 )
+        || !strncmp( str, "http://", 7 )
+        || !strncmp( str, "https://", 8 );
 }
 
-static void
-onErrorResponse( GtkWidget * dialog,
-                 int resp    UNUSED,
-                 gpointer    glist )
+gboolean
+gtr_is_magnet_link( const char * str )
 {
-    GSList * list = glist;
-
-    if( list )
-    {
-        callbackfunc_t func = list->data;
-        gpointer       user_data = list->next->data;
-        func( user_data );
-        g_slist_free( list );
-    }
-
-    gtk_widget_destroy( dialog );
+    return !strncmp( str, "magnet:?", 8 );
 }
 
-static GtkWidget *
-verrmsg_full( GtkWindow *    wind,
-              callbackfunc_t func,
-              void *         data,
-              const char *   format,
-              va_list        ap )
+gboolean
+gtr_is_hex_hashcode( const char * str )
 {
-    GtkWidget *dialog;
-    char *     msg;
-    GSList *   funcdata = NULL;
+    int i;
 
-    msg = g_strdup_vprintf( format, ap );
+    if( !str || ( strlen( str ) != 40 ) )
+        return FALSE;
 
-    if( NULL == wind )
-        dialog = gtk_message_dialog_new(
-            NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", msg );
-    else
-        dialog = gtk_message_dialog_new(
-            wind,
-            GTK_DIALOG_MODAL |
-            GTK_DIALOG_DESTROY_WITH_PARENT,
-            GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-            "%s", msg );
+    for( i=0; i<40; ++i )
+        if( !isxdigit( str[i] ) )
+            return FALSE;
 
-    if( func )
-    {
-        funcdata = g_slist_append( funcdata, (gpointer)func );
-        funcdata = g_slist_append( funcdata, data );
-    }
-    g_signal_connect( dialog, "response", G_CALLBACK(
-                          onErrorResponse ), funcdata );
-    g_free( msg );
+    return TRUE;
+}
 
-    return dialog;
+static GtkWindow *
+getWindow( GtkWidget * w )
+{
+    if( w == NULL )
+        return NULL;
+
+    if( GTK_IS_WINDOW( w ) )
+        return GTK_WINDOW( w );
+
+    return GTK_WINDOW( gtk_widget_get_ancestor( w, GTK_TYPE_WINDOW ) );
 }
 
 void
@@ -365,35 +368,24 @@ addTorrentErrorDialog( GtkWidget *  child,
                        int          err,
                        const char * filename )
 {
-    GtkWidget *  w;
-    GtkWidget *  win;
+    char * secondary;
     const char * fmt;
-    char *       secondary;
+    GtkWidget * w;
+    GtkWindow * win = getWindow( child );
 
     switch( err )
     {
-        case TR_EINVALID:
-            fmt = _( "The torrent file \"%s\" contains invalid data." );
-            break;
-
-        case TR_EDUPLICATE:
-            fmt = _( "The torrent file \"%s\" is already in use." ); break;
-
-        default:
-            fmt = _(
-                "The torrent file \"%s\" encountered an unknown error." );
-            break;
+        case TR_PARSE_ERR: fmt = _( "The torrent file \"%s\" contains invalid data." ); break;
+        case TR_PARSE_DUPLICATE: fmt = _( "The torrent file \"%s\" is already in use." ); break;
+        default: fmt = _( "The torrent file \"%s\" encountered an unknown error." ); break;
     }
     secondary = g_strdup_printf( fmt, filename );
-    win = ( !child || GTK_IS_WINDOW( child ) )
-          ? child
-          : gtk_widget_get_ancestor( child ? GTK_WIDGET(
-                                         child ) : NULL, GTK_TYPE_WINDOW );
-    w = gtk_message_dialog_new( GTK_WINDOW( win ),
-                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                               GTK_MESSAGE_ERROR,
-                               GTK_BUTTONS_CLOSE,
-                               _( "Error opening torrent" ) );
+
+    w = gtk_message_dialog_new( win,
+                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                GTK_MESSAGE_ERROR,
+                                GTK_BUTTONS_CLOSE,
+                                "%s", _( "Error opening torrent" ) );
     gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ),
                                               "%s", secondary );
     g_signal_connect_swapped( w, "response",
@@ -402,46 +394,6 @@ addTorrentErrorDialog( GtkWidget *  child,
     g_free( secondary );
 }
 
-void
-errmsg( GtkWindow *  wind,
-        const char * format,
-        ... )
-{
-    GtkWidget * dialog;
-    va_list     ap;
-
-    va_start( ap, format );
-    dialog = verrmsg_full( wind, NULL, NULL, format, ap );
-    va_end( ap );
-
-    if( NULL != wind && !GTK_WIDGET_MAPPED( GTK_WIDGET( wind ) ) )
-    {
-        g_signal_connect_swapped( wind, "map",
-                                  G_CALLBACK( gtk_widget_show ), dialog );
-    }
-    else
-    {
-        gtk_widget_show( dialog );
-    }
-}
-
-GtkWidget *
-errmsg_full( GtkWindow *    wind,
-             callbackfunc_t func,
-             void *         data,
-             const char *   format,
-             ... )
-{
-    GtkWidget * dialog;
-    va_list     ap;
-
-    va_start( ap, format );
-    dialog = verrmsg_full( wind, func, data, format, ap );
-    va_end( ap );
-
-    return dialog;
-}
-
 typedef void ( PopupFunc )( GtkWidget*, GdkEventButton* );
 
 /* pop up the context menu if a user right-clicks.
@@ -471,7 +423,8 @@ on_tree_view_button_pressed( GtkWidget *      view,
             gtk_tree_path_free( path );
         }
 
-        ( (PopupFunc*)func )( view, event );
+        if( func != NULL )
+            ( (PopupFunc*)func )( view, event );
 
         return TRUE;
     }
@@ -495,7 +448,6 @@ on_tree_view_button_released( GtkWidget *      view,
     {
         GtkTreeSelection * selection = gtk_tree_view_get_selection( tv );
         gtk_tree_selection_unselect_all ( selection );
-        return TRUE;
     }
 
     return FALSE;
@@ -568,8 +520,13 @@ gtr_open_file( const char * path )
         if( !opened )
         {
             char * argv[] = { (char*)"xdg-open", (char*)path, NULL };
-            g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-                           NULL, NULL, NULL, NULL );
+            opened = g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+                                    NULL, NULL, NULL, NULL );
+        }
+
+        if( !opened )
+        {
+            g_message( "Unable to open \"%s\"", path );
         }
     }
 }
@@ -582,28 +539,40 @@ gboolean
 gtr_dbus_add_torrent( const char * filename )
 {
     /* FIXME: why is this static? */
-    static gboolean success = FALSE;
-#ifdef HAVE_DBUS_GLIB
-    DBusGProxy * proxy = NULL;
-    GError * err = NULL;
-    DBusGConnection * conn;
-    char * file_contents;
-    gsize file_length;
+    static gboolean handled = FALSE;
 
+#ifdef HAVE_DBUS_GLIB
+    char * payload;
+    gsize file_length;
+    char * file_contents = NULL;
+
+    /* If it's a file, load its contents and send them over the wire...
+     * it might be a temporary file that's going to disappear. */
     if( g_file_get_contents( filename, &file_contents, &file_length, NULL ) )
+        payload = tr_base64_encode( file_contents, file_length, NULL );
+    else if( gtr_is_supported_url( filename ) || gtr_is_magnet_link( filename ) )
+        payload = tr_strdup( filename );
+    else
+        payload = NULL;
+
+    if( payload != NULL )
     {
-        char * b64 = tr_base64_encode( file_contents, file_length, NULL );
+        GError * err = NULL;
+        DBusGConnection * conn;
+        DBusGProxy * proxy = NULL;
+
         if(( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err )))
             proxy = dbus_g_proxy_new_for_name (conn, VALUE_SERVICE_NAME,
                                                      VALUE_SERVICE_OBJECT_PATH,
                                                      VALUE_SERVICE_INTERFACE );
         else if( err )
            g_message( "err: %s", err->message );
+
         if( proxy )
             dbus_g_proxy_call( proxy, "AddMetainfo", &err,
-                               G_TYPE_STRING, b64,
+                               G_TYPE_STRING, payload,
                                G_TYPE_INVALID,
-                               G_TYPE_BOOLEAN, &success,
+                               G_TYPE_BOOLEAN, &handled,
                                G_TYPE_INVALID );
         if( err )
            g_message( "err: %s", err->message );
@@ -613,13 +582,13 @@ gtr_dbus_add_torrent( const char * filename )
         if( conn )
             dbus_g_connection_unref( conn );
 
-        tr_free( b64 );
-        g_free( file_contents );
+        tr_free( payload );
     }
-    else g_message( "couldn't read %s", filename );
+
+    g_free( file_contents );
 
 #endif
-    return success;
+    return handled;
 }
 
 gboolean
@@ -667,15 +636,85 @@ gtr_button_new_from_stock( const char * stock,
 ****
 ***/
 
-guint
-gtr_timeout_add_seconds( guint seconds, GSourceFunc function, gpointer data )
+void
+gtr_priority_combo_set_value( GtkWidget * w, tr_priority_t value )
 {
-#if GLIB_CHECK_VERSION( 2,14,0 )
-    return g_timeout_add_seconds( seconds, function, data );
-#else
-    return g_timeout_add( seconds*1000, function, data );
-#endif
+    int i;
+    int currentValue;
+    const int column = 0;
+    GtkTreeIter iter;
+    GtkComboBox * combobox = GTK_COMBO_BOX( w );
+    GtkTreeModel * model = gtk_combo_box_get_model( combobox );
+
+    /* do the value and current value match? */
+    if( gtk_combo_box_get_active_iter( combobox, &iter ) ) {
+        gtk_tree_model_get( model, &iter, column, &currentValue, -1 );
+        if( currentValue == value )
+            return;
+    }
+
+    /* find the one to select */
+    i = 0;
+    while(( gtk_tree_model_iter_nth_child( model, &iter, NULL, i++ ))) {
+        gtk_tree_model_get( model, &iter, column, &currentValue, -1 );
+        if( currentValue == value ) {
+            gtk_combo_box_set_active_iter( combobox, &iter );
+            return;
+        }
+    }
 }
+ 
+tr_priority_t
+gtr_priority_combo_get_value( GtkWidget * w )
+{
+    int value = 0;
+    GtkTreeIter iter;
+    GtkComboBox * combo_box = GTK_COMBO_BOX( w );
+
+    if( gtk_combo_box_get_active_iter( combo_box, &iter ) )
+        gtk_tree_model_get( gtk_combo_box_get_model( combo_box ), &iter, 0, &value, -1 );
+
+    return value;
+}
+
+GtkWidget *
+gtr_priority_combo_new( void )
+{
+    int i;
+    GtkWidget * w;
+    GtkCellRenderer * r;
+    GtkListStore * store;
+    const struct {
+        int value;
+        const char * text;
+    } items[] = {
+        { TR_PRI_HIGH,   N_( "High" )  },
+        { TR_PRI_NORMAL, N_( "Normal" ) },
+        { TR_PRI_LOW,    N_( "Low" )  }
+    };
+
+    store = gtk_list_store_new( 2, G_TYPE_INT, G_TYPE_STRING );
+    for( i=0; i<(int)G_N_ELEMENTS(items); ++i ) {
+        GtkTreeIter iter;
+        gtk_list_store_append( store, &iter );
+        gtk_list_store_set( store, &iter, 0, items[i].value,
+                                          1, _( items[i].text ),
+                                         -1 );
+    }
+
+    w = gtk_combo_box_new_with_model( GTK_TREE_MODEL( store ) );
+    r = gtk_cell_renderer_text_new( );
+    gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( w ), r, TRUE );
+    gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( w ), r, "text", 1, NULL );
+
+    /* cleanup */
+    g_object_unref( store );
+    return w;
+}
+
+/***
+****
+***/
 
 void
 gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip )
@@ -689,3 +728,119 @@ gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip )
     gtk_tooltips_set_tip( tips, w, tip, NULL );
 #endif
 }
+
+void
+gtr_toolbar_set_orientation( GtkToolbar      * toolbar,
+                             GtkOrientation    orientation )
+{
+#if GTK_CHECK_VERSION( 2,16,0 )
+    gtk_orientable_set_orientation( GTK_ORIENTABLE( toolbar ), orientation );
+#else
+    gtk_toolbar_set_orientation( toolbar, orientation );
+#endif
+}
+
+/***
+****
+***/
+
+#if !GTK_CHECK_VERSION( 2,12,0 )
+struct gtr_func_data
+{
+    GSourceFunc function;
+    gpointer data;
+};
+
+static void
+gtr_func_data_free( gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,10,0 )
+    g_slice_free( struct gtr_func_data, data );
+#else
+    g_free( data );
+#endif
+}
+
+static struct gtr_func_data *
+gtr_func_data_new( GSourceFunc function, gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,10,0 )
+    struct gtr_func_data * d = g_slice_new( struct gtr_func_data );
+#else
+    struct gtr_func_data * d = g_new( struct gtr_func_data, 1 );
+#endif
+    d->function = function;
+    d->data = data;
+    return d;
+}
+
+static gboolean
+gtr_thread_func( gpointer data )
+{
+    gboolean more;
+    struct gtr_func_data * idle_data = data;
+
+    gdk_threads_enter( );
+    more = idle_data->function( idle_data->data );
+    gdk_threads_leave( );
+
+    return more;
+}
+#endif
+
+void
+gtr_idle_add( GSourceFunc function, gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,12,0 )
+    gdk_threads_add_idle( function, data );
+#else
+    g_idle_add_full( G_PRIORITY_DEFAULT,
+                     gtr_thread_func,
+                     gtr_func_data_new( function, data ),
+                     gtr_func_data_free );
+#endif
+}
+
+guint
+gtr_timeout_add_seconds( guint seconds, GSourceFunc function, gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,14,0 )
+    return gdk_threads_add_timeout_seconds( seconds, function, data );
+#elif GTK_CHECK_VERSION( 2,12,0 )
+    return gdk_threads_add_timeout( seconds*1000, function, data );
+#else
+    return g_timeout_add_full( G_PRIORITY_DEFAULT,
+                               seconds * 1000,
+                               gtr_thread_func,
+                               gtr_func_data_new( function, data ),
+                               gtr_func_data_free );
+#endif
+}
+
+void
+gtr_unrecognized_url_dialog( GtkWidget * parent, const char * url )
+{
+    const char * xt = "xt=urn:btih";
+
+    GtkWindow * window = getWindow( parent );
+
+    GString * gstr = g_string_new( NULL );
+
+    GtkWidget * w = gtk_message_dialog_new( window, 0,
+                                            GTK_MESSAGE_ERROR,
+                                            GTK_BUTTONS_CLOSE,
+                                            "%s", _( "Unrecognized URL" ) );
+
+    g_string_append_printf( gstr, _( "Transmission doesn't know how to use \"%s\"" ), url );
+
+    if( gtr_is_magnet_link( url ) && ( strstr( url, xt ) == NULL ) )
+    {
+        g_string_append_printf( gstr, "\n \n" );
+        g_string_append_printf( gstr, _( "This magnet link appears to be intended for something other than BitTorrent.  BitTorrent magnet links have a section containing \"%s\"." ), xt ); 
+    }
+
+    gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ), "%s", gstr->str );
+    g_signal_connect_swapped( w, "response", G_CALLBACK( gtk_widget_destroy ), w );
+    gtk_widget_show( w );
+    g_string_free( gstr, TRUE );
+}
