@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,19 +7,21 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 8646 2009-06-07 17:54:28Z charles $
+ * $Id: bencode.c 10342 2010-03-09 22:59:13Z charles $
  */
 
 #include <assert.h>
-#include <ctype.h> /* isdigit, isprint, isspace */
+#include <ctype.h> /* isdigit */
 #include <errno.h>
 #include <math.h> /* fabs */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <sys/types.h> /* stat() */
+#include <sys/stat.h> /* stat() */
 #include <locale.h>
-#include <unistd.h> /* close() */
+#include <unistd.h> /* stat(), close() */
 
 #include <event.h> /* evbuffer */
 
@@ -83,7 +85,6 @@ tr_bencParseInt( const uint8_t *  buf,
                  const uint8_t ** setme_end,
                  int64_t *        setme_val )
 {
-    int          err = 0;
     char *       endptr;
     const void * begin;
     const void * end;
@@ -102,16 +103,13 @@ tr_bencParseInt( const uint8_t *  buf,
     errno = 0;
     val = evutil_strtoll( begin, &endptr, 10 );
     if( errno || ( endptr != end ) ) /* incomplete parse */
-        err = EILSEQ;
-    else if( val && *(const char*)begin == '0' ) /* no leading zeroes! */
-        err = EILSEQ;
-    else
-    {
-        *setme_end = (const uint8_t*)end + 1;
-        *setme_val = val;
-    }
+        return EILSEQ;
+    if( val && *(const char*)begin == '0' ) /* no leading zeroes! */
+        return EILSEQ;
 
-    return err;
+    *setme_end = (const uint8_t*)end + 1;
+    *setme_val = val;
+    return 0;
 }
 
 /**
@@ -357,23 +355,16 @@ tr_bencLoad( const void * buf_in,
 ****
 ***/
 
-/* returns true if the given string length would fit in our string buffer */
-static TR_INLINE int
-stringFitsInBuffer( const tr_benc * val, int len )
-{
-    return len < (int)sizeof( val->val.s.str.buf );
-}
-
 /* returns true if the benc's string was malloced.
  * this occurs when the string is too long for our string buffer */
-static TR_INLINE int
+static inline int
 stringIsAlloced( const tr_benc * val )
 {
-    return !stringFitsInBuffer( val, val->val.s.len );
+    return val->val.s.len >= sizeof( val->val.s.str.buf );
 }
 
 /* returns a const pointer to the benc's string */
-static TR_INLINE const char*
+static inline const char*
 getStr( const tr_benc* val )
 {
     return stringIsAlloced(val) ? val->val.s.str.ptr : val->val.s.str.buf;
@@ -579,29 +570,34 @@ tr_bencDictFindRaw( tr_benc         * dict,
 void
 tr_bencInitRaw( tr_benc * val, const void * src, size_t byteCount )
 {
+    char * setme;
     tr_bencInit( val, TR_TYPE_STR );
 
-    if( stringFitsInBuffer( val, val->val.s.len = byteCount ))
-        memcpy( val->val.s.str.buf, src, byteCount );
+    /* There's no way in benc notation to distinguish between
+     * zero-terminated strings and raw byte arrays.
+     * Because of this, tr_bencMergeDicts() and tr_bencListCopy()
+     * don't know whether or not a TR_TYPE_STR node needs a '\0'.
+     * Append one, een to the raw arrays, just to be safe. */
+
+    if( byteCount < sizeof( val->val.s.str.buf ) )
+        setme = val->val.s.str.buf;
     else
-        val->val.s.str.ptr = tr_memdup( src, byteCount );
+        setme = val->val.s.str.ptr = tr_new( char, byteCount + 1 );
+
+    memcpy( setme, src, byteCount );
+    setme[byteCount] = '\0';
+    val->val.s.len = byteCount;
 }
 
 void
 tr_bencInitStr( tr_benc * val, const void * str, int len )
 {
-    tr_bencInit( val, TR_TYPE_STR );
-
     if( str == NULL )
         len = 0;
     else if( len < 0 )
         len = strlen( str );
 
-    if( stringFitsInBuffer( val, val->val.s.len = len )) {
-        memcpy( val->val.s.str.buf, str, len );
-        val->val.s.str.buf[len] = '\0';
-    } else
-        val->val.s.str.ptr = tr_strndup( str, len );
+    tr_bencInitRaw( val, str, len );
 }
 
 void
@@ -673,8 +669,7 @@ tr_bencListAdd( tr_benc * list )
 }
 
 tr_benc *
-tr_bencListAddInt( tr_benc * list,
-                   int64_t   val )
+tr_bencListAddInt( tr_benc * list, int64_t val )
 {
     tr_benc * node = tr_bencListAdd( list );
 
@@ -683,15 +678,37 @@ tr_bencListAddInt( tr_benc * list,
 }
 
 tr_benc *
-tr_bencListAddStr( tr_benc *    list,
-                   const char * val )
+tr_bencListAddReal( tr_benc * list, double val )
 {
     tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitReal( node, val );
+    return node;
+}
 
+tr_benc *
+tr_bencListAddBool( tr_benc * list, tr_bool val )
+{
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitBool( node, val );
+    return node;
+}
+
+tr_benc *
+tr_bencListAddStr( tr_benc * list, const char * val )
+{
+    tr_benc * node = tr_bencListAdd( list );
     tr_bencInitStr( node, val, -1 );
     return node;
 }
 
+tr_benc *
+tr_bencListAddRaw( tr_benc * list, const uint8_t * val, size_t len )
+{
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitRaw( node, val, len );
+    return node;
+}
+
 tr_benc*
 tr_bencListAddList( tr_benc * list,
                     size_t    reserveCount )
@@ -804,6 +821,35 @@ tr_bencDictAddStr( tr_benc * dict, const char * key, const char * val )
     return child;
 }
 
+tr_benc*
+tr_bencDictAddRaw( tr_benc *    dict,
+                   const char * key,
+                   const void * src,
+                   size_t       len )
+{
+    tr_benc * child;
+
+    /* see if it already exists, and if so, try to reuse it */
+    if(( child = tr_bencDictFind( dict, key ))) {
+        if( tr_bencIsString( child ) ) {
+            if( stringIsAlloced( child ) )
+                tr_free( child->val.s.str.ptr );
+        } else {
+            tr_bencDictRemove( dict, key );
+            child = NULL;
+        }
+    }
+
+    /* if it doesn't exist, create it */
+    if( child == NULL )
+        child = tr_bencDictAdd( dict, key );
+
+    /* set it */
+    tr_bencInitRaw( child, src, len );
+
+    return child;
+}
+
 tr_benc*
 tr_bencDictAddList( tr_benc *    dict,
                     const char * key,
@@ -826,18 +872,6 @@ tr_bencDictAddDict( tr_benc *    dict,
     return child;
 }
 
-tr_benc*
-tr_bencDictAddRaw( tr_benc *    dict,
-                   const char * key,
-                   const void * src,
-                   size_t       len )
-{
-    tr_benc * child = tr_bencDictAdd( dict, key );
-
-    tr_bencInitRaw( child, src, len );
-    return child;
-}
-
 int
 tr_bencDictRemove( tr_benc *    dict,
                    const char * key )
@@ -1055,7 +1089,7 @@ bencWalk( const tr_benc          * top,
 
                 default:
                     /* did caller give us an uninitialized val? */
-                    tr_err( _( "Invalid metadata" ) );
+                    tr_err( "%s", _( "Invalid metadata" ) );
                     break;
             }
     }
@@ -1180,6 +1214,7 @@ tr_bencFree( tr_benc * val )
 ****
 ***/
 
+/** @brief Implementation helper class for tr_bencToBuffer(TR_FMT_JSON) */
 struct ParentState
 {
     int    bencType;
@@ -1187,6 +1222,7 @@ struct ParentState
     int    childCount;
 };
 
+/** @brief Implementation helper class for tr_bencToBuffer(TR_FMT_JSON) */
 struct jsonWalk
 {
     tr_bool doIndent;
@@ -1301,7 +1337,7 @@ jsonRealFunc( const tr_benc * val, void * vdata )
         /* json requires a '.' decimal point regardless of locale */
         tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
         setlocale( LC_NUMERIC, "POSIX" );
-        evbuffer_add_printf( data->out, "%.4f", val->val.d );
+        evbuffer_add_printf( data->out, "%.4f", tr_truncd( val->val.d, 4 ) );
         setlocale( LC_NUMERIC, locale );
     }
 
@@ -1339,9 +1375,7 @@ jsonStringFunc( const tr_benc * val, void * vdata )
                     UTF32        buf = 0;
                     UTF32 *      u32 = &buf;
                     ConversionResult result = ConvertUTF8toUTF32( &tmp, end, &u32, &buf + 1, 0 );
-                    if( ( result != conversionOK ) && ( tmp == it ) )
-                        ++it; /* it's beyond help; skip it */
-                    else {
+                    if((( result==conversionOK ) || (result==targetExhausted)) && (tmp!=it)) {
                         evbuffer_add_printf( data->out, "\\u%04x", (unsigned int)buf );
                         it = tmp - 1;
                     }
@@ -1406,6 +1440,51 @@ static const struct WalkFuncs jsonWalkFuncs = { jsonIntFunc,
 ****
 ***/
 
+static void
+tr_bencListCopy( tr_benc * target, const tr_benc * src )
+{
+    int i = 0;
+    const tr_benc * val;
+
+    while(( val = tr_bencListChild( (tr_benc*)src, i++ )))
+    {
+       if( tr_bencIsBool( val ) )
+       {
+           tr_bool boolVal = 0;
+           tr_bencGetBool( val, &boolVal );
+           tr_bencListAddBool( target, boolVal );
+       }
+       else if( tr_bencIsReal( val ) )
+       {
+           double realVal = 0;
+           tr_bencGetReal( val, &realVal );
+           tr_bencListAddReal( target, realVal );
+       }
+       else if( tr_bencIsInt( val ) )
+       {
+           int64_t intVal = 0;
+           tr_bencGetInt( val, &intVal );
+           tr_bencListAddInt( target, intVal );
+       }
+       else if( tr_bencIsString( val ) )
+       {
+           tr_bencListAddRaw( target, (const uint8_t*)getStr( val ), val->val.s.len );
+       }
+       else if( tr_bencIsDict( val ) )
+       {
+           tr_bencMergeDicts( tr_bencListAddDict( target, 0 ), val );
+       }
+       else if ( tr_bencIsList( val ) )
+       {
+           tr_bencListCopy( tr_bencListAddList( target, 0 ), val );
+       }
+       else
+       {
+           tr_err( "tr_bencListCopy skipping item" );
+       }
+   }
+}
+
 static size_t
 tr_bencDictSize( const tr_benc * dict )
 {
@@ -1435,7 +1514,7 @@ tr_bencDictChild( tr_benc * dict, size_t n, const char ** key, tr_benc ** val )
     return success;
 }
 
-void 
+void
 tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
 {
     size_t i;
@@ -1460,26 +1539,31 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
             }
             else if( tr_bencIsReal( val ) )
             {
-                double realVal;
+                double realVal = 0;
                 tr_bencGetReal( val, &realVal );
                 tr_bencDictAddReal( target, key, realVal );
             }
             else if( tr_bencIsInt( val ) )
             {
-                int64_t intVal;
+                int64_t intVal = 0;
                 tr_bencGetInt( val, &intVal );
                 tr_bencDictAddInt( target, key, intVal );
             }
             else if( tr_bencIsString( val ) )
             {
-                const char * strVal;
-                tr_bencGetStr( val, &strVal );
-                tr_bencDictAddStr( target, key, strVal );
+                tr_bencDictAddRaw( target, key, getStr( val ), val->val.s.len );
             }
             else if( tr_bencIsDict( val ) && tr_bencDictFindDict( target, key, &t ) )
             {
                 tr_bencMergeDicts( t, val );
             }
+            else if( tr_bencIsList( val ) )
+            {
+                if( tr_bencDictFind( target, key ) == NULL )
+                {
+                    tr_bencListCopy( tr_bencDictAddList( target, key, tr_bencListSize( val ) ), val );
+                }
+            }
             else
             {
                 tr_dbg( "tr_bencMergeDicts skipping \"%s\"", key );
@@ -1496,6 +1580,7 @@ void
 tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode, struct evbuffer * buf )
 {
     evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
+    evbuffer_expand( buf, 4096 ); /* alloc a little memory to start off with */
 
     switch( mode )
     {
@@ -1533,31 +1618,63 @@ tr_bencToStr( const tr_benc * top, tr_fmt_mode mode, int * len )
 int
 tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
 {
+    char * tmp;
+    int fd;
     int err = 0;
-    FILE * fp = fopen( filename, "wb+" );
 
-    if( fp == NULL )
-    {
-        err = errno;
-        tr_err( _( "Couldn't open \"%1$s\": %2$s" ),
-                filename, tr_strerror( errno ) );
-    }
-    else
+    /* if the file already exists, try to move it out of the way & keep it as a backup */
+    tmp = tr_strdup_printf( "%s.tmp.XXXXXX", filename );
+    fd = mkstemp( tmp );
+    if( fd >= 0 )
     {
         int len;
         char * str = tr_bencToStr( top, mode, &len );
+        tr_dbg( "Writing %d bytes to temporary file \"%s\"", (int)len, tmp );
 
-        if( fwrite( str, 1, len, fp ) == (size_t)len )
-            tr_dbg( "tr_bencToFile saved \"%s\"", filename );
-        else {
+        if( write( fd, str, len ) == (ssize_t)len )
+        {
+            fsync( fd );
+            close( fd );
+
+            if( !unlink( filename ) || ( errno == ENOENT ) )
+            {
+                tr_dbg( "Renaming \"%s\" as \"%s\"", tmp, filename );
+
+                if( !rename( tmp, filename ) )
+                {
+                    tr_inf( _( "Saved \"%s\"" ), filename );
+                }
+                else
+                {
+                    err = errno;
+                    tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
+                    unlink( tmp );
+                }
+            }
+            else
+            {
+                err = errno;
+                tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
+                unlink( tmp );
+            }
+        }
+        else
+        {
             err = errno;
-            tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
+            tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
+            close( fd );
+            unlink( tmp );
         }
 
         tr_free( str );
-        fclose( fp );
+    }
+    else
+    {
+        err = errno;
+        tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
     }
 
+    tr_free( tmp );
     return err;
 }
 
