@@ -1,3 +1,8 @@
+#ifndef __TRANSMISSION__
+#error only libtransmission should #include this header.
+#endif
+
+
 /* File ggets.h  - goodgets is a safe alternative to gets */
 /* By C.B. Falconer.  Public domain 2002-06-22            */
 /*    attribution appreciated.                            */
@@ -34,8 +39,11 @@
 extern "C" {
 #endif
 
-int fggets( char* *ln,
-            FILE * f );
+/**
+ * @brief read a line from a file and place it in a newly-allocated string
+ * @ingroup utils
+ */
+int fggets( char* *ln, FILE * f );
 
 #define ggets( ln ) fggets( ln, stdin )
 
