@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: list.c 7788 2009-01-24 02:33:25Z charles $
+ * $Id: list.c 9891 2010-01-06 00:18:33Z charles $
  */
 
 #include "transmission.h"
@@ -165,12 +165,6 @@ tr_list_size( const tr_list * list )
  * insert/remove operations
  */
 
-void
-__tr_list_init( struct __tr_list * head )
-{
-    head->next = head;
-    head->prev = head;
-}
 
 void
 __tr_list_insert( struct __tr_list * list,
@@ -183,7 +177,7 @@ __tr_list_insert( struct __tr_list * list,
     prev->next = list;
 }
 
-void
+static void
 __tr_list_splice( struct __tr_list * prev,
 		  struct __tr_list * next)
 {
@@ -191,14 +185,6 @@ __tr_list_splice( struct __tr_list * prev,
     prev->next = next;
 }
 
- 
-void
-__tr_list_append( struct __tr_list * head,
-		  struct __tr_list * list)
-{
-    __tr_list_insert( list, head->prev, head );
-}
-
 void
 __tr_list_remove( struct __tr_list * head )
 {
