@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.h 8204 2009-04-10 17:34:25Z charles $
+ * $Id: makemeta.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef TR_MAKEMETA_H
@@ -49,7 +49,6 @@ typedef struct tr_metainfo_builder
     uint32_t                    pieceSize;
     uint32_t                    pieceCount;
     int                         isSingleFile;
-    tr_session *                handle;
 
     /**
     ***  These are set inside tr_makeMetaInfo()
@@ -75,7 +74,8 @@ typedef struct tr_metainfo_builder
     int                        isDone;
     tr_metainfo_builder_err    result;
 
-    /* file in use when result was set to _IO_READ or _IO_WRITE */
+    /* file in use when result was set to _IO_READ or _IO_WRITE,
+     * or the URL in use when the result was set to _URL */
     char    errfile[2048];
 
     /* errno encountered when result was set to _IO_READ or _IO_WRITE */
@@ -91,8 +91,7 @@ typedef struct tr_metainfo_builder
 tr_metainfo_builder;
 
 
-tr_metainfo_builder*tr_metaInfoBuilderCreate( tr_session * session,
-                                              const char * topFile );
+tr_metainfo_builder*tr_metaInfoBuilderCreate( const char * topFile );
 
 void                tr_metaInfoBuilderFree( tr_metainfo_builder* );
 
