@@ -1,26 +1,15 @@
-/******************************************************************************
- * $Id: metainfo.c 8889 2009-08-10 20:04:08Z charles $
+/*
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
+            trackers[trackerCount].id = 0;
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: metainfo.c 10239 2010-02-20 15:57:05Z charles $
+ */
 
 #include <assert.h>
 #include <errno.h>
@@ -136,24 +125,28 @@ tr_metainfoMigrate( tr_session * session,
 ***/
 
 static tr_bool
-getfile( char        ** setme,
-         const char   * root,
-         tr_benc      * path )
+path_is_suspicious( const char * path )
+{
+    return ( path == NULL )
+        || ( strstr( path, "../" ) != NULL );
+}
+
+static tr_bool
+getfile( char ** setme, const char * root, tr_benc * path )
 {
     tr_bool success = FALSE;
 
     if( tr_bencIsList( path ) )
     {
+        int i;
+        const int n = tr_bencListSize( path );
         struct evbuffer * buf = evbuffer_new( );
-        int               n = tr_bencListSize( path );
-        int               i;
 
         evbuffer_add( buf, root, strlen( root ) );
         for( i = 0; i < n; ++i )
         {
             const char * str;
-            if( tr_bencGetStr( tr_bencListChild( path, i ), &str )
-              && strcmp( str, ".." ) )
+            if( tr_bencGetStr( tr_bencListChild( path, i ), &str ) )
             {
                 evbuffer_add( buf, TR_PATH_DELIMITER_STR, 1 );
                 evbuffer_add( buf, str, strlen( str ) );
@@ -166,13 +159,18 @@ getfile( char        ** setme,
         success = TRUE;
     }
 
+    if( ( *setme != NULL ) && path_is_suspicious( *setme ) )
+    {
+        tr_free( *setme );
+        *setme = NULL;
+        success = FALSE;
+    }
+
     return success;
 }
 
 static const char*
-parseFiles( tr_info *       inf,
-            tr_benc *       files,
-            const tr_benc * length )
+parseFiles( tr_info * inf, tr_benc * files, const tr_benc * length )
 {
     int64_t len;
 
@@ -211,6 +209,9 @@ parseFiles( tr_info *       inf,
     }
     else if( tr_bencGetInt( length, &len ) ) /* single-file mode */
     {
+        if( path_is_suspicious( inf->name ) )
+            return "path";
+
         inf->isMultifile      = 0;
         inf->fileCount        = 1;
         inf->files            = tr_new0( tr_file, 1 );
@@ -227,7 +228,7 @@ parseFiles( tr_info *       inf,
 }
 
 static char *
-announceToScrape( const char * announce )
+tr_convertAnnounceToScrape( const char * announce )
 {
     char *       scrape = NULL;
     const char * s;
@@ -257,8 +258,7 @@ announceToScrape( const char * announce )
 }
 
 static const char*
-getannounce( tr_info * inf,
-             tr_benc * meta )
+getannounce( tr_info * inf, tr_benc * meta )
 {
     const char *      str;
     tr_tracker_info * trackers = NULL;
@@ -277,7 +277,6 @@ getannounce( tr_info * inf,
             n += tr_bencListSize( tr_bencListChild( tiers, i ) );
 
         trackers = tr_new0( tr_tracker_info, n );
-        trackerCount = 0;
 
         for( i = 0, validTiers = 0; i < numTiers; ++i )
         {
@@ -289,14 +288,16 @@ getannounce( tr_info * inf,
                 if( tr_bencGetStr( tr_bencListChild( tier, j ), &str ) )
                 {
                     char * url = tr_strstrip( tr_strdup( str ) );
-                    if( tr_httpIsValidURL( url ) )
+                    if( tr_urlIsValidTracker( url ) )
                     {
-                        tr_tracker_info * t = trackers + trackerCount++;
+                        tr_tracker_info * t = trackers + trackerCount;
                         t->tier = validTiers;
                         t->announce = tr_strdup( url );
-                        t->scrape = announceToScrape( url );
+                        t->scrape = tr_convertAnnounceToScrape( url );
+                        t->id = trackerCount;
 
                         anyAdded = TRUE;
+                        ++trackerCount;
                     }
                     tr_free( url );
                 }
@@ -319,12 +320,14 @@ getannounce( tr_info * inf,
       && tr_bencDictFindStr( meta, "announce", &str ) )
     {
         char * url = tr_strstrip( tr_strdup( str ) );
-        if( tr_httpIsValidURL( url ) )
+        if( tr_urlIsValidTracker( url ) )
         {
             trackers = tr_new0( tr_tracker_info, 1 );
             trackers[trackerCount].tier = 0;
             trackers[trackerCount].announce = tr_strdup( url );
-            trackers[trackerCount++].scrape = announceToScrape( url );
+            trackers[trackerCount].scrape = tr_convertAnnounceToScrape( url );
+            trackers[trackerCount].id = 0;
+            trackerCount++;
             /*fprintf( stderr, "single announce: [%s]\n", url );*/
         }
         tr_free( url );
@@ -333,7 +336,7 @@ getannounce( tr_info * inf,
     inf->trackers = trackers;
     inf->trackerCount = trackerCount;
 
-    return inf->trackerCount ? NULL : "announce";
+    return NULL;
 }
 
 static void
@@ -357,41 +360,116 @@ geturllist( tr_info * inf,
     }
 }
 
+static int
+is_rfc2396_alnum( char ch )
+{
+    return ( '0' <= ch && ch <= '9' )
+        || ( 'A' <= ch && ch <= 'Z' )
+        || ( 'a' <= ch && ch <= 'z' );
+}
+
+static void
+escape( char * out, const uint8_t * in, size_t in_len ) /* rfc2396 */
+{
+    const uint8_t *end = in + in_len;
+
+    while( in != end )
+        if( is_rfc2396_alnum( *in ) )
+            *out++ = (char) *in++;
+        else
+            out += tr_snprintf( out, 4, "%%%02X", (unsigned int)*in++ );
+
+    *out = '\0';
+}
+
 static const char*
-tr_metainfoParseImpl( const tr_session * session,
-                      tr_info *         inf,
-                      const tr_benc *   meta_in )
+tr_metainfoParseImpl( const tr_session  * session,
+                      tr_info           * inf,
+                      tr_bool           * hasInfoDict,
+                      int               * infoDictOffset,
+                      int               * infoDictLength,
+                      const tr_benc     * meta_in )
 {
     int64_t         i;
     size_t          raw_len;
     const char *    str;
     const uint8_t * raw;
-    tr_benc *       beInfo;
+    tr_benc *       d;
+    tr_benc *       infoDict = NULL;
     tr_benc *       meta = (tr_benc *) meta_in;
     tr_bool         err;
+    tr_bool         b;
+    tr_bool         isMagnet = FALSE;
 
     /* info_hash: urlencoded 20-byte SHA1 hash of the value of the info key
      * from the Metainfo file. Note that the value will be a bencoded
      * dictionary, given the definition of the info key above. */
-    if( !tr_bencDictFindDict( meta, "info", &beInfo ) )
-        return "info";
+    b = tr_bencDictFindDict( meta, "info", &infoDict );
+    if( hasInfoDict != NULL )
+        *hasInfoDict = b;
+    if( !b )
+    {
+        /* no info dictionary... is this a magnet link? */
+        if( tr_bencDictFindDict( meta, "magnet-info", &d ) )
+        {
+            isMagnet = TRUE;
+
+            /* get the info-hash */
+            if( !tr_bencDictFindRaw( d, "info_hash", &raw, &raw_len ) )
+                return "info_hash";
+            if( raw_len != SHA_DIGEST_LENGTH )
+                return "info_hash";
+            memcpy( inf->hash, raw, raw_len );
+            tr_sha1_to_hex( inf->hashString, inf->hash );
+            escape( inf->hashEscaped, inf->hash, SHA_DIGEST_LENGTH );
+
+            /* maybe get the display name */
+            if( tr_bencDictFindStr( d, "display-name", &str ) ) {
+                tr_free( inf->name );
+                inf->name = tr_strdup( str );
+            }
+        }
+        else /* not a magnet link and has no info dict... */
+        {
+            return "info";
+        }
+    }
     else
     {
         int len;
-        char * bstr = tr_bencToStr( beInfo, TR_FMT_BENC, &len );
+        char * bstr = tr_bencToStr( infoDict, TR_FMT_BENC, &len );
         tr_sha1( inf->hash, bstr, len, NULL );
         tr_sha1_to_hex( inf->hashString, inf->hash );
+        escape( inf->hashEscaped, inf->hash, SHA_DIGEST_LENGTH );
+
+        if( infoDictLength != NULL )
+            *infoDictLength = len;
+
+        if( infoDictOffset != NULL )
+        {
+            int mlen = 0;
+            char * mstr = tr_bencToStr( meta_in, TR_FMT_BENC, &mlen );
+            const char * offset = tr_memmem( mstr, mlen, bstr, len );
+            if( offset != NULL )
+                *infoDictOffset = offset - mstr;
+            tr_free( mstr );
+            if( offset == NULL )
+                return "info";
+        }
+
         tr_free( bstr );
     }
 
     /* name */
-    if( !tr_bencDictFindStr( beInfo, "name.utf-8", &str ) )
-        if( !tr_bencDictFindStr( beInfo, "name", &str ) )
-            str = "";
-    if( !str || !*str )
-        return "name";
-    tr_free( inf->name );
-    inf->name = tr_utf8clean( str, -1, &err );
+    if( !isMagnet ) {
+        if( !tr_bencDictFindStr( infoDict, "name.utf-8", &str ) )
+            if( !tr_bencDictFindStr( infoDict, "name", &str ) )
+                str = "";
+        if( !str || !*str )
+            return "name";
+        tr_free( inf->name );
+        inf->name = tr_utf8clean( str, -1, &err );
+    }
 
     /* comment */
     if( !tr_bencDictFindStr( meta, "comment.utf-8", &str ) )
@@ -413,35 +491,42 @@ tr_metainfoParseImpl( const tr_session * session,
     inf->dateCreated = i;
 
     /* private */
-    if( !tr_bencDictFindInt( beInfo, "private", &i ) )
+    if( !tr_bencDictFindInt( infoDict, "private", &i ) )
         if( !tr_bencDictFindInt( meta, "private", &i ) )
             i = 0;
     inf->isPrivate = i != 0;
 
     /* piece length */
-    if( !tr_bencDictFindInt( beInfo, "piece length", &i ) || ( i < 1 ) )
-        return "piece length";
-    inf->pieceSize = i;
+    if( !isMagnet ) {
+        if( !tr_bencDictFindInt( infoDict, "piece length", &i ) || ( i < 1 ) )
+            return "piece length";
+        inf->pieceSize = i;
+    }
 
     /* pieces */
-    if( !tr_bencDictFindRaw( beInfo, "pieces", &raw,
-                             &raw_len ) || ( raw_len % SHA_DIGEST_LENGTH ) )
-        return "pieces";
-    inf->pieceCount = raw_len / SHA_DIGEST_LENGTH;
-    inf->pieces = tr_new0( tr_piece, inf->pieceCount );
-    for( i = 0; i < inf->pieceCount; ++i )
-        memcpy( inf->pieces[i].hash, &raw[i * SHA_DIGEST_LENGTH],
-                SHA_DIGEST_LENGTH );
+    if( !isMagnet ) {
+        if( !tr_bencDictFindRaw( infoDict, "pieces", &raw, &raw_len ) )
+            return "pieces";
+        if( raw_len % SHA_DIGEST_LENGTH )
+            return "pieces";
+        inf->pieceCount = raw_len / SHA_DIGEST_LENGTH;
+        inf->pieces = tr_new0( tr_piece, inf->pieceCount );
+        for( i = 0; i < inf->pieceCount; ++i )
+            memcpy( inf->pieces[i].hash, &raw[i * SHA_DIGEST_LENGTH],
+                    SHA_DIGEST_LENGTH );
+    }
 
     /* files */
-    if( ( str = parseFiles( inf, tr_bencDictFind( beInfo, "files" ),
-                                 tr_bencDictFind( beInfo, "length" ) ) ) )
-        return str;
-    if( !inf->fileCount || !inf->totalSize )
-        return "files";
-    if( (uint64_t) inf->pieceCount !=
-       ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
-        return "files";
+    if( !isMagnet ) {
+        if( ( str = parseFiles( inf, tr_bencDictFind( infoDict, "files" ),
+                                     tr_bencDictFind( infoDict, "length" ) ) ) )
+            return str;
+        if( !inf->fileCount || !inf->totalSize )
+            return "files";
+        if( (uint64_t) inf->pieceCount !=
+           ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
+            return "files";
+    }
 
     /* get announce or announce-list */
     if( ( str = getannounce( inf, meta ) ) )
@@ -459,10 +544,18 @@ tr_metainfoParseImpl( const tr_session * session,
 
 tr_bool
 tr_metainfoParse( const tr_session * session,
+                  const tr_benc    * meta_in,
                   tr_info          * inf,
-                  const tr_benc    * meta_in )
+                  tr_bool          * hasInfoDict,
+                  int              * infoDictOffset,
+                  int              * infoDictLength )
 {
-    const char * badTag = tr_metainfoParseImpl( session, inf, meta_in );
+    const char * badTag = tr_metainfoParseImpl( session,
+                                                inf,
+                                                hasInfoDict,
+                                                infoDictOffset,
+                                                infoDictLength,
+                                                meta_in );
     const tr_bool success = badTag == NULL;
 
     if( badTag )
@@ -477,8 +570,8 @@ tr_metainfoParse( const tr_session * session,
 void
 tr_metainfoFree( tr_info * inf )
 {
+    int i;
     tr_file_index_t ff;
-    int             i;
 
     for( i = 0; i < inf->webseedCount; ++i )
         tr_free( inf->webseeds[i] );
@@ -518,4 +611,3 @@ tr_metainfoRemoveSaved( const tr_session * session,
     unlink( filename );
     tr_free( filename );
 }
-
