@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: net.h 8204 2009-04-10 17:34:25Z charles $
+ * $Id: net.h 10018 2010-01-25 07:35:38Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -61,8 +61,10 @@ struct tr_session;
 typedef enum tr_address_type
 {
     TR_AF_INET,
-    TR_AF_INET6
-} tr_address_type;
+    TR_AF_INET6,
+    NUM_TR_AF_INET_TYPES
+}
+tr_address_type;
 
 typedef struct tr_address
 {
@@ -89,36 +91,17 @@ int tr_compareAddresses( const tr_address * a,
 
 tr_bool tr_isValidPeerAddress( const tr_address * addr, tr_port port );
 
-static TR_INLINE tr_bool tr_isAddress( const tr_address * a ) { return ( a != NULL ) && ( a->type==TR_AF_INET || a->type==TR_AF_INET6 ); }
+static inline tr_bool tr_isAddress( const tr_address * a ) { return ( a != NULL ) && ( a->type==TR_AF_INET || a->type==TR_AF_INET6 ); }
 
 tr_bool tr_net_hasIPv6( tr_port );
 
-/***********************************************************************
- * Socket list housekeeping
- **********************************************************************/
-typedef struct tr_socketList tr_socketList;
-tr_socketList *tr_socketListAppend( tr_socketList * const head,
-                                    const tr_address * const addr );
-tr_socketList *tr_socketListNew( const tr_address * const addr );
-void tr_socketListFree( tr_socketList * const head );
-void tr_socketListRemove( tr_socketList * const head,
-                          tr_socketList * const el);
-void tr_socketListTruncate( tr_socketList * const head,
-                            tr_socketList * const start );
-int tr_socketListGetSocket( const tr_socketList * const el );
-const tr_address *tr_socketListGetAddress( const tr_socketList * const el );
-void tr_socketListForEach( tr_socketList * const head,
-                           void ( * cb ) ( int * const,
-                                           tr_address * const,
-                                           void * const ),
-                           void * const userData);
-
 /***********************************************************************
  * Sockets
  **********************************************************************/
-int  tr_netOpenTCP( tr_session       * session,
-                    const tr_address * addr,
-                    tr_port            port );
+int  tr_netOpenPeerSocket( tr_session       * session,
+                           const tr_address * addr,
+                           tr_port            port,
+                           tr_bool            clientIsSeed );
 
 int  tr_netBindTCP( const tr_address * addr,
                     tr_port            port,
@@ -132,8 +115,13 @@ int  tr_netAccept( tr_session * session,
 int  tr_netSetTOS( int s,
                    int tos );
 
-void tr_netClose( int s );
+void tr_netClose( tr_session * session, int s );
+
+void tr_netCloseSocket( int fd );
 
 void tr_netInit( void );
 
+const unsigned char *tr_globalIPv6( void );
+
+
 #endif /* _TR_NET_H_ */
