@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 8605 2009-06-03 22:52:08Z charles $
+ * $Id: peer-io.c 10300 2010-03-02 04:51:40Z charles $
  */
 
 #include <assert.h>
@@ -32,8 +32,7 @@
 #include "list.h"
 #include "net.h"
 #include "peer-io.h"
-#include "platform.h" /* MAX_STACK_ARRAY_SIZE */
-#include "trevent.h"
+#include "trevent.h" /* tr_runInEventThread() */
 #include "utils.h"
 
 #define MAGIC_NUMBER 206745
@@ -57,7 +56,7 @@ guessPacketOverhead( size_t d )
      *  (1500-72)/(38+1500) = 92.8479 %  IPv6, TCP timestamps
      *  (1500-72)/(42+1500) = 92.6070 %  802.1q, IPv6, ICP timestamps
      */
-    static const double assumed_payload_data_rate = 94.0;
+    const double assumed_payload_data_rate = 94.0;
 
     return (size_t)( d * ( 100.0 / assumed_payload_data_rate ) - d );
 }
@@ -99,7 +98,7 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 
         if( io->didWrite )
             io->didWrite( io, payload, next->isPieceData, io->userData );
-        
+
         if( tr_isPeerIo( io ) )
         {
             bytes_transferred -= payload;
@@ -115,18 +114,22 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 static void
 canReadWrapper( tr_peerIo * io )
 {
-    tr_bool done = 0;
     tr_bool err = 0;
-    tr_session * session = io->session;
+    tr_bool done = 0;
+    tr_session * session;
 
     dbgmsg( io, "canRead" );
 
+    assert( tr_isPeerIo( io ) );
+    assert( tr_isSession( io->session ) );
     tr_peerIoRef( io );
 
+    session = io->session;
+
     /* try to consume the input buffer */
     if( io->canRead )
     {
-        tr_globalLock( session );
+        tr_sessionLock( session );
 
         while( !done && !err )
         {
@@ -160,11 +163,20 @@ canReadWrapper( tr_peerIo * io )
                     err = 1;
                     break;
             }
+
+            assert( tr_isPeerIo( io ) );
         }
 
-        tr_globalUnlock( session );
+        tr_sessionUnlock( session );
     }
 
+    /* keep the iobuf's excess capacity from growing too large */
+    if( EVBUFFER_LENGTH( io->inbuf ) == 0 ) {
+        evbuffer_free( io->inbuf );
+        io->inbuf = evbuffer_new( );
+    }
+
+    assert( tr_isPeerIo( io ) );
     tr_peerIoUnref( io );
 }
 
@@ -261,6 +273,12 @@ tr_evbuffer_write( tr_peerIo * io, int fd, size_t howmuch )
     if( n > 0 )
         evbuffer_drain( buffer, n );
 
+    /* keep the iobuf's excess capacity from growing too large */
+    if( EVBUFFER_LENGTH( io->outbuf ) == 0 ) {
+        evbuffer_free( io->outbuf );
+        io->outbuf = evbuffer_new( );
+    }
+
     return n;
 }
 
@@ -344,13 +362,16 @@ tr_peerIoNew( tr_session       * session,
               const tr_address * addr,
               tr_port            port,
               const uint8_t    * torrentHash,
-              int                isIncoming,
+              tr_bool            isIncoming,
+              tr_bool            isSeed,
               int                socket )
 {
     tr_peerIo * io;
 
     assert( session != NULL );
     assert( session->events != NULL );
+    assert( tr_isBool( isIncoming ) );
+    assert( tr_isBool( isSeed ) );
     assert( tr_amInEventThread( session ) );
 
     if( socket >= 0 )
@@ -362,11 +383,12 @@ tr_peerIoNew( tr_session       * session,
     io->crypto = tr_cryptoNew( torrentHash, isIncoming );
     io->session = session;
     io->addr = *addr;
+    io->isSeed = isSeed;
     io->port = port;
     io->socket = socket;
     io->isIncoming = isIncoming != 0;
     io->hasFinishedConnecting = FALSE;
-    io->timeCreated = time( NULL );
+    io->timeCreated = tr_time( );
     io->inbuf = evbuffer_new( );
     io->outbuf = evbuffer_new( );
     tr_bandwidthConstruct( &io->bandwidth, session, parent );
@@ -386,13 +408,13 @@ tr_peerIoNewIncoming( tr_session        * session,
                       tr_bandwidth      * parent,
                       const tr_address  * addr,
                       tr_port             port,
-                      int                 socket )
+                      int                 fd )
 {
     assert( session );
     assert( tr_isAddress( addr ) );
-    assert( socket >= 0 );
+    assert( fd >= 0 );
 
-    return tr_peerIoNew( session, parent, addr, port, NULL, 1, socket );
+    return tr_peerIoNew( session, parent, addr, port, NULL, TRUE, FALSE, fd );
 }
 
 tr_peerIo*
@@ -400,22 +422,96 @@ tr_peerIoNewOutgoing( tr_session        * session,
                       tr_bandwidth      * parent,
                       const tr_address  * addr,
                       tr_port             port,
-                      const uint8_t     * torrentHash )
+                      const uint8_t     * torrentHash,
+                      tr_bool             isSeed )
 {
-    int socket;
+    int fd;
 
     assert( session );
     assert( tr_isAddress( addr ) );
     assert( torrentHash );
 
-    socket = tr_netOpenTCP( session, addr, port );
-    dbgmsg( NULL, "tr_netOpenTCP returned fd %d", socket );
+    fd = tr_netOpenPeerSocket( session, addr, port, isSeed );
+    dbgmsg( NULL, "tr_netOpenPeerSocket returned fd %d", fd );
 
-    return socket < 0
-           ? NULL
-           : tr_peerIoNew( session, parent, addr, port, torrentHash, 0, socket );
+    return fd < 0 ? NULL
+                  : tr_peerIoNew( session, parent, addr, port, torrentHash, FALSE, isSeed, fd );
 }
 
+/***
+****
+***/
+
+static void
+event_enable( tr_peerIo * io, short event )
+{
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session != NULL );
+    assert( io->session->events != NULL );
+    assert( event_initialized( &io->event_read ) );
+    assert( event_initialized( &io->event_write ) );
+
+    if( ( event & EV_READ ) && ! ( io->pendingEvents & EV_READ ) )
+    {
+        dbgmsg( io, "enabling libevent ready-to-read polling" );
+        event_add( &io->event_read, NULL );
+        io->pendingEvents |= EV_READ;
+    }
+
+    if( ( event & EV_WRITE ) && ! ( io->pendingEvents & EV_WRITE ) )
+    {
+        dbgmsg( io, "enabling libevent ready-to-write polling" );
+        event_add( &io->event_write, NULL );
+        io->pendingEvents |= EV_WRITE;
+    }
+}
+
+static void
+event_disable( struct tr_peerIo * io, short event )
+{
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session != NULL );
+    assert( io->session->events != NULL );
+    assert( event_initialized( &io->event_read ) );
+    assert( event_initialized( &io->event_write ) );
+
+    if( ( event & EV_READ ) && ( io->pendingEvents & EV_READ ) )
+    {
+        dbgmsg( io, "disabling libevent ready-to-read polling" );
+        event_del( &io->event_read );
+        io->pendingEvents &= ~EV_READ;
+    }
+
+    if( ( event & EV_WRITE ) && ( io->pendingEvents & EV_WRITE ) )
+    {
+        dbgmsg( io, "disabling libevent ready-to-write polling" );
+        event_del( &io->event_write );
+        io->pendingEvents &= ~EV_WRITE;
+    }
+}
+
+void
+tr_peerIoSetEnabled( tr_peerIo    * io,
+                     tr_direction   dir,
+                     tr_bool        isEnabled )
+{
+    const short event = dir == TR_UP ? EV_WRITE : EV_READ;
+
+    assert( tr_isPeerIo( io ) );
+    assert( tr_isDirection( dir ) );
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session->events != NULL );
+
+    if( isEnabled )
+        event_enable( io, event );
+    else
+        event_disable( io, event );
+}
+
+/***
+****
+***/
+
 static void
 trDatatypeFree( void * data )
 {
@@ -433,16 +529,15 @@ io_dtor( void * vio )
     assert( io->session->events != NULL );
 
     dbgmsg( io, "in tr_peerIo destructor" );
-    event_del( &io->event_read );
-    event_del( &io->event_write );
+    event_disable( io, EV_READ | EV_WRITE );
     tr_bandwidthDestruct( &io->bandwidth );
     evbuffer_free( io->outbuf );
     evbuffer_free( io->inbuf );
-    tr_netClose( io->socket );
+    tr_netClose( io->session, io->socket );
     tr_cryptoFree( io->crypto );
     __tr_list_destroy( &io->outbuf_datatypes, trDatatypeFree );
 
-    memset( io, ~0, sizeof( tr_peerIo ) ); 
+    memset( io, ~0, sizeof( tr_peerIo ) );
     tr_free( io );
 }
 
@@ -498,10 +593,10 @@ tr_peerIoAddrStr( const tr_address * addr, tr_port port )
 {
     static char buf[512];
 
-    if( addr->type == TR_AF_INET ) 
-        tr_snprintf( buf, sizeof( buf ), "%s:%u", tr_ntop_non_ts( addr ), ntohs( port ) ); 
-    else 
-        tr_snprintf( buf, sizeof( buf ), "[%s]:%u", tr_ntop_non_ts( addr ), ntohs( port ) ); 
+    if( addr->type == TR_AF_INET )
+        tr_snprintf( buf, sizeof( buf ), "%s:%u", tr_ntop_non_ts( addr ), ntohs( port ) );
+    else
+        tr_snprintf( buf, sizeof( buf ), "[%s]:%u", tr_ntop_non_ts( addr ), ntohs( port ) );
     return buf;
 }
 
@@ -529,15 +624,28 @@ tr_peerIoClear( tr_peerIo * io )
 int
 tr_peerIoReconnect( tr_peerIo * io )
 {
+    int pendingEvents;
+    tr_session * session;
+
+    assert( tr_isPeerIo( io ) );
     assert( !tr_peerIoIsIncoming( io ) );
 
+    session = tr_peerIoGetSession( io );
+
+    pendingEvents = io->pendingEvents;
+    event_disable( io, EV_READ | EV_WRITE );
+
     if( io->socket >= 0 )
-        tr_netClose( io->socket );
+        tr_netClose( session, io->socket );
+
+    io->socket = tr_netOpenPeerSocket( session, &io->addr, io->port, io->isSeed );
+    event_set( &io->event_read, io->socket, EV_READ, event_read_cb, io );
+    event_set( &io->event_write, io->socket, EV_WRITE, event_write_cb, io );
+    event_enable( io, pendingEvents );
 
-    io->socket = tr_netOpenTCP( io->session, &io->addr, io->port ); 
     if( io->socket >= 0 )
     {
-        tr_netSetTOS( io->socket, io->session->peerSocketTOS );
+        tr_netSetTOS( io->socket, session->peerSocketTOS );
         return 0;
     }
 
@@ -701,17 +809,19 @@ tr_peerIoWrite( tr_peerIo   * io,
         case PEER_ENCRYPTION_RC4:
         {
             /* FIXME(libevent2): use evbuffer_reserve_space() and evbuffer_commit_space() instead of tmp */
-            uint8_t tmp[MAX_STACK_ARRAY_SIZE];
+            void * tmp = tr_sessionGetBuffer( io->session );
+            const size_t tmplen = SESSION_BUFFER_SIZE;
             const uint8_t * walk = bytes;
             evbuffer_expand( io->outbuf, byteCount );
             while( byteCount > 0 )
             {
-                const size_t thisPass = MIN( byteCount, sizeof( tmp ) );
+                const size_t thisPass = MIN( byteCount, tmplen );
                 tr_cryptoEncrypt( io->crypto, thisPass, walk, tmp );
                 evbuffer_add( io->outbuf, tmp, thisPass );
                 walk += thisPass;
                 byteCount -= thisPass;
             }
+            tr_sessionReleaseBuffer( io->session );
             break;
         }
 
@@ -772,14 +882,17 @@ tr_peerIoDrain( tr_peerIo       * io,
                 struct evbuffer * inbuf,
                 size_t            byteCount )
 {
-    uint8_t tmp[MAX_STACK_ARRAY_SIZE];
+    void * buf = tr_sessionGetBuffer( io->session );
+    const size_t buflen = SESSION_BUFFER_SIZE;
 
     while( byteCount > 0 )
     {
-        const size_t thisPass = MIN( byteCount, sizeof( tmp ) );
-        tr_peerIoReadBytes( io, inbuf, tmp, thisPass );
+        const size_t thisPass = MIN( byteCount, buflen );
+        tr_peerIoReadBytes( io, inbuf, buf, thisPass );
         byteCount -= thisPass;
     }
+
+    tr_sessionReleaseBuffer( io->session );
 }
 
 /***
@@ -835,7 +948,9 @@ tr_peerIoTryWrite( tr_peerIo * io, size_t howmuch )
         {
             const short what = EVBUFFER_WRITE | EVBUFFER_ERROR;
             dbgmsg( io, "tr_peerIoTryWrite got an error. res is %d, what is %hd, errno is %d (%s)", n, what, e, strerror( e ) );
-            io->gotError( io, what, io->userData );
+
+            if( io->gotError != NULL )
+                io->gotError( io, what, io->userData );
         }
     }
 
@@ -880,75 +995,3 @@ tr_peerIoFlushOutgoingProtocolMsgs( tr_peerIo * io )
 
     return tr_peerIoFlush( io, TR_UP, byteCount );
 }
-
-
-/***
-****
-****/
-
-static void
-event_enable( tr_peerIo * io, short event )
-{
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session != NULL );
-    assert( io->session->events != NULL );
-    assert( event_initialized( &io->event_read ) );
-    assert( event_initialized( &io->event_write ) );
-
-    if( ( event & EV_READ ) && ! ( io->pendingEvents & EV_READ ) )
-    {
-        dbgmsg( io, "enabling libevent ready-to-read polling" );
-        event_add( &io->event_read, NULL );
-        io->pendingEvents |= EV_READ;
-    }
-
-    if( ( event & EV_WRITE ) && ! ( io->pendingEvents & EV_WRITE ) )
-    {
-        dbgmsg( io, "enabling libevent ready-to-write polling" );
-        event_add( &io->event_write, NULL );
-        io->pendingEvents |= EV_WRITE;
-    }
-}
-
-static void
-event_disable( struct tr_peerIo * io, short event )
-{
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session != NULL );
-    assert( io->session->events != NULL );
-    assert( event_initialized( &io->event_read ) );
-    assert( event_initialized( &io->event_write ) );
-
-    if( ( event & EV_READ ) && ( io->pendingEvents & EV_READ ) )
-    {
-        dbgmsg( io, "disabling libevent ready-to-read polling" );
-        event_del( &io->event_read );
-        io->pendingEvents &= ~EV_READ;
-    }
-
-    if( ( event & EV_WRITE ) && ( io->pendingEvents & EV_WRITE ) )
-    {
-        dbgmsg( io, "disabling libevent ready-to-write polling" );
-        event_del( &io->event_write );
-        io->pendingEvents &= ~EV_WRITE;
-    }
-}
-
-
-void
-tr_peerIoSetEnabled( tr_peerIo    * io,
-                     tr_direction   dir,
-                     tr_bool        isEnabled )
-{
-    const short event = dir == TR_UP ? EV_WRITE : EV_READ;
-
-    assert( tr_isPeerIo( io ) );
-    assert( tr_isDirection( dir ) );
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session->events != NULL );
-
-    if( isEnabled )
-        event_enable( io, event );
-    else
-        event_disable( io, event );
-}
