@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-msgs.h 7455 2008-12-22 00:51:14Z charles $
+ * $Id: peer-msgs.h 10302 2010-03-06 14:56:15Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -25,6 +25,11 @@ struct tr_torrent;
 struct tr_peer;
 struct tr_bitfield;
 
+/**
+ * @addtogroup peers Peers
+ * @{
+ */
+
 typedef struct tr_peermsgs tr_peermsgs;
 
 tr_peermsgs* tr_peerMsgsNew( struct tr_torrent * torrent,
@@ -34,8 +39,9 @@ tr_peermsgs* tr_peerMsgsNew( struct tr_torrent * torrent,
                              tr_publisher_tag *  setme );
 
 
-void         tr_peerMsgsSetChoke(         tr_peermsgs *,
-                                      int doChoke );
+void         tr_peerMsgsSetChoke( tr_peermsgs *, int doChoke );
+
+int          tr_peerMsgsIsReadingBlock( const tr_peermsgs * msgs, tr_block_index_t block );
 
 void         tr_peerMsgsHave( tr_peermsgs * msgs,
                               uint32_t      pieceIndex );
@@ -43,18 +49,10 @@ void         tr_peerMsgsHave( tr_peermsgs * msgs,
 void         tr_peerMsgsPulse( tr_peermsgs * msgs );
 
 void         tr_peerMsgsCancel( tr_peermsgs * msgs,
-                                uint32_t      pieceIndex,
-                                uint32_t      offset,
-                                uint32_t      length );
-
+                                tr_block_index_t block );
 
 void         tr_peerMsgsFree( tr_peermsgs* );
 
-tr_addreq_t  tr_peerMsgsAddRequest( tr_peermsgs * peer,
-                                    uint32_t      pieceIndex,
-                                    uint32_t      offset,
-                                    uint32_t      length );
-
 void         tr_peerMsgsUnsubscribe( tr_peermsgs      * peer,
                                      tr_publisher_tag   tag );
 
@@ -65,4 +63,5 @@ size_t       tr_generateAllowedSet( tr_piece_index_t  * setmePieces,
                                     const tr_address  * addr );
 
 
+/* @} */
 #endif
