@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,24 +7,23 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: port-forwarding.c 8698 2009-06-15 03:24:40Z charles $
+ * $Id: port-forwarding.c 10184 2010-02-12 04:57:40Z charles $
  */
 
-#include <errno.h>
+#include <assert.h>
 #include <string.h>
 #include <stdio.h>
 
 #include <sys/types.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "natpmp.h"
 #include "net.h"
 #include "peer-io.h"
 #include "peer-mgr.h"
 #include "port-forwarding.h"
+#include "session.h"
 #include "torrent.h"
-#include "trevent.h"
 #include "upnp.h"
 #include "utils.h"
 
@@ -89,17 +88,9 @@ natPulse( tr_shared * s, tr_bool doPortCheck )
 }
 
 static void
-onTimer( int fd UNUSED, short what UNUSED, void * vshared )
+set_evtimer_from_status( tr_shared * s )
 {
     int sec=0, msec=0;
-    tr_shared * s = vshared;
-
-    assert( s );
-    assert( s->timer );
-
-    /* do something */
-    natPulse( s, s->doPortCheck );
-    s->doPortCheck = FALSE;
 
     /* when to wake up again */
     switch( tr_sharedTraversalStatus( s ) )
@@ -122,7 +113,24 @@ onTimer( int fd UNUSED, short what UNUSED, void * vshared )
             break;
     }
 
-    tr_timerAdd( s->timer, sec, msec );
+    if( s->timer != NULL )
+        tr_timerAdd( s->timer, sec, msec );
+}
+
+static void
+onTimer( int fd UNUSED, short what UNUSED, void * vshared )
+{
+    tr_shared * s = vshared;
+
+    assert( s );
+    assert( s->timer );
+
+    /* do something */
+    natPulse( s, s->doPortCheck );
+    s->doPortCheck = FALSE;
+
+    /* set up the timer for the next pulse */
+    set_evtimer_from_status( s );
 }
 
 /***
@@ -130,21 +138,23 @@ onTimer( int fd UNUSED, short what UNUSED, void * vshared )
 ***/
 
 tr_shared *
-tr_sharedInit( tr_session  * session, tr_bool isEnabled )
+tr_sharedInit( tr_session  * session )
 {
     tr_shared * s = tr_new0( tr_shared, 1 );
 
     s->session      = session;
-    s->isEnabled    = isEnabled;
+    s->isEnabled    = FALSE;
     s->upnpStatus   = TR_PORT_UNMAPPED;
     s->natpmpStatus = TR_PORT_UNMAPPED;
 
+#if 0
     if( isEnabled )
     {
         s->timer = tr_new0( struct event, 1 );
         evtimer_set( s->timer, onTimer, s );
         tr_timerAdd( s->timer, 0, 333000 );
     }
+#endif
 
     return s;
 }
@@ -163,12 +173,17 @@ stop_timer( tr_shared * s )
 static void
 stop_forwarding( tr_shared * s )
 {
-    tr_ninf( getKey( ), _( "Stopped" ) );
+    tr_ninf( getKey( ), "%s", _( "Stopped" ) );
     natPulse( s, FALSE );
+
     tr_natpmpClose( s->natpmp );
     s->natpmp = NULL;
+    s->natpmpStatus = TR_PORT_UNMAPPED;
+
     tr_upnpClose( s->upnp );
     s->upnp = NULL;
+    s->upnpStatus = TR_PORT_UNMAPPED;
+
     stop_timer( s );
 }
 
@@ -188,7 +203,7 @@ start_timer( tr_shared * s )
 {
     s->timer = tr_new0( struct event, 1 );
     evtimer_set( s->timer, onTimer, s );
-    onTimer( 0, 0, s );
+    set_evtimer_from_status( s );
 }
 
 void
@@ -208,6 +223,7 @@ tr_sharedPortChanged( tr_session * session )
     if( s->isEnabled )
     {
         stop_timer( s );
+        natPulse( s, FALSE );
         start_timer( s );
     }
 }
