@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: resume.c 9086 2009-09-10 02:21:03Z charles $
+ * $Id: resume.c 10084 2010-02-02 22:45:22Z charles $
  */
 
 #include <unistd.h> /* unlink */
@@ -17,7 +17,6 @@
 #include "transmission.h"
 #include "bencode.h"
 #include "completion.h"
-#include "fastresume.h"
 #include "peer-mgr.h" /* pex */
 #include "platform.h" /* tr_getResumeDir */
 #include "resume.h"
@@ -32,10 +31,11 @@
 #define KEY_DOWNLOAD_DIR        "destination"
 #define KEY_DND                 "dnd"
 #define KEY_DOWNLOADED          "downloaded"
+#define KEY_INCOMPLETE_DIR      "incomplete-dir"
 #define KEY_MAX_PEERS           "max-peers"
 #define KEY_PAUSED              "paused"
-#define KEY_PEERS               "peers"
-#define KEY_PEERS6              "peers6"
+#define KEY_PEERS               "peers2"
+#define KEY_PEERS6              "peers2-6"
 #define KEY_FILE_PRIORITIES     "priority"
 #define KEY_BANDWIDTH_PRIORITY  "bandwidth-priority"
 #define KEY_PROGRESS            "progress"
@@ -69,7 +69,7 @@ getResumeFilename( const tr_torrent * tor )
     return tr_strdup_printf( "%s%c%s.%16.16s.resume",
                              tr_getResumeDir( tor->session ),
                              TR_PATH_DELIMITER,
-                             tor->info.name,
+                             tr_torrentName( tor ),
                              tor->info.hashString );
 }
 
@@ -83,12 +83,12 @@ savePeers( tr_benc * dict, const tr_torrent * tor )
     int count;
     tr_pex * pex;
 
-    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET, MAX_REMEMBERED_PEERS );
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET, TR_PEERS_ALL, MAX_REMEMBERED_PEERS );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS, pex, sizeof( tr_pex ) * count );
     tr_free( pex );
 
-    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6, MAX_REMEMBERED_PEERS );
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6, TR_PEERS_ALL, MAX_REMEMBERED_PEERS );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS6, pex, sizeof( tr_pex ) * count );
 
@@ -115,7 +115,7 @@ addPeers( tr_torrent * tor, const uint8_t * buf, int buflen )
         memcpy( &pex, buf + ( i * sizeof( tr_pex ) ), sizeof( tr_pex ) );
         if( tr_isPex( &pex ) )
         {
-            tr_peerMgrAddPex( tor, TR_PEER_FROM_CACHE, &pex );
+            tr_peerMgrAddPex( tor, TR_PEER_FROM_RESUME, &pex );
             ++numAdded;
         }
     }
@@ -501,32 +501,28 @@ tr_torrentSaveResume( const tr_torrent * tor )
     if( !tr_isTorrent( tor ) )
         return;
 
-    tr_tordbg( tor, "Saving .resume file for \"%s\"", tor->info.name );
+    tr_tordbg( tor, "Saving .resume file for \"%s\"", tr_torrentName( tor ) );
 
-    tr_bencInitDict( &top, 32 ); /* arbitrary "big enough" number */
-    tr_bencDictAddInt( &top, KEY_ACTIVITY_DATE,
-                       tor->activityDate );
-    tr_bencDictAddInt( &top, KEY_ADDED_DATE,
-                       tor->addedDate );
-    tr_bencDictAddInt( &top, KEY_CORRUPT,
-                       tor->corruptPrev + tor->corruptCur );
-    tr_bencDictAddInt( &top, KEY_DONE_DATE,
-                       tor->doneDate );
-    tr_bencDictAddStr( &top, KEY_DOWNLOAD_DIR,
-                       tor->downloadDir );
-    tr_bencDictAddInt( &top, KEY_DOWNLOADED,
-                       tor->downloadedPrev + tor->downloadedCur );
-    tr_bencDictAddInt( &top, KEY_UPLOADED,
-                       tor->uploadedPrev + tor->uploadedCur );
-    tr_bencDictAddInt( &top, KEY_MAX_PEERS,
-                       tor->maxConnectedPeers );
-    tr_bencDictAddInt( &top, KEY_BANDWIDTH_PRIORITY,
-                       tr_torrentGetPriority( tor ) );
+    tr_bencInitDict( &top, 33 ); /* arbitrary "big enough" number */
+    tr_bencDictAddInt( &top, KEY_ACTIVITY_DATE, tor->activityDate );
+    tr_bencDictAddInt( &top, KEY_ADDED_DATE, tor->addedDate );
+    tr_bencDictAddInt( &top, KEY_CORRUPT, tor->corruptPrev + tor->corruptCur );
+    tr_bencDictAddInt( &top, KEY_DONE_DATE, tor->doneDate );
+    tr_bencDictAddStr( &top, KEY_DOWNLOAD_DIR, tor->downloadDir );
+    if( tor->incompleteDir != NULL )
+        tr_bencDictAddStr( &top, KEY_INCOMPLETE_DIR, tor->incompleteDir );
+    tr_bencDictAddInt( &top, KEY_DOWNLOADED, tor->downloadedPrev + tor->downloadedCur );
+    tr_bencDictAddInt( &top, KEY_UPLOADED, tor->uploadedPrev + tor->uploadedCur );
+    tr_bencDictAddInt( &top, KEY_MAX_PEERS, tor->maxConnectedPeers );
+    tr_bencDictAddInt( &top, KEY_BANDWIDTH_PRIORITY, tr_torrentGetPriority( tor ) );
     tr_bencDictAddBool( &top, KEY_PAUSED, !tor->isRunning );
     savePeers( &top, tor );
-    saveFilePriorities( &top, tor );
-    saveDND( &top, tor );
-    saveProgress( &top, tor );
+    if( tr_torrentHasMetadata( tor ) )
+    {
+        saveFilePriorities( &top, tor );
+        saveDND( &top, tor );
+        saveProgress( &top, tor );
+    }
     saveSpeedLimits( &top, tor );
     saveRatioLimits( &top, tor );
 
@@ -549,20 +545,13 @@ loadFromFile( tr_torrent * tor,
     tr_bool boolVal;
     const tr_bool  wasDirty = tor->isDirty;
 
+    assert( tr_isTorrent( tor ) );
+
     filename = getResumeFilename( tor );
 
     if( tr_bencLoadFile( &top, TR_FMT_BENC, filename ) )
     {
-        tr_tordbg( tor, "Couldn't read \"%s\"; trying old format.",
-                   filename );
-        fieldsLoaded = tr_fastResumeLoad( tor, fieldsToLoad );
-
-        if( ( fieldsLoaded != 0 ) && ( fieldsToLoad == ~(uint64_t)0 ) )
-        {
-            tr_torrentSaveResume( tor );
-            tr_fastResumeRemove( tor );
-            tr_tordbg( tor, "Migrated resume file to \"%s\"", filename );
-        }
+        tr_tordbg( tor, "Couldn't read \"%s\"", filename );
 
         tr_free( filename );
         return fieldsLoaded;
@@ -586,6 +575,15 @@ loadFromFile( tr_torrent * tor,
         fieldsLoaded |= TR_FR_DOWNLOAD_DIR;
     }
 
+    if( ( fieldsToLoad & ( TR_FR_PROGRESS | TR_FR_INCOMPLETE_DIR ) )
+      && ( tr_bencDictFindStr( &top, KEY_INCOMPLETE_DIR, &str ) )
+      && ( str && *str ) )
+    {
+        tr_free( tor->incompleteDir );
+        tor->incompleteDir = tr_strdup( str );
+        fieldsLoaded |= TR_FR_INCOMPLETE_DIR;
+    }
+
     if( ( fieldsToLoad & TR_FR_DOWNLOADED )
       && tr_bencDictFindInt( &top, KEY_DOWNLOADED, &i ) )
     {
@@ -730,6 +728,8 @@ tr_torrentLoadResume( tr_torrent *    tor,
 {
     uint64_t ret = 0;
 
+    assert( tr_isTorrent( tor ) );
+
     ret |= useManditoryFields( tor, fieldsToLoad, ctor );
     fieldsToLoad &= ~ret;
     ret |= loadFromFile( tor, fieldsToLoad );
@@ -743,9 +743,7 @@ void
 tr_torrentRemoveResume( const tr_torrent * tor )
 {
     char * filename = getResumeFilename( tor );
-
     unlink( filename );
-    tr_fastResumeRemove( tor );
     tr_free( filename );
 }
 
