@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpcimpl.c 8679 2009-06-12 12:10:09Z charles $
+ * $Id: rpcimpl.c 10288 2010-02-25 23:06:05Z livings124 $
  */
 
 #include <assert.h>
@@ -150,7 +150,7 @@ getTorrents( tr_session * session,
         if( !strcmp( str, "recently-active" ) )
         {
             tr_torrent * tor = NULL;
-            const time_t now = time( NULL );
+            const time_t now = tr_time( );
             const time_t window = RECENTLY_ACTIVE_SECONDS;
             const int n = tr_sessionCountTorrents( session );
             torrents = tr_new0( tr_torrent *, n );
@@ -183,7 +183,7 @@ static const char*
 torrentStart( tr_session               * session,
               tr_benc                  * args_in,
               tr_benc                  * args_out UNUSED,
-              struct tr_rpc_idle_data  * idle_data )
+              struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int           i, torrentCount;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
@@ -193,8 +193,11 @@ torrentStart( tr_session               * session,
     for( i = 0; i < torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
-        tr_torrentStart( tor );
-        notify( session, TR_RPC_TORRENT_STARTED, tor );
+        if( !tor->isRunning )
+        {
+            tr_torrentStart( tor );
+            notify( session, TR_RPC_TORRENT_STARTED, tor );
+        }
     }
     tr_free( torrents );
     return NULL;
@@ -204,7 +207,7 @@ static const char*
 torrentStop( tr_session               * session,
              tr_benc                  * args_in,
              tr_benc                  * args_out UNUSED,
-             struct tr_rpc_idle_data  * idle_data )
+             struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int           i, torrentCount;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
@@ -214,8 +217,11 @@ torrentStop( tr_session               * session,
     for( i = 0; i < torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
-        tr_torrentStop( tor );
-        notify( session, TR_RPC_TORRENT_STOPPED, tor );
+        if( tor->isRunning )
+        {
+            tr_torrentStop( tor );
+            notify( session, TR_RPC_TORRENT_STOPPED, tor );
+        }
     }
     tr_free( torrents );
     return NULL;
@@ -225,7 +231,7 @@ static const char*
 torrentRemove( tr_session               * session,
                tr_benc                  * args_in,
                tr_benc                  * args_out UNUSED,
-               struct tr_rpc_idle_data  * idle_data )
+               struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int i;
     int torrentCount;
@@ -252,7 +258,7 @@ static const char*
 torrentReannounce( tr_session               * session,
                    tr_benc                  * args_in,
                    tr_benc                  * args_out UNUSED,
-                   struct tr_rpc_idle_data  * idle_data )
+                   struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int i, torrentCount;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
@@ -277,7 +283,7 @@ static const char*
 torrentVerify( tr_session               * session,
                tr_benc                  * args_in,
                tr_benc                  * args_out UNUSED,
-               struct tr_rpc_idle_data  * idle_data )
+               struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int           i, torrentCount;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
@@ -359,13 +365,51 @@ addTrackers( const tr_info * info,
     for( i = 0; i < info->trackerCount; ++i )
     {
         const tr_tracker_info * t = &info->trackers[i];
-        tr_benc *               d = tr_bencListAddDict( trackers, 3 );
+        tr_benc *               d = tr_bencListAddDict( trackers, 4 );
         tr_bencDictAddStr( d, "announce", t->announce );
+        tr_bencDictAddInt( d, "id", t->id );
         tr_bencDictAddStr( d, "scrape", t->scrape );
         tr_bencDictAddInt( d, "tier", t->tier );
     }
 }
 
+static void
+addTrackerStats( const tr_tracker_stat * st, int n, tr_benc * list )
+{
+    int i;
+
+    for( i=0; i<n; ++i )
+    {
+        const tr_tracker_stat * s = &st[i];
+        tr_benc * d = tr_bencListAddDict( list, 25 );
+        tr_bencDictAddStr ( d, "announce", s->announce );
+        tr_bencDictAddInt ( d, "announceState", s->announceState );
+        tr_bencDictAddInt ( d, "downloadCount", s->downloadCount );
+        tr_bencDictAddBool( d, "hasAnnounced", s->hasAnnounced );
+        tr_bencDictAddBool( d, "hasScraped", s->hasScraped );
+        tr_bencDictAddStr ( d, "host", s->host );
+        tr_bencDictAddInt ( d, "id", s->id );
+        tr_bencDictAddBool( d, "isBackup", s->isBackup );
+        tr_bencDictAddInt ( d, "lastAnnouncePeerCount", s->lastAnnouncePeerCount );
+        tr_bencDictAddStr ( d, "lastAnnounceResult", s->lastAnnounceResult );
+        tr_bencDictAddInt ( d, "lastAnnounceStartTime", s->lastAnnounceStartTime );
+        tr_bencDictAddBool( d, "lastAnnounceSucceeded", s->lastAnnounceSucceeded );
+        tr_bencDictAddInt ( d, "lastAnnounceTime", s->lastAnnounceTime );
+        tr_bencDictAddBool( d, "lastAnnounceTimedOut", s->lastAnnounceTimedOut );
+        tr_bencDictAddStr ( d, "lastScrapeResult", s->lastScrapeResult );
+        tr_bencDictAddInt ( d, "lastScrapeStartTime", s->lastScrapeStartTime );
+        tr_bencDictAddBool( d, "lastScrapeSucceeded", s->lastScrapeSucceeded );
+        tr_bencDictAddInt ( d, "lastScrapeTime", s->lastScrapeTime );
+        tr_bencDictAddInt ( d, "lastScrapeTimedOut", s->lastScrapeTimedOut );
+        tr_bencDictAddInt ( d, "leecherCount", s->leecherCount );
+        tr_bencDictAddInt ( d, "nextAnnounceTime", s->nextAnnounceTime );
+        tr_bencDictAddInt ( d, "nextScrapeTime", s->nextScrapeTime );
+        tr_bencDictAddInt ( d, "scrapeState", s->scrapeState );
+        tr_bencDictAddInt ( d, "seederCount", s->seederCount );
+        tr_bencDictAddInt ( d, "tier", s->tier );
+    }
+}
+
 static void
 addPeers( const tr_torrent * tor,
           tr_benc *          list )
@@ -416,10 +460,6 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, st->activityDate );
     else if( tr_streq( key, keylen, "addedDate" ) )
         tr_bencDictAddInt( d, key, st->addedDate );
-    else if( tr_streq( key, keylen, "announceResponse" ) )
-        tr_bencDictAddStr( d, key, st->announceResponse );
-    else if( tr_streq( key, keylen, "announceURL" ) )
-        tr_bencDictAddStr( d, key, st->announceURL );
     else if( tr_streq( key, keylen, "bandwidthPriority" ) )
         tr_bencDictAddInt( d, key, tr_torrentGetPriority( tor ) );
     else if( tr_streq( key, keylen, "comment" ) )
@@ -438,8 +478,6 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddStr( d, key, tr_torrentGetDownloadDir( tor ) );
     else if( tr_streq( key, keylen, "downloadedEver" ) )
         tr_bencDictAddInt( d, key, st->downloadedEver );
-    else if( tr_streq( key, keylen, "downloaders" ) )
-        tr_bencDictAddInt( d, key, st->downloaders );
     else if( tr_streq( key, keylen, "downloadLimit" ) )
         tr_bencDictAddInt( d, key, tr_torrentGetSpeedLimit( tor, TR_DOWN ) );
     else if( tr_streq( key, keylen, "downloadLimited" ) )
@@ -466,24 +504,21 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, st->id );
     else if( tr_streq( key, keylen, "isPrivate" ) )
         tr_bencDictAddBool( d, key, tr_torrentIsPrivate( tor ) );
-    else if( tr_streq( key, keylen, "lastAnnounceTime" ) )
-        tr_bencDictAddInt( d, key, st->lastAnnounceTime );
-    else if( tr_streq( key, keylen, "lastScrapeTime" ) )
-        tr_bencDictAddInt( d, key, st->lastScrapeTime );
-    else if( tr_streq( key, keylen, "leechers" ) )
-        tr_bencDictAddInt( d, key, st->leechers );
     else if( tr_streq( key, keylen, "leftUntilDone" ) )
         tr_bencDictAddInt( d, key, st->leftUntilDone );
     else if( tr_streq( key, keylen, "manualAnnounceTime" ) )
         tr_bencDictAddInt( d, key, st->manualAnnounceTime );
     else if( tr_streq( key, keylen, "maxConnectedPeers" ) )
         tr_bencDictAddInt( d, key,  tr_torrentGetPeerLimit( tor ) );
+    else if( tr_streq( key, keylen, "magnetLink" ) ) {
+        char * str = tr_torrentGetMagnetLink( tor );
+        tr_bencDictAddStr( d, key, str );
+        tr_free( str );
+    }
+    else if( tr_streq( key, keylen, "metadataPercentComplete" ) )
+        tr_bencDictAddReal( d, key, st->metadataPercentComplete );
     else if( tr_streq( key, keylen, "name" ) )
         tr_bencDictAddStr( d, key, inf->name );
-    else if( tr_streq( key, keylen, "nextAnnounceTime" ) )
-        tr_bencDictAddInt( d, key, st->nextAnnounceTime );
-    else if( tr_streq( key, keylen, "nextScrapeTime" ) )
-        tr_bencDictAddInt( d, key, st->nextScrapeTime );
     else if( tr_streq( key, keylen, "percentDone" ) )
         tr_bencDictAddReal( d, key, st->percentDone );
     else if( tr_streq( key, keylen, "peer-limit" ) )
@@ -494,10 +529,12 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, st->peersConnected );
     else if( tr_streq( key, keylen, "peersFrom" ) )
     {
-        tr_benc *   tmp = tr_bencDictAddDict( d, key, 4 );
+        tr_benc *   tmp = tr_bencDictAddDict( d, key, 6 );
         const int * f = st->peersFrom;
-        tr_bencDictAddInt( tmp, "fromCache",    f[TR_PEER_FROM_CACHE] );
+        tr_bencDictAddInt( tmp, "fromCache",    f[TR_PEER_FROM_RESUME] );
+        tr_bencDictAddInt( tmp, "fromDht",      f[TR_PEER_FROM_DHT] );
         tr_bencDictAddInt( tmp, "fromIncoming", f[TR_PEER_FROM_INCOMING] );
+        tr_bencDictAddInt( tmp, "fromLtep",     f[TR_PEER_FROM_LTEP] );
         tr_bencDictAddInt( tmp, "fromPex",      f[TR_PEER_FROM_PEX] );
         tr_bencDictAddInt( tmp, "fromTracker",  f[TR_PEER_FROM_TRACKER] );
     }
@@ -510,7 +547,7 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
     else if( tr_streq( key, keylen, "pieces" ) ) {
         const tr_bitfield * pieces = tr_cpPieceBitfield( &tor->completion );
         char * str = tr_base64_encode( pieces->bits, pieces->byteCount, NULL );
-        tr_bencDictAddStr( d, key, str );
+        tr_bencDictAddStr( d, key, str!=NULL ? str : "" );
         tr_free( str );
     }
     else if( tr_streq( key, keylen, "pieceCount" ) )
@@ -530,12 +567,6 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, (int)( st->pieceUploadSpeed * 1024 ) );
     else if( tr_streq( key, keylen, "recheckProgress" ) )
         tr_bencDictAddReal( d, key, st->recheckProgress );
-    else if( tr_streq( key, keylen, "scrapeResponse" ) )
-        tr_bencDictAddStr( d, key, st->scrapeResponse );
-    else if( tr_streq( key, keylen, "scrapeURL" ) )
-        tr_bencDictAddStr( d, key, st->scrapeURL );
-    else if( tr_streq( key, keylen, "seeders" ) )
-        tr_bencDictAddInt( d, key, st->seeders );
     else if( tr_streq( key, keylen, "seedRatioLimit" ) )
         tr_bencDictAddReal( d, key, tr_torrentGetRatioLimit( tor ) );
     else if( tr_streq( key, keylen, "seedRatioMode" ) )
@@ -546,12 +577,14 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, st->startDate );
     else if( tr_streq( key, keylen, "status" ) )
         tr_bencDictAddInt( d, key, st->activity );
-    else if( tr_streq( key, keylen, "swarmSpeed" ) )
-        tr_bencDictAddInt( d, key, (int)( st->swarmSpeed * 1024 ) );
-    else if( tr_streq( key, keylen, "timesCompleted" ) )
-        tr_bencDictAddInt( d, key, st->timesCompleted );
     else if( tr_streq( key, keylen, "trackers" ) )
         addTrackers( inf, tr_bencDictAddList( d, key, inf->trackerCount ) );
+    else if( tr_streq( key, keylen, "trackerStats" ) ) {
+        int n;
+        tr_tracker_stat * s = tr_torrentTrackers( tor, &n );
+        addTrackerStats( s, n, tr_bencDictAddList( d, key, n ) );
+        tr_torrentTrackersFree( s, n );
+    }
     else if( tr_streq( key, keylen, "torrentFile" ) )
         tr_bencDictAddStr( d, key, inf->torrent );
     else if( tr_streq( key, keylen, "totalSize" ) )
@@ -597,7 +630,7 @@ static const char*
 torrentGet( tr_session               * session,
             tr_benc                  * args_in,
             tr_benc                  * args_out,
-            struct tr_rpc_idle_data  * idle_data )
+            struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int           i, torrentCount;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
@@ -611,7 +644,7 @@ torrentGet( tr_session               * session,
     if( tr_bencDictFindStr( args_in, "ids", &strVal ) && !strcmp( strVal, "recently-active" ) ) {
         int n = 0;
         tr_benc * d;
-        const time_t now = time( NULL );
+        const time_t now = tr_time( );
         const int interval = RECENTLY_ACTIVE_SECONDS;
         tr_benc * removed_out = tr_bencDictAddList( args_out, "removed", 0 );
         while(( d = tr_bencListChild( &session->removedTorrents, n++ ))) {
@@ -626,7 +659,7 @@ torrentGet( tr_session               * session,
     if( !tr_bencDictFindList( args_in, "fields", &fields ) )
         msg = "no fields specified";
     else for( i = 0; i < torrentCount; ++i )
-            addInfo( torrents[i], tr_bencListAdd( list ), fields );
+        addInfo( torrents[i], tr_bencListAdd( list ), fields );
 
     tr_free( torrents );
     return msg;
@@ -716,7 +749,7 @@ static const char*
 torrentSet( tr_session               * session,
             tr_benc                  * args_in,
             tr_benc                  * args_out UNUSED,
-            struct tr_rpc_idle_data  * idle_data )
+            struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     const char * errmsg = NULL;
     int i, torrentCount;
@@ -735,10 +768,10 @@ torrentSet( tr_session               * session,
         if( tr_bencDictFindInt( args_in, "bandwidthPriority", &tmp ) )
             if( tr_isPriority( tmp ) )
                 tr_torrentSetPriority( tor, tmp );
-        if( tr_bencDictFindList( args_in, "files-unwanted", &files ) )
-            setFileDLs( tor, FALSE, files );
-        if( tr_bencDictFindList( args_in, "files-wanted", &files ) )
-            setFileDLs( tor, TRUE, files );
+        if( !errmsg && tr_bencDictFindList( args_in, "files-unwanted", &files ) )
+            errmsg = setFileDLs( tor, FALSE, files );
+        if( !errmsg && tr_bencDictFindList( args_in, "files-wanted", &files ) )
+            errmsg = setFileDLs( tor, TRUE, files );
         if( tr_bencDictFindInt( args_in, "peer-limit", &tmp ) )
             tr_torrentSetPeerLimit( tor, tmp );
         if( !errmsg &&  tr_bencDictFindList( args_in, "priority-high", &files ) )
@@ -772,7 +805,7 @@ static const char*
 torrentSetLocation( tr_session               * session,
                     tr_benc                  * args_in,
                     tr_benc                  * args_out UNUSED,
-                    struct tr_rpc_idle_data  * idle_data )
+                    struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     const char * errmsg = NULL;
     const char * location = NULL;
@@ -795,7 +828,7 @@ torrentSetLocation( tr_session               * session,
         {
             tr_torrent * tor = torrents[i];
             tr_torrentSetLocation( tor, location, move, NULL, NULL );
-            notify( session, TR_RPC_TORRENT_CHANGED, tor );
+            notify( session, TR_RPC_TORRENT_MOVED, tor );
         }
 
         tr_free( torrents );
@@ -926,11 +959,11 @@ addTorrentImpl( struct tr_rpc_idle_data * data, tr_ctor * ctor )
         notify( data->session, TR_RPC_TORRENT_ADDED, tor );
         tr_bencFree( &fields );
     }
-    else if( err == TR_EDUPLICATE )
+    else if( err == TR_PARSE_DUPLICATE )
     {
         result = "duplicate torrent";
     }
-    else if( err == TR_EINVALID )
+    else if( err == TR_PARSE_ERR )
     {
         result = "invalid or corrupt torrent file";
     }
@@ -957,7 +990,7 @@ gotMetadataFromURL( tr_session       * session UNUSED,
     dbgmsg( "torrentAdd: HTTP response code was %ld (%s); response length was %zu bytes",
             response_code, tr_webGetResponseStr( response_code ), response_byte_count );
 
-    if( response_code == 200 )
+    if( response_code==200 || response_code==221 ) /* http or ftp success.. */
     {
         tr_ctorSetMetainfo( data->ctor, response, response_byte_count );
         addTorrentImpl( data->data, data->ctor );
@@ -979,9 +1012,9 @@ isCurlURL( const char * filename )
     if( filename == NULL )
         return FALSE;
 
-    return ( strstr( filename, "ftp://" ) != NULL )
-        || ( strstr( filename, "http://" ) != NULL )
-        || ( strstr( filename, "https://" ) != NULL );
+    return !strncmp( filename, "ftp://", 6 ) ||
+           !strncmp( filename, "http://", 7 ) ||
+           !strncmp( filename, "https://", 8 );
 }
 
 static tr_file_index_t*
@@ -1027,7 +1060,7 @@ torrentAdd( tr_session               * session,
 
         /* set the optional arguments */
 
-        if( tr_bencDictFindStr( args_in, "download-dir", &str ) )
+        if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_DOWNLOAD_DIR, &str ) )
             tr_ctorSetDownloadDir( ctor, TR_FORCE, str );
 
         if( tr_bencDictFindBool( args_in, "paused", &boolVal ) )
@@ -1036,6 +1069,9 @@ torrentAdd( tr_session               * session,
         if( tr_bencDictFindInt( args_in, "peer-limit", &i ) )
             tr_ctorSetPeerLimit( ctor, TR_FORCE, i );
 
+        if( tr_bencDictFindInt( args_in, "bandwidthPriority", &i ) )
+            tr_ctorSetBandwidthPriority( ctor, i );
+
         if( tr_bencDictFindList( args_in, "files-unwanted", &l ) ) {
             tr_file_index_t fileCount;
             tr_file_index_t * files = fileListFromList( l, &fileCount );
@@ -1068,7 +1104,7 @@ torrentAdd( tr_session               * session,
             tr_free( files );
         }
 
-        dbgmsg( "torrentAdd: filename is \"%s\"", filename );
+        dbgmsg( "torrentAdd: filename is \"%s\"", filename ? filename : "(null)" );
 
         if( isCurlURL( filename ) )
         {
@@ -1079,15 +1115,27 @@ torrentAdd( tr_session               * session,
         }
         else
         {
-            if( filename != NULL )
-                tr_ctorSetMetainfoFromFile( ctor, filename );
-            else {
+            char * fname = tr_strstrip( tr_strdup( filename ) );
+
+            if( fname == NULL )
+            {
                 int len;
-                char * metainfo = tr_base64_decode( metainfo_base64, -1,  &len );
+                char * metainfo = tr_base64_decode( metainfo_base64, -1, &len );
                 tr_ctorSetMetainfo( ctor, (uint8_t*)metainfo, len );
                 tr_free( metainfo );
             }
+            else if( !strncmp( fname, "magnet:?", 8 ) )
+            {
+                tr_ctorSetMetainfoFromMagnetLink( ctor, fname );
+            }
+            else
+            {
+                tr_ctorSetMetainfoFromFile( ctor, fname );
+            }
+
             addTorrentImpl( idle_data, ctor );
+
+            tr_free( fname );
         }
 
     }
@@ -1103,7 +1151,7 @@ static const char*
 sessionSet( tr_session               * session,
             tr_benc                  * args_in,
             tr_benc                  * args_out UNUSED,
-            struct tr_rpc_idle_data  * idle_data )
+            struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int64_t      i;
     double       d;
@@ -1130,6 +1178,10 @@ sessionSet( tr_session               * session,
         tr_blocklistSetEnabled( session, boolVal );
     if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_DOWNLOAD_DIR, &str ) )
         tr_sessionSetDownloadDir( session, str );
+    if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_INCOMPLETE_DIR, &str ) )
+        tr_sessionSetIncompleteDir( session, str );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, &boolVal ) )
+        tr_sessionSetIncompleteDirEnabled( session, boolVal );
     if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_PEER_LIMIT_GLOBAL, &i ) )
         tr_sessionSetPeerLimit( session, i );
     if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_PEER_LIMIT_TORRENT, &i ) )
@@ -1144,19 +1196,21 @@ sessionSet( tr_session               * session,
         tr_sessionSetPeerPort( session, i );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_PORT_FORWARDING, &boolVal ) )
         tr_sessionSetPortForwardingEnabled( session, boolVal );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_RENAME_PARTIAL_FILES, &boolVal ) )
+        tr_sessionSetIncompleteFileNamingEnabled( session, boolVal );
     if( tr_bencDictFindReal( args_in, "seedRatioLimit", &d ) )
         tr_sessionSetRatioLimit( session, d );
     if( tr_bencDictFindBool( args_in, "seedRatioLimited", &boolVal ) )
         tr_sessionSetRatioLimited( session, boolVal );
-    if( tr_bencDictFindInt( args_in, "speed-limit-down", &i ) )
+    if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_DSPEED, &i ) )
         tr_sessionSetSpeedLimit( session, TR_DOWN, i );
-    if( tr_bencDictFindBool( args_in, "speed-limit-down-enabled", &boolVal ) )
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_DSPEED_ENABLED, &boolVal ) )
         tr_sessionLimitSpeed( session, TR_DOWN, boolVal );
-    if( tr_bencDictFindInt( args_in, "speed-limit-up", &i ) )
+    if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_USPEED, &i ) )
         tr_sessionSetSpeedLimit( session, TR_UP, i );
-    if( tr_bencDictFindBool( args_in, "speed-limit-up-enabled", &boolVal ) )
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_USPEED_ENABLED, &boolVal ) )
         tr_sessionLimitSpeed( session, TR_UP, boolVal );
-    if( tr_bencDictFindStr( args_in, "encryption", &str ) ) {
+    if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_ENCRYPTION, &str ) ) {
         if( !strcmp( str, "required" ) )
             tr_sessionSetEncryption( session, TR_ENCRYPTION_REQUIRED );
         else if( !strcmp( str, "tolerated" ) )
@@ -1174,13 +1228,13 @@ static const char*
 sessionStats( tr_session               * session,
               tr_benc                  * args_in UNUSED,
               tr_benc                  * args_out,
-              struct tr_rpc_idle_data  * idle_data )
+              struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     int running = 0;
     int total = 0;
-    tr_benc * d;  
-    tr_session_stats currentStats = { 0.0f, 0, 0, 0, 0, 0 }; 
-    tr_session_stats cumulativeStats = { 0.0f, 0, 0, 0, 0, 0 }; 
+    tr_benc * d;
+    tr_session_stats currentStats = { 0.0f, 0, 0, 0, 0, 0 };
+    tr_session_stats cumulativeStats = { 0.0f, 0, 0, 0, 0, 0 };
     tr_torrent * tor = NULL;
 
     assert( idle_data == NULL );
@@ -1191,8 +1245,8 @@ sessionStats( tr_session               * session,
             ++running;
     }
 
-    tr_sessionGetStats( session, &currentStats ); 
-    tr_sessionGetCumulativeStats( session, &cumulativeStats ); 
+    tr_sessionGetStats( session, &currentStats );
+    tr_sessionGetCumulativeStats( session, &cumulativeStats );
 
     tr_bencDictAddInt( args_out, "activeTorrentCount", running );
     tr_bencDictAddInt( args_out, "downloadSpeed", (int)( tr_sessionGetPieceSpeed( session, TR_DOWN ) * 1024 ) );
@@ -1200,19 +1254,19 @@ sessionStats( tr_session               * session,
     tr_bencDictAddInt( args_out, "torrentCount", total );
     tr_bencDictAddInt( args_out, "uploadSpeed", (int)( tr_sessionGetPieceSpeed( session, TR_UP ) * 1024 ) );
 
-    d = tr_bencDictAddDict( args_out, "cumulative-stats", 5 );  
-    tr_bencDictAddInt( d, "downloadedBytes", cumulativeStats.downloadedBytes ); 
-    tr_bencDictAddInt( d, "filesAdded", cumulativeStats.filesAdded ); 
-    tr_bencDictAddInt( d, "secondsActive", cumulativeStats.secondsActive ); 
-    tr_bencDictAddInt( d, "sessionCount", cumulativeStats.sessionCount ); 
-    tr_bencDictAddInt( d, "uploadedBytes", cumulativeStats.uploadedBytes ); 
+    d = tr_bencDictAddDict( args_out, "cumulative-stats", 5 );
+    tr_bencDictAddInt( d, "downloadedBytes", cumulativeStats.downloadedBytes );
+    tr_bencDictAddInt( d, "filesAdded", cumulativeStats.filesAdded );
+    tr_bencDictAddInt( d, "secondsActive", cumulativeStats.secondsActive );
+    tr_bencDictAddInt( d, "sessionCount", cumulativeStats.sessionCount );
+    tr_bencDictAddInt( d, "uploadedBytes", cumulativeStats.uploadedBytes );
 
-    d = tr_bencDictAddDict( args_out, "current-stats", 5 );  
-    tr_bencDictAddInt( d, "downloadedBytes", currentStats.downloadedBytes ); 
-    tr_bencDictAddInt( d, "filesAdded", currentStats.filesAdded ); 
-    tr_bencDictAddInt( d, "secondsActive", currentStats.secondsActive ); 
-    tr_bencDictAddInt( d, "sessionCount", currentStats.sessionCount ); 
-    tr_bencDictAddInt( d, "uploadedBytes", currentStats.uploadedBytes ); 
+    d = tr_bencDictAddDict( args_out, "current-stats", 5 );
+    tr_bencDictAddInt( d, "downloadedBytes", currentStats.downloadedBytes );
+    tr_bencDictAddInt( d, "filesAdded", currentStats.filesAdded );
+    tr_bencDictAddInt( d, "secondsActive", currentStats.secondsActive );
+    tr_bencDictAddInt( d, "sessionCount", currentStats.sessionCount );
+    tr_bencDictAddInt( d, "uploadedBytes", currentStats.uploadedBytes );
 
     return NULL;
 }
@@ -1221,7 +1275,7 @@ static const char*
 sessionGet( tr_session               * s,
             tr_benc                  * args_in UNUSED,
             tr_benc                  * args_out,
-            struct tr_rpc_idle_data  * idle_data )
+            struct tr_rpc_idle_data  * idle_data UNUSED )
 {
     const char * str;
     tr_benc *    d = args_out;
@@ -1236,29 +1290,33 @@ sessionGet( tr_session               * s,
     tr_bencDictAddBool( d, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED, tr_sessionUsesAltSpeedTime(s) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_BLOCKLIST_ENABLED, tr_blocklistIsEnabled( s ) );
     tr_bencDictAddInt ( d, "blocklist-size", tr_blocklistGetRuleCount( s ) );
+    tr_bencDictAddStr ( d, "config-dir", tr_sessionGetConfigDir( s ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_DOWNLOAD_DIR, tr_sessionGetDownloadDir( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_LIMIT_GLOBAL, tr_sessionGetPeerLimit( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_LIMIT_TORRENT, tr_sessionGetPeerLimitPerTorrent( s ) );
+    tr_bencDictAddStr ( d, TR_PREFS_KEY_INCOMPLETE_DIR, tr_sessionGetIncompleteDir( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, tr_sessionIsIncompleteDirEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PEX_ENABLED, tr_sessionIsPexEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DHT_ENABLED, tr_sessionIsDHTEnabled( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_PORT, tr_sessionGetPeerPort( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START, tr_sessionGetPeerPortRandomOnStart( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PORT_FORWARDING, tr_sessionIsPortForwardingEnabled( s ) );
-    tr_bencDictAddInt ( d, "rpc-version", 6 );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_RENAME_PARTIAL_FILES, tr_sessionIsIncompleteFileNamingEnabled( s ) );
+    tr_bencDictAddInt ( d, "rpc-version", 8 );
     tr_bencDictAddInt ( d, "rpc-version-minimum", 1 );
     tr_bencDictAddReal( d, "seedRatioLimit", tr_sessionGetRatioLimit( s ) );
     tr_bencDictAddBool( d, "seedRatioLimited", tr_sessionIsRatioLimited( s ) );
-    tr_bencDictAddInt ( d, "speed-limit-up", tr_sessionGetSpeedLimit( s, TR_UP ) );
-    tr_bencDictAddBool( d, "speed-limit-up-enabled", tr_sessionIsSpeedLimited( s, TR_UP ) );
-    tr_bencDictAddInt ( d, "speed-limit-down", tr_sessionGetSpeedLimit( s, TR_DOWN ) );
-    tr_bencDictAddBool( d, "speed-limit-down-enabled", tr_sessionIsSpeedLimited( s, TR_DOWN ) );
+    tr_bencDictAddInt ( d, TR_PREFS_KEY_USPEED, tr_sessionGetSpeedLimit( s, TR_UP ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_USPEED_ENABLED, tr_sessionIsSpeedLimited( s, TR_UP ) );
+    tr_bencDictAddInt ( d, TR_PREFS_KEY_DSPEED, tr_sessionGetSpeedLimit( s, TR_DOWN ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED, tr_sessionIsSpeedLimited( s, TR_DOWN ) );
     tr_bencDictAddStr ( d, "version", LONG_VERSION_STRING );
     switch( tr_sessionGetEncryption( s ) ) {
         case TR_CLEAR_PREFERRED: str = "tolerated"; break;
-        case TR_ENCRYPTION_REQUIRED: str = "required"; break; 
+        case TR_ENCRYPTION_REQUIRED: str = "required"; break;
         default: str = "preferred"; break;
     }
-    tr_bencDictAddStr( d, "encryption", str );
+    tr_bencDictAddStr( d, TR_PREFS_KEY_ENCRYPTION, str );
 
     return NULL;
 }
