@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,12 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-ctor.c 8602 2009-06-03 16:59:32Z charles $
+ * $Id: torrent-ctor.c 10084 2010-02-02 22:45:22Z charles $
  */
 
-#include <errno.h>
+#include <errno.h> /* EINVAL */
 #include "transmission.h"
 #include "bencode.h"
+#include "magnet.h"
 #include "platform.h"
 #include "session.h" /* tr_sessionFindTorrentFile() */
 #include "torrent.h" /* tr_ctorGetSave() */
@@ -37,6 +38,7 @@ struct tr_ctor
     tr_bool                 saveInOurTorrentsDir;
     tr_bool                 doDelete;
 
+    tr_priority_t           bandwidthPriority;
     tr_bool                 isSet_metainfo;
     tr_bool                 isSet_delete;
     tr_benc                 metainfo;
@@ -44,6 +46,8 @@ struct tr_ctor
 
     struct optional_args    optionalArgs[2];
 
+    char                  * incompleteDir;
+
     tr_file_index_t       * want;
     tr_file_index_t         wantSize;
     tr_file_index_t       * notWant;
@@ -99,6 +103,30 @@ tr_ctorGetSourceFile( const tr_ctor * ctor )
     return ctor->sourceFile;
 }
 
+int
+tr_ctorSetMetainfoFromMagnetLink( tr_ctor * ctor, const char * magnet_link )
+{
+    int err;
+    tr_magnet_info * magnet_info = tr_magnetParse( magnet_link );
+
+    if( magnet_info == NULL )
+        err = -1;
+    else {
+        int len;
+        tr_benc tmp;
+        char * str;
+
+        tr_magnetCreateMetainfo( magnet_info, &tmp );
+        str = tr_bencToStr( &tmp, TR_FMT_BENC, &len );
+        err = tr_ctorSetMetainfo( ctor, (const uint8_t*)str, len );
+
+        tr_free( str );
+        tr_magnetFree( magnet_info );
+    }
+
+    return err;
+}
+
 int
 tr_ctorSetMetainfoFromFile( tr_ctor *    ctor,
                             const char * filename )
@@ -300,12 +328,19 @@ tr_ctorSetDownloadDir( tr_ctor *    ctor,
     }
 }
 
+void
+tr_ctorSetIncompleteDir( tr_ctor * ctor, const char * directory )
+{
+    tr_free( ctor->incompleteDir );
+    ctor->incompleteDir = tr_strdup( directory );
+}
+
 int
 tr_ctorGetPeerLimit( const tr_ctor * ctor,
                      tr_ctorMode     mode,
                      uint16_t *      setmeCount )
 {
-    int                          err = 0;
+    int err = 0;
     const struct optional_args * args = &ctor->optionalArgs[mode];
 
     if( !args->isSet_connected )
@@ -348,6 +383,20 @@ tr_ctorGetDownloadDir( const tr_ctor * ctor,
     return err;
 }
 
+int
+tr_ctorGetIncompleteDir( const tr_ctor  * ctor,
+                         const char    ** setmeIncompleteDir )
+{
+    int err = 0;
+
+    if( ctor->incompleteDir == NULL )
+        err = 1;
+    else
+        *setmeIncompleteDir = ctor->incompleteDir;
+
+    return err;
+}
+
 int
 tr_ctorGetMetainfo( const tr_ctor *  ctor,
                     const tr_benc ** setme )
@@ -372,12 +421,37 @@ tr_ctorGetSession( const tr_ctor * ctor )
 ****
 ***/
 
+static tr_bool
+isPriority( int i )
+{
+    return (i==TR_PRI_LOW) || (i==TR_PRI_NORMAL) || (i==TR_PRI_HIGH);
+}
+
+void
+tr_ctorSetBandwidthPriority( tr_ctor * ctor, tr_priority_t priority )
+{
+fprintf( stderr, "in tr_ctorSetPriority with %d\n", (int)priority );
+    if( isPriority( priority ) )
+        ctor->bandwidthPriority = priority;
+}
+
+tr_priority_t
+tr_ctorGetBandwidthPriority( const tr_ctor * ctor )
+{
+    return ctor->bandwidthPriority;
+}
+
+/***
+****
+***/
+
 tr_ctor*
 tr_ctorNew( const tr_session * session )
 {
     tr_ctor * ctor = tr_new0( struct tr_ctor, 1 );
 
     ctor->session = session;
+    ctor->bandwidthPriority = TR_PRI_NORMAL;
     tr_ctorSetPaused( ctor, TR_FALLBACK, FALSE );
     if( session != NULL ) {
         tr_ctorSetPeerLimit( ctor, TR_FALLBACK, session->peerLimitPerTorrent );
@@ -393,6 +467,7 @@ tr_ctorFree( tr_ctor * ctor )
     clearMetainfo( ctor );
     tr_free( ctor->optionalArgs[1].downloadDir );
     tr_free( ctor->optionalArgs[0].downloadDir );
+    tr_free( ctor->incompleteDir );
     tr_free( ctor->want );
     tr_free( ctor->notWant );
     tr_free( ctor->low );
