@@ -5,9 +5,11 @@
 #include "bitfield.h"
 #include "ConvertUTF.h" /* tr_utf8_validate*/
 #include "platform.h"
-#include "utils.h"
 #include "crypto.h"
+#include "utils.h"
+#include "web.h"
 
+/* #define VERBOSE */
 #undef VERBOSE
 #define NUM_LOOPS 1
 #define SPEED_TEST 0
@@ -227,6 +229,43 @@ test_numbers( void )
     return 0;
 }
 
+static int
+compareInts( const void * va, const void * vb )
+{
+    const int a = *(const int *)va;
+    const int b = *(const int*)vb;
+    return a - b;
+}
+
+static int
+test_lowerbound( void )
+{
+    int i;
+    const int A[] = { 1, 2, 3, 3, 3, 5, 8 };
+    const int expected_pos[] = { 0, 1, 2, 5, 5, 6, 6, 6, 7, 7 };
+    const int expected_exact[] = { TRUE, TRUE, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE };
+    const int N = sizeof(A) / sizeof(A[0]);
+
+    for( i=1; i<=10; ++i )
+    {
+        tr_bool exact;
+        const int pos = tr_lowerBound( &i, A, N, sizeof(int), compareInts, &exact );
+
+#if 0
+        fprintf( stderr, "searching for %d.  ", i );
+        fprintf( stderr, "result: index = %d, ", pos );
+        if( pos != N )
+            fprintf( stderr, "A[%d] == %d\n", pos, A[pos] );
+        else
+            fprintf( stderr, "which is off the end.\n" );
+#endif
+        check( pos == expected_pos[i-1] )
+        check( exact == expected_exact[i-1] )
+    }
+
+    return 0;
+}
+
 static int
 test_memmem( void )
 {
@@ -243,14 +282,97 @@ test_memmem( void )
     return 0;
 }
 
+static int
+test_hex( void )
+{
+    char hex1[41];
+    char hex2[41];
+    uint8_t sha1[20];
+    /*uint8_t sha2[20];*/
+
+    memcpy( hex1, "fb5ef5507427b17e04b69cef31fa3379b456735a", 41 );
+    tr_hex_to_sha1( sha1, hex1 );
+    tr_sha1_to_hex( hex2, sha1 );
+    check( !strcmp( hex1, hex2 ) )
+
+    return 0;
+}
+
+static int
+test_array( void )
+{
+    int i;
+    int array[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
+    int n = sizeof( array ) / sizeof( array[0] );
+
+    tr_removeElementFromArray( array, 5, sizeof( int ), n-- );
+    for( i=0; i<n; ++i )
+        check( array[i] == ( i<5 ? i : i+1 ) );
+
+    tr_removeElementFromArray( array, 0, sizeof( int ), n-- );
+    for( i=0; i<n; ++i )
+        check( array[i] == ( i<4 ? i+1 : i+2 ) );
+
+    tr_removeElementFromArray( array, n-1, sizeof( int ), n ); n--;
+    for( i=0; i<n; ++i )
+        check( array[i] == ( i<4 ? i+1 : i+2 ) );
+
+    return 0;
+}
+
+static int
+test_url( void )
+{
+    int port;
+    char * scheme;
+    char * host;
+    char * path;
+    char * str;
+    const char * url;
+
+    url = "http://www.some-tracker.org/some/path";
+    check( !tr_urlParse( url, -1, &scheme, &host, &port, &path ) )
+    check( !strcmp( scheme, "http" ) )
+    check( !strcmp( host, "www.some-tracker.org" ) )
+    check( !strcmp( path, "/some/path" ) )
+    check( port == 80 )
+    tr_free( scheme );
+    tr_free( path );
+    tr_free( host );
+
+    url = "http://www.some-tracker.org:80/some/path";
+    check( !tr_urlParse( url, -1, &scheme, &host, &port, &path ) )
+    check( !strcmp( scheme, "http" ) )
+    check( !strcmp( host, "www.some-tracker.org" ) )
+    check( !strcmp( path, "/some/path" ) )
+    check( port == 80 )
+    tr_free( scheme );
+    tr_free( path );
+    tr_free( host );
+
+    url = "http%3A%2F%2Fwww.example.com%2F~user%2F%3Ftest%3D1%26test1%3D2";
+    str = tr_http_unescape( url, strlen( url ) );
+    check( !strcmp( str, "http://www.example.com/~user/?test=1&test1=2" ) )
+    tr_free( str );
+
+    return 0;
+}
+
 int
 main( void )
 {
+    char buf[32];
     char *in, *out;
     int   len;
     int   i;
     int   l;
 
+    /* tr_truncd */
+    tr_snprintf( buf, sizeof( buf ), "%.2f%%", 99.999 );
+    check( !strcmp( buf, "100.00%" ) );
+    tr_snprintf( buf, sizeof( buf ), "%.2f%%", tr_truncd( 99.999, 2 ) );
+    check( !strcmp( buf, "99.99%" ) );
+
     /* base64 */
     out = tr_base64_encode( "YOYO!", -1, &len );
     check( out );
@@ -262,7 +384,14 @@ main( void )
     check( len == 5 );
     tr_free( in );
     tr_free( out );
+    out = tr_base64_encode( NULL, 0, &len );
+    check( out == NULL );
+    check( len == 0 );
 
+    if( ( i = test_hex( ) ) )
+        return i;
+    if( ( i = test_lowerbound( ) ) )
+        return i;
     if( ( i = test_strstrip( ) ) )
         return i;
     if( ( i = test_buildpath( ) ) )
@@ -273,6 +402,10 @@ main( void )
         return i;
     if( ( i = test_memmem( ) ) )
         return i;
+    if( ( i = test_array( ) ) )
+        return i;
+    if( ( i = test_url( ) ) )
+        return i;
 
     /* test that tr_cryptoRandInt() stays in-bounds */
     for( i = 0; i < 100000; ++i )
