@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: verify.c 8390 2009-05-13 18:37:56Z charles $
+ * $Id: verify.c 10274 2010-02-23 06:06:45Z charles $
  */
 
 #include <unistd.h> /* S_ISREG */
@@ -15,7 +15,9 @@
 
 #ifdef HAVE_POSIX_FADVISE
  #define _XOPEN_SOURCE 600
- #include <fcntl.h> /* posix_fadvise() */
+#endif
+#if defined(HAVE_POSIX_FADVISE) || defined(SYS_DARWIN)
+ #include <fcntl.h> /* posix_fadvise() / fcntl() */
 #endif
 
 #include <openssl/sha.h>
@@ -23,7 +25,6 @@
 #include "transmission.h"
 #include "completion.h"
 #include "fdlimit.h"
-#include "resume.h" /* tr_torrentSaveResume() */
 #include "inout.h"
 #include "list.h"
 #include "platform.h"
@@ -36,6 +37,11 @@
 ****
 ***/
 
+enum
+{
+    MSEC_TO_SLEEP_PER_SECOND_DURING_VERIFY = 200
+};
+
 /* #define STOPWATCH */
 
 static tr_bool
@@ -46,14 +52,16 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
     int64_t filePos = 0;
     tr_bool changed = 0;
     tr_bool hadPiece = 0;
+    time_t lastSleptAt = 0;
     uint32_t piecePos = 0;
+    uint32_t pieceBytesRead = 0;
     tr_file_index_t fileIndex = 0;
+    tr_file_index_t prevFileIndex = !fileIndex;
     tr_piece_index_t pieceIndex = 0;
-    const int64_t buflen = tor->info.pieceSize;
-    uint8_t * buffer = tr_new( uint8_t, buflen );
-#ifdef STOPWATCH
-    time_t now = time( NULL );
-#endif
+    const time_t begin = tr_time( );
+    time_t end;
+    const int64_t buflen = 16384;
+    uint8_t * buffer = tr_valloc( buflen );
 
     SHA1_Init( &sha );
 
@@ -72,12 +80,13 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
         }
 
         /* if we're starting a new file... */
-        if( !filePos && (fd<0) )
+        if( !filePos && (fd<0) && (fileIndex!=prevFileIndex) )
         {
-            char * filename = tr_buildPath( tor->downloadDir, file->name, NULL );
-            fd = tr_open_file_for_scanning( filename );
+            char * filename = tr_torrentFindFile( tor, fileIndex );
+            fd = filename == NULL ? -1 : tr_open_file_for_scanning( filename );
             /* fprintf( stderr, "opening file #%d (%s) -- %d\n", fileIndex, filename, fd ); */
             tr_free( filename );
+            prevFileIndex = fileIndex;
         }
 
         /* figure out how much we can read this pass */
@@ -88,10 +97,16 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
         /* fprintf( stderr, "reading this pass: %d\n", (int)bytesThisPass ); */
 
         /* read a bit */
-        if( (fd>=0) && tr_lseek( fd, filePos, SEEK_SET ) != -1 ) {
-            const int64_t numRead = read( fd, buffer, bytesThisPass );
+        if( fd >= 0 ) {
+            const ssize_t numRead = tr_pread( fd, buffer, bytesThisPass, filePos );
             if( numRead == bytesThisPass )
                 SHA1_Update( &sha, buffer, numRead );
+            if( numRead > 0 ) {
+                pieceBytesRead += numRead;
+#if defined HAVE_POSIX_FADVISE && defined POSIX_FADV_DONTNEED
+                posix_fadvise( fd, filePos, bytesThisPass, POSIX_FADV_DONTNEED );
+#endif
+            }
         }
 
         /* move our offsets */
@@ -103,6 +118,7 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
         /* if we're finishing a piece... */
         if( leftInPiece == 0 )
         {
+            time_t now;
             tr_bool hasPiece;
             uint8_t hash[SHA_DIGEST_LENGTH];
 
@@ -119,10 +135,20 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
                 changed = TRUE;
             }
             tr_torrentSetPieceChecked( tor, pieceIndex, TRUE );
+            now = tr_time( );
+            tor->anyDate = now;
+
+            /* sleeping even just a few msec per second goes a long
+             * way towards reducing IO load... */
+            if( lastSleptAt != now ) {
+                lastSleptAt = now;
+                tr_wait_msec( MSEC_TO_SLEEP_PER_SECOND_DURING_VERIFY );
+            }
 
             SHA1_Init( &sha );
             ++pieceIndex;
             piecePos = 0;
+            pieceBytesRead = 0;
         }
 
         /* if we're finishing a file... */
@@ -138,15 +164,12 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
     /* cleanup */
     if( fd >= 0 )
         tr_close_file( fd );
-    tr_free( buffer );
+    free( buffer );
 
-#ifdef STOPWATCH
-{
-    time_t now2 = time( NULL );
-    fprintf( stderr, "it took %d seconds to verify %"PRIu64" bytes (%"PRIu64" bytes per second)\n",
-             (int)(now2-now), tor->info.totalSize, (uint64_t)(tor->info.totalSize/(1+(now2-now))) );
-}
-#endif
+    /* stopwatch */
+    end = tr_time( );
+    tr_tordbg( tor, "it took %d seconds to verify %"PRIu64" bytes (%"PRIu64" bytes per second)",
+               (int)(end-begin), tor->info.totalSize, (uint64_t)(tor->info.totalSize/(1+(end-begin))) );
 
     return changed;
 }
@@ -209,7 +232,7 @@ verifyThreadFunc( void * unused UNUSED )
         tr_free( node );
         tr_lockUnlock( getVerifyLock( ) );
 
-        tr_torinf( tor, _( "Verifying torrent" ) );
+        tr_torinf( tor, "%s", _( "Verifying torrent" ) );
         tr_torrentSetVerifyState( tor, TR_VERIFY_NOW );
         changed = verifyTorrent( tor, &stopCurrent );
         tr_torrentSetVerifyState( tor, TR_VERIFY_NONE );
@@ -218,7 +241,7 @@ verifyThreadFunc( void * unused UNUSED )
         if( !stopCurrent )
         {
             if( changed )
-                tr_torrentSaveResume( tor );
+                tr_torrentSetDirty( tor );
             fireCheckDone( tor, currentNode.verify_done_cb );
         }
     }
@@ -227,24 +250,59 @@ verifyThreadFunc( void * unused UNUSED )
     tr_lockUnlock( getVerifyLock( ) );
 }
 
+static tr_bool
+torrentHasAnyLocalData( const tr_torrent * tor )
+{
+    tr_file_index_t i;
+    tr_bool hasAny = FALSE;
+    const tr_file_index_t n = tor->info.fileCount;
+
+    assert( tr_isTorrent( tor ) );
+
+    for( i=0; i<n && !hasAny; ++i )
+    {
+        struct stat sb;
+        char * path = tr_torrentFindFile( tor, i );
+        if( ( path != NULL ) && !stat( path, &sb ) && ( sb.st_size > 0 ) )
+            hasAny = TRUE;
+        tr_free( path );
+    }
+
+    return hasAny;
+}
+
 void
 tr_verifyAdd( tr_torrent *      tor,
               tr_verify_done_cb verify_done_cb )
 {
-    const int uncheckedCount = tr_torrentCountUncheckedPieces( tor );
-
     assert( tr_isTorrent( tor ) );
 
-    if( !uncheckedCount )
+    if( tr_torrentCountUncheckedPieces( tor ) == 0 )
     {
         /* doesn't need to be checked... */
         fireCheckDone( tor, verify_done_cb );
     }
+    else if( !torrentHasAnyLocalData( tor ) )
+    {
+        /* we haven't downloaded anything for this torrent yet...
+         * no need to leave it waiting in the back of the queue.
+         * we can mark it as all-missing from here and fire
+         * the "done" callback */
+        const tr_bool hadAny = tr_cpHaveTotal( &tor->completion ) != 0;
+        tr_piece_index_t i;
+        for( i=0; i<tor->info.pieceCount; ++i ) {
+            tr_torrentSetHasPiece( tor, i, FALSE );
+            tr_torrentSetPieceChecked( tor, i, TRUE );
+        }
+        if( hadAny ) /* if we thought we had some, flag as dirty */
+            tr_torrentSetDirty( tor );
+        fireCheckDone( tor, verify_done_cb );
+    }
     else
     {
         struct verify_node * node;
 
-        tr_torinf( tor, _( "Queued for verification" ) );
+        tr_torinf( tor, "%s", _( "Queued for verification" ) );
 
         node = tr_new( struct verify_node, 1 );
         node->torrent = tor;
@@ -299,14 +357,14 @@ tr_verifyRemove( tr_torrent * tor )
         while( stopCurrent )
         {
             tr_lockUnlock( lock );
-            tr_wait( 100 );
+            tr_wait_msec( 100 );
             tr_lockLock( lock );
         }
     }
     else
     {
         tr_free( tr_list_remove( &verifyList, tor, compareVerifyByTorrent ) );
-        tor->verifyState = TR_VERIFY_NONE;
+        tr_torrentSetVerifyState( tor, TR_VERIFY_NONE );
     }
 
     tr_lockUnlock( lock );
