@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Controller.h 8386 2009-05-12 00:58:22Z livings124 $
+ * $Id: Controller.h 10073 2010-02-02 01:47:22Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,18 +24,19 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
-#import "PrefsController.h"
-#import "InfoWindowController.h"
-#import "MessageWindowController.h"
-#import "AddWindowController.h"
-#import "DragOverlayWindow.h"
-#import "Badger.h"
-
+#import <Quartz/Quartz.h>
 #import <Growl/Growl.h>
 
-@class TorrentTableView;
-@class StatusBarView;
+@class AddWindowController;
+@class Badger;
+@class DragOverlayWindow;
 @class FilterButton;
+@class InfoWindowController;
+@class MessageWindowController;
+@class PrefsController;
+@class StatusBarView;
+@class Torrent;
+@class TorrentTableView;
 
 typedef enum
 {
@@ -46,9 +47,10 @@ typedef enum
     ADD_CREATED
 } addType;
 
-@interface Controller : NSObject <GrowlApplicationBridgeDelegate>
+#warning uncomment
+@interface Controller : NSObject <GrowlApplicationBridgeDelegate> //, QLPreviewPanelDataSource, QLPreviewPanelDelegate>
 {
-    tr_session                       * fLib;
+    tr_session                      * fLib;
     
     NSMutableArray                  * fTorrents, * fDisplayedTorrents;
     
@@ -96,6 +98,10 @@ typedef enum
     IBOutlet NSTextField            * fURLSheetTextField;
     IBOutlet NSButton               * fURLSheetOpenButton;
     
+    #warning change to QLPreviewPanel
+    id                              fPreviewPanel;
+    BOOL                            fQuitting;
+    
     BOOL                            fUpdateInProgress;
     BOOL                            fPauseOnLaunch;
     
@@ -110,16 +116,20 @@ typedef enum
     BOOL                            fSoundPlaying;
 }
 
-- (void) openFiles:             (NSArray *) filenames addType: (addType) type forcePath: (NSString *) path;
-- (void) askOpenConfirmed:      (AddWindowController *) addController add: (BOOL) add;
-- (void) openCreatedFile:       (NSNotification *) notification;
-- (void) openFilesWithDict:     (NSDictionary *) dictionary;
-- (void) openShowSheet:         (id) sender;
+- (void) openFiles: (NSArray *) filenames addType: (addType) type forcePath: (NSString *) path;
+- (void) openMagnet: (NSString *) address;
+
+- (void) askOpenConfirmed: (AddWindowController *) addController add: (BOOL) add;
+- (void) openCreatedFile: (NSNotification *) notification;
+- (void) openFilesWithDict: (NSDictionary *) dictionary;
+- (void) openShowSheet: (id) sender;
 
 - (void) invalidOpenAlert: (NSString *) filename;
+- (void) invalidOpenMagnetAlert: (NSString *) address;
 - (void) duplicateOpenAlert: (NSString *) name;
+- (void) duplicateOpenMagnetAlert: (NSString *) address transferName: (NSString *) name;
 
-- (void) openURL:               (NSURL *) torrentURL;
+- (void) openURL:               (NSString *) urlString;
 - (void) openURLEndSheet:       (id) sender;
 - (void) openURLCancelEndSheet: (id) sender;
 - (void) openURLShowSheet:      (id) sender;
@@ -140,15 +150,12 @@ typedef enum
 - (void) stopAllTorrents:           (id) sender;
 - (void) stopTorrents:              (NSArray *) torrents;
 
-- (void) removeTorrents: (NSArray *) torrents
-        deleteData: (BOOL) deleteData deleteTorrent: (BOOL) deleteData;
+- (void) removeTorrents: (NSArray *) torrents deleteData: (BOOL) deleteData;
 - (void) removeSheetDidEnd: (NSWindow *) sheet returnCode: (NSInteger) returnCode
                         contextInfo: (NSDictionary *) dict;
-- (void) confirmRemoveTorrents: (NSArray *) torrents deleteData: (BOOL) deleteData deleteTorrent: (BOOL) deleteTorrent;
+- (void) confirmRemoveTorrents: (NSArray *) torrents deleteData: (BOOL) deleteData;
 - (void) removeNoDelete:                (id) sender;
 - (void) removeDeleteData:              (id) sender;
-- (void) removeDeleteTorrent:           (id) sender;
-- (void) removeDeleteDataAndTorrent:    (id) sender;
 
 - (void) moveDataFilesSelected: (id) sender;
 - (void) moveDataFiles: (NSArray *) torrents;
@@ -157,6 +164,8 @@ typedef enum
 - (void) copyTorrentFiles: (id) sender;
 - (void) copyTorrentFileForTorrents: (NSMutableArray *) torrents;
 
+- (void) copyMagnetLinks: (id) sender;
+
 - (void) revealFile: (id) sender;
 
 - (void) announceSelectedTorrents: (id) sender;
@@ -183,7 +192,7 @@ typedef enum
 - (void) updateSpeedFieldsToolTips;
 
 - (void) updateTorrentsInQueue;
-- (NSInteger) numToStartFromQueue: (BOOL) downloadQueue;
+- (NSUInteger) numToStartFromQueue: (BOOL) downloadQueue;
 
 - (void) torrentFinishedDownloading: (NSNotification *) notification;
 - (void) torrentRestartedDownloading: (NSNotification *) notification;
@@ -246,14 +255,12 @@ typedef enum
 - (void) setWindowSizeToFit;
 - (NSRect) sizedWindowFrame;
 
+- (void) resizeFilterBar;
+
 - (void) updateForExpandCollape;
 
 - (void) showMainWindow: (id) sender;
 
-- (NSArray *) quickLookURLs;
-- (BOOL) canQuickLook;
-- (BOOL) canQuickLookTorrent: (Torrent *) torrent;
-- (NSRect) quickLookFrameWithURL: (NSURL*) url;
 - (void) toggleQuickLook: (id) sender;
 
 - (void) linkHomepage: (id) sender;
@@ -266,5 +273,6 @@ typedef enum
 - (void) rpcRemoveTorrent: (Torrent *) torrent;
 - (void) rpcStartedStoppedTorrent: (Torrent *) torrent;
 - (void) rpcChangedTorrent: (Torrent *) torrent;
+- (void) rpcMovedTorrent: (Torrent *) torrent;
 
 @end
