@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: CreatorWindowController.h 6038 2008-06-04 20:55:38Z livings124 $
+ * $Id: CreatorWindowController.h 9844 2010-01-01 21:12:04Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,7 +41,6 @@
     tr_metainfo_builder * fInfo;
     NSString * fPath, * fLocation;
     NSMutableArray * fTrackers;
-    BOOL fOpenTorrent;
     
     NSTimer * fTimer;
     BOOL fStarted;
@@ -49,12 +48,10 @@
     NSUserDefaults * fDefaults;
 }
 
-+ (void) createTorrentFile: (tr_handle *) handle;
-+ (void) createTorrentFile: (tr_handle *) handle forFile: (NSString *) file;
++ (void) createTorrentFile: (tr_session *) handle;
++ (void) createTorrentFile: (tr_session *) handle forFile: (NSString *) file;
 
-- (id) initWithHandle: (tr_handle *) handle path: (NSString *) path;
-
-- (void) toggleOpenCheck: (id) sender;
+- (id) initWithHandle: (tr_session *) handle path: (NSString *) path;
 
 - (void) setLocation: (id) sender;
 - (void) create: (id) sender;
