@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: DragOverlayWindow.m 7666 2009-01-11 18:04:45Z livings124 $
+ * $Id: DragOverlayWindow.m 10097 2010-02-05 02:51:42Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,17 +26,23 @@
 #import "DragOverlayView.h"
 #import "NSStringAdditions.h"
 
+@interface DragOverlayWindow (Private)
+
+- (void) resizeWindow;
+
+@end
+
 @implementation DragOverlayWindow
 
 - (id) initWithLib: (tr_session *) lib forWindow: (NSWindow *) window
 {
-    if ((self = ([super initWithContentRect: NSMakeRect(0, 0, 1.0, 1.0) styleMask: NSBorderlessWindowMask
+    if ((self = ([super initWithContentRect: [window frame] styleMask: NSBorderlessWindowMask
                     backing: NSBackingStoreBuffered defer: NO])))
     {
         fLib = lib;
         
-        [self setBackgroundColor: [NSColor colorWithCalibratedWhite: 0.0f alpha: 0.5f]];
-        [self setAlphaValue: 0.0f];
+        [self setBackgroundColor: [NSColor colorWithCalibratedWhite: 0.0 alpha: 0.5]];
+        [self setAlphaValue: 0.0];
         [self setOpaque: NO];
         [self setHasShadow: NO];
         
@@ -60,12 +66,17 @@
         [fFadeOutAnimation setAnimationBlockingMode: NSAnimationNonblockingThreaded];
         
         [window addChildWindow: self ordered: NSWindowAbove];
+        
+        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(resizeWindow)
+            name: NSWindowDidResizeNotification object: window];
     }
     return self;
 }
 
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fFadeInAnimation release];
     [fFadeOutAnimation release];
     
@@ -81,16 +92,15 @@
     BOOL folder;
     NSInteger fileCount = 0;
     
-    NSString * file;
-    NSEnumerator * enumerator = [files objectEnumerator];
-    while ((file = [enumerator nextObject]))
+    for (NSString * file in files)
     {
-        if ([[file pathExtension] caseInsensitiveCompare: @"torrent"] == NSOrderedSame)
+        if ([[[NSWorkspace sharedWorkspace] typeOfFile: file error: NULL] isEqualToString: @"org.bittorrent.torrent"]
+            || [[file pathExtension] caseInsensitiveCompare: @"torrent"] == NSOrderedSame)
         {
             tr_ctor * ctor = tr_ctorNew(fLib);
             tr_ctorSetMetainfoFromFile(ctor, [file UTF8String]);
             tr_info info;
-            if (tr_torrentParse(fLib, ctor, &info) == TR_OK)
+            if (tr_torrentParse(ctor, &info) == TR_PARSE_OK)
             {
                 count++;
                 size += info.totalSize;
@@ -159,7 +169,6 @@
         [fFadeOutAnimation stopAnimation];
         [fFadeInAnimation setCurrentProgress: 1.0 - [fFadeOutAnimation currentProgress]];
     }
-    [self setFrame: [[self parentWindow] frame] display: YES];
     [fFadeInAnimation startAnimation];
 }
 
@@ -171,8 +180,17 @@
         [fFadeInAnimation stopAnimation];
         [fFadeOutAnimation setCurrentProgress: 1.0 - [fFadeInAnimation currentProgress]];
     }
-    if ([self alphaValue] > 0.0f)
+    if ([self alphaValue] > 0.0)
         [fFadeOutAnimation startAnimation];
 }
 
 @end
+
+@implementation DragOverlayWindow (Private)
+
+- (void) resizeWindow
+{
+    [self setFrame: [[self parentWindow] frame] display: NO];
+}
+
+@end
