@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineView.m 7018 2008-11-02 14:04:57Z livings124 $
+ * $Id: FileOutlineView.m 10206 2010-02-15 14:56:14Z livings124 $
  * 
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,13 +22,14 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-#import "FileOutlineView.h"
+#import "InfoWindowController.h"
+#import "FileListNode.h"
 #import "FileNameCell.h"
+#import "FileOutlineView.h"
 #import "FilePriorityCell.h"
 #import "Torrent.h"
-#import "FileListNode.h"
-#import "QuickLookController.h"
-#import "CTGradient.h"
+#import "NSApplicationAdditions.h"
+#import <Quartz/Quartz.h>
 
 @implementation FileOutlineView
 
@@ -47,15 +48,15 @@
     
     NSColor * endingColor = [NSColor colorWithCalibratedRed: 217.0/255.0 green: 250.0/255.0 blue: 211.0/255.0 alpha: 1.0];
     NSColor * beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fHighPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fHighPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     endingColor = [NSColor colorWithCalibratedRed: 255.0/255.0 green: 243.0/255.0 blue: 206.0/255.0 alpha: 1.0];
     beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fLowPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fLowPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     endingColor = [NSColor colorWithCalibratedRed: 225.0/255.0 green: 218.0/255.0 blue: 255.0/255.0 alpha: 1.0];
     beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fMixedPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fMixedPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     fMouseRow = -1;
 }
@@ -87,21 +88,24 @@
 
 - (void) keyDown: (NSEvent *) event
 {
-    unichar firstChar = [[event charactersIgnoringModifiers] characterAtIndex: 0];
-    if (firstChar == ' ')
-        [[QuickLookController quickLook] toggleQuickLook];
-    else if (firstChar == NSRightArrowFunctionKey)
-        [[QuickLookController quickLook] pressRight];
-    else if (firstChar == NSLeftArrowFunctionKey)
-        [[QuickLookController quickLook] pressLeft];
-    else;
+    const unichar firstChar = [[event charactersIgnoringModifiers] characterAtIndex: 0];
+    
+    //don't allow quick look on add window
+    #warning get rid of conversion of protocol and replace with @protocol(QLPreviewPanelDataSource)
+    if ([NSApp isOnSnowLeopardOrBetter] && firstChar == ' ' && [[[self window] windowController] conformsToProtocol: NSProtocolFromString(@"QLPreviewPanelDataSource")])
+    {
+        if ([[QLPreviewPanelSL sharedPreviewPanel] isVisible])
+            [[QLPreviewPanelSL sharedPreviewPanel] orderOut: nil];
+        else
+            [[QLPreviewPanelSL sharedPreviewPanel] makeKeyAndOrderFront: nil];
+    }
     
     [super keyDown: event];  
 }
 
 - (NSMenu *) menuForEvent: (NSEvent *) event
 {
-    int row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
+    const NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     
     if (row >= 0)
     {
@@ -119,7 +123,7 @@
     FileNameCell * cell = (FileNameCell *)[self preparedCellAtColumn: [self columnWithIdentifier: @"Name"] row: row];
     NSRect iconRect = [cell imageRectForBounds: [self rectOfRow: row]];
     
-    iconRect.origin.x += [self indentationPerLevel] * (float)([self levelForRow: row] + 1);
+    iconRect.origin.x += [self indentationPerLevel] * (CGFloat)([self levelForRow: row] + 1);
     return iconRect;
 }
 
@@ -127,9 +131,7 @@
 {
     [super updateTrackingAreas];
     
-    NSEnumerator * enumerator = [[self trackingAreas] objectEnumerator];
-    NSTrackingArea * area;
-    while ((area = [enumerator nextObject]))
+    for (NSTrackingArea * area in [self trackingAreas])
     {
         if ([area owner] == self && [[area userInfo] objectForKey: @"Row"])
             [self removeTrackingArea: area];
@@ -151,7 +153,7 @@
     }
 }
 
-- (int) hoveredRow
+- (NSInteger) hoveredRow
 {
     return fMouseRow;
 }
@@ -185,10 +187,10 @@
         
         if ([fTorrent checkForFiles: indexes] != NSOffState)
         {
-            CTGradient * gradient = nil;
+            NSGradient * gradient = nil;
             
             NSSet * priorities = [fTorrent filePrioritiesForIndexes: indexes];
-            int count = [priorities count];
+            const NSUInteger count = [priorities count];
             if (count == 1)
             {
                 switch ([[priorities anyObject] intValue])
@@ -209,7 +211,7 @@
             {
                 NSRect rect = [self rectOfRow: row];
                 rect.size.height -= 1.0;
-                [gradient fillRect: rect angle: 90];
+                [gradient drawInRect: rect angle: 90];
             }
         }
     }
