@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: MessageWindowController.m 7802 2009-01-26 02:16:03Z livings124 $
+ * $Id: MessageWindowController.m 10236 2010-02-20 14:29:33Z livings124 $
  *
- * Copyright (c) 2006-2009 Transmission authors and contributors
+ * Copyright (c) 2006-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,19 +23,21 @@
  *****************************************************************************/
 
 #import "MessageWindowController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import <transmission.h>
+#import <utils.h>
 
 #define LEVEL_ERROR 0
 #define LEVEL_INFO  1
 #define LEVEL_DEBUG 2
 
-#define UPDATE_SECONDS  0.6
-#define MAX_MESSAGES    4000
+#define UPDATE_SECONDS  0.75
 
 @interface MessageWindowController (Private)
 
 - (void) resizeColumn;
+- (BOOL) shouldIncludeMessageForFilter: (NSString *) filterString message: (NSDictionary *) message;
 - (NSString *) stringForMessage: (NSDictionary *) message;
 
 @end
@@ -49,8 +51,13 @@
 
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fTimer invalidate];
+    [fLock release];
+    
     [fMessages release];
+    [fDisplayedMessages release];
     
     [fAttributes release];
     
@@ -66,27 +73,23 @@
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(resizeColumn)
         name: @"NSTableViewColumnDidResizeNotification" object: fMessageTable];
     
-    [window setContentBorderThickness: [[fMessageTable enclosingScrollView] frame].origin.y forEdge: NSMinYEdge];
+    [window setContentBorderThickness: NSMinY([[fMessageTable enclosingScrollView] frame]) forEdge: NSMinYEdge];
     
     //initially sort peer table by date
     if ([[fMessageTable sortDescriptors] count] == 0)
         [fMessageTable setSortDescriptors: [NSArray arrayWithObject: [[fMessageTable tableColumnWithIdentifier: @"Date"]
                                             sortDescriptorPrototype]]];
     
-    fErrorImage = [NSImage imageNamed: @"RedDot.png"];
-    fInfoImage = [NSImage imageNamed: @"YellowDot.png"];
-    fDebugImage = [NSImage imageNamed: @"PurpleDot.png"];
-    
     [[self window] setTitle: NSLocalizedString(@"Message Log", "Message window -> title")];
     
     //set images and text for popup button items
-    [[fLevelButton itemAtIndex: LEVEL_ERROR] setImage: fErrorImage];
     [[fLevelButton itemAtIndex: LEVEL_ERROR] setTitle: NSLocalizedString(@"Error", "Message window -> level string")];
-    [[fLevelButton itemAtIndex: LEVEL_INFO] setImage: fInfoImage];
     [[fLevelButton itemAtIndex: LEVEL_INFO] setTitle: NSLocalizedString(@"Info", "Message window -> level string")];
-    [[fLevelButton itemAtIndex: LEVEL_DEBUG] setImage: fDebugImage];
     [[fLevelButton itemAtIndex: LEVEL_DEBUG] setTitle: NSLocalizedString(@"Debug", "Message window -> level string")];
     
+    const CGFloat levelButtonOldWidth = NSWidth([fLevelButton frame]);
+    [fLevelButton sizeToFit];
+    
     //set table column text
     [[[fMessageTable tableColumnWithIdentifier: @"Date"] headerCell] setTitle: NSLocalizedString(@"Date",
         "Message window -> table column")];
@@ -101,20 +104,29 @@
     
     NSRect saveButtonFrame = [fSaveButton frame];
     saveButtonFrame.size.width += 10.0;
+    saveButtonFrame.origin.x += NSWidth([fLevelButton frame]) - levelButtonOldWidth;
     [fSaveButton setFrame: saveButtonFrame];
     
-    float oldClearButtonWidth = [fClearButton frame].size.width;
+    const CGFloat oldClearButtonWidth = [fClearButton frame].size.width;
     
     [fClearButton setTitle: NSLocalizedString(@"Clear", "Message window -> save button")];
     [fClearButton sizeToFit];
     
     NSRect clearButtonFrame = [fClearButton frame];
     clearButtonFrame.size.width = MAX(clearButtonFrame.size.width + 10.0, saveButtonFrame.size.width);
-    clearButtonFrame.origin.x -= clearButtonFrame.size.width - oldClearButtonWidth;
+    clearButtonFrame.origin.x -= NSWidth(clearButtonFrame) - oldClearButtonWidth;
     [fClearButton setFrame: clearButtonFrame];
     
+    [[fFilterField cell] setPlaceholderString: NSLocalizedString(@"Filter", "Message window -> filter field")];
+    NSRect filterButtonFrame = [fFilterField frame];
+    filterButtonFrame.origin.x -= NSWidth(clearButtonFrame) - oldClearButtonWidth;
+    [fFilterField setFrame: filterButtonFrame];
+    
+    fAttributes = [[[[[fMessageTable tableColumnWithIdentifier: @"Message"] dataCell] attributedStringValue]
+                    attributesAtIndex: 0 effectiveRange: NULL] retain];
+    
     //select proper level in popup button
-    switch (tr_getMessageLevel())
+    switch ([[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"])
     {
         case TR_MSG_ERR:
             [fLevelButton selectItemAtIndex: LEVEL_ERROR];
@@ -124,18 +136,26 @@
             break;
         case TR_MSG_DBG:
             [fLevelButton selectItemAtIndex: LEVEL_DEBUG];
+            break;
+        default: //safety
+            [[NSUserDefaults standardUserDefaults] setInteger: TR_MSG_ERR forKey: @"MessageLevel"];
+            [fLevelButton selectItemAtIndex: LEVEL_ERROR];
     }
     
     fMessages = [[NSMutableArray alloc] init];
-    fIndex = 0;
+    fDisplayedMessages = [[NSMutableArray alloc] init];
+    
+    fLock = [[NSLock alloc] init];
 }
 
 - (void) windowDidBecomeKey: (NSNotification *) notification
 {
     if (!fTimer)
+    {
         fTimer = [NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: self
                     selector: @selector(updateLog:) userInfo: nil repeats: YES];
-    [self updateLog: nil];
+        [self updateLog: nil];
+    }
 }
 
 - (void) windowWillClose: (id)sender
@@ -150,71 +170,96 @@
     if ((messages = tr_getQueuedMessages()) == NULL)
         return;
     
+    [fLock lock];
+    
+    static NSUInteger currentIndex = 0;
+    
+    NSScroller * scroller = [[fMessageTable enclosingScrollView] verticalScroller];
+    const BOOL shouldScroll = currentIndex == 0 || [scroller floatValue] == 1.0 || [scroller isHidden]
+                                || [scroller knobProportion] == 1.0;
+    
+    const NSInteger maxLevel = [[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"];
+    NSString * filterString = [fFilterField stringValue];
+    
+    BOOL changed = NO;
+    
     for (tr_msg_list * currentMessage = messages; currentMessage != NULL; currentMessage = currentMessage->next)
     {
         NSString * name = currentMessage->name != NULL ? [NSString stringWithUTF8String: currentMessage->name]
                             : [[NSProcessInfo processInfo] processName];
         
+        NSString * file = [[[NSString stringWithUTF8String: currentMessage->file] lastPathComponent] stringByAppendingFormat: @":%d",
+                            currentMessage->line];
+        
         NSDictionary * message  = [NSDictionary dictionaryWithObjectsAndKeys:
                                     [NSString stringWithUTF8String: currentMessage->message], @"Message",
                                     [NSDate dateWithTimeIntervalSince1970: currentMessage->when], @"Date",
-                                    [NSNumber numberWithUnsignedInt: fIndex++], @"Index", //more accurate when sorting by date
-                                    [NSNumber numberWithInt: currentMessage->level], @"Level",
+                                    [NSNumber numberWithUnsignedInteger: currentIndex++], @"Index", //more accurate when sorting by date
+                                    [NSNumber numberWithInteger: currentMessage->level], @"Level",
                                     name, @"Name",
-                                    [NSString stringWithUTF8String: currentMessage->file], @"File",
-                                    [NSNumber numberWithInt: currentMessage->line], @"Line", nil];
-                                
+                                    file, @"File", nil];
+        
         [fMessages addObject: message];
+        
+        if (currentMessage->level <= maxLevel && [self shouldIncludeMessageForFilter: filterString message: message])
+        {
+            [fDisplayedMessages addObject: message];
+            changed = YES;
+        }
     }
     
+    if ([fMessages count] > TR_MAX_MSG_LOG)
+    {
+        const NSUInteger oldCount = [fDisplayedMessages count];
+        
+        NSIndexSet * removeIndexes = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fMessages count]-TR_MAX_MSG_LOG)];
+        NSArray * itemsToRemove = [fMessages objectsAtIndexes: removeIndexes];
+        
+        [fMessages removeObjectsAtIndexes: removeIndexes];
+        [fDisplayedMessages removeObjectsInArray: itemsToRemove];
+        
+        changed |= oldCount > [fDisplayedMessages count];
+    }
+    
+    if (changed)
+    {
+        [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
+        
+        [fMessageTable reloadData];
+        if (shouldScroll)
+            [fMessageTable scrollRowToVisible: [fMessageTable numberOfRows]-1];
+    }
+    
+    [fLock unlock];
+    
     tr_freeMessageList(messages);
-    
-    NSScroller * scroller = [[fMessageTable enclosingScrollView] verticalScroller];
-    BOOL shouldScroll = [scroller floatValue] == 1.0 || [scroller isHidden] || [scroller knobProportion] == 1.0;
-    
-    NSUInteger total = [fMessages count];
-    if (total > MAX_MESSAGES)
-    {
-        //remove the oldest
-        NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
-        [fMessages sortUsingDescriptors: [NSArray arrayWithObject: descriptor]];
-        
-        [fMessages removeObjectsInRange: NSMakeRange(0, total-MAX_MESSAGES)];
-        
-        [fMessageTable noteHeightOfRowsWithIndexesChanged: [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, MAX_MESSAGES)]];
-        total = MAX_MESSAGES;
-    }
-    
-    [fMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
-    
-    [fMessageTable reloadData];
-    if (shouldScroll)
-        [fMessageTable scrollRowToVisible: total-1];
 }
 
 - (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
 {
-    return [fMessages count];
+    return [fDisplayedMessages count];
 }
 
 - (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) column row: (NSInteger) row
 {
     NSString * ident = [column identifier];
-    NSDictionary * message = [fMessages objectAtIndex: row];
+    NSDictionary * message = [fDisplayedMessages objectAtIndex: row];
 
     if ([ident isEqualToString: @"Date"])
         return [message objectForKey: @"Date"];
     else if ([ident isEqualToString: @"Level"])
     {
-        switch ([[message objectForKey: @"Level"] intValue])
+        const NSInteger level = [[message objectForKey: @"Level"] integerValue];
+        switch (level)
         {
             case TR_MSG_ERR:
-                return fErrorImage;
+                return [NSImage imageNamed: @"RedDot.png"];
             case TR_MSG_INF:
-                return fInfoImage;
+                return [NSImage imageNamed: @"YellowDot.png"];
             case TR_MSG_DBG:
-                return fDebugImage;
+                return [NSImage imageNamed: @"PurpleDot.png"];
             default:
+                NSAssert1(NO, @"Unknown message log level: %d", level);
                 return nil;
         }
     }
@@ -227,41 +272,47 @@
 #warning don't cut off end
 - (CGFloat) tableView: (NSTableView *) tableView heightOfRow: (NSInteger) row
 {
+    NSString * message = [[fDisplayedMessages objectAtIndex: row] objectForKey: @"Message"];
+    
     NSTableColumn * column = [tableView tableColumnWithIdentifier: @"Message"];
-    
-    if (!fAttributes)
-        fAttributes = [[[[column dataCell] attributedStringValue] attributesAtIndex: 0 effectiveRange: NULL] retain];
-    
-    const CGFloat count = floorf([[[fMessages objectAtIndex: row] objectForKey: @"Message"] sizeWithAttributes: fAttributes].width
-                                / [column width]);
+    const CGFloat count = floorf([message sizeWithAttributes: fAttributes].width / [column width]);
     return [tableView rowHeight] * (count + 1.0);
 }
 
 - (void) tableView: (NSTableView *) tableView sortDescriptorsDidChange: (NSArray *) oldDescriptors
 {
-    [fMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
+    [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
     [fMessageTable reloadData];
 }
 
 - (NSString *) tableView: (NSTableView *) tableView toolTipForCell: (NSCell *) cell rect: (NSRectPointer) rect
                 tableColumn: (NSTableColumn *) column row: (NSInteger) row mouseLocation: (NSPoint) mouseLocation
 {
-    NSDictionary * message = [fMessages objectAtIndex: row];
-    return [NSString stringWithFormat: @"%@:%@", [[message objectForKey: @"File"] lastPathComponent], [message objectForKey: @"Line"]];
+    NSDictionary * message = [fDisplayedMessages objectAtIndex: row];
+    return [message objectForKey: @"File"];
 }
 
 - (void) copy: (id) sender
 {
-    NSPasteboard * pb = [NSPasteboard generalPasteboard];
-    [pb declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: self];
-    
     NSIndexSet * indexes = [fMessageTable selectedRowIndexes];
     NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [indexes count]];
     
-    for (NSDictionary * message in [fMessages objectsAtIndexes: indexes])
+    for (NSDictionary * message in [fDisplayedMessages objectsAtIndexes: indexes])
         [messageStrings addObject: [self stringForMessage: message]];
     
-    [pb setString: [messageStrings componentsJoinedByString: @"\n"] forType: NSStringPboardType];
+    NSString * messageString = [messageStrings componentsJoinedByString: @"\n"];
+    
+    NSPasteboard * pb = [NSPasteboard generalPasteboard];
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        [pb clearContents];
+        [pb writeObjects: [NSArray arrayWithObject: messageString]];
+    }
+    else
+    {
+        [pb declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: nil];
+        [pb setString: messageString forType: NSStringPboardType];
+    }
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -276,9 +327,7 @@
 
 - (void) changeLevel: (id) sender
 {
-    [self updateLog: nil];
-    
-    int level;
+    NSInteger level;
     switch ([fLevelButton indexOfSelectedItem])
     {
         case LEVEL_ERROR:
@@ -290,16 +339,73 @@
         case LEVEL_DEBUG:
             level = TR_MSG_DBG;
             break;
+        default:
+            NSAssert1(NO, @"Unknown message log level: %d", [fLevelButton indexOfSelectedItem]);
     }
     
-    tr_setMessageLevel(level);
+    if ([[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"] == level)
+        return;
+    
+    [fLock lock];
+    
     [[NSUserDefaults standardUserDefaults] setInteger: level forKey: @"MessageLevel"];
+    
+    NSString * filterString = [fFilterField stringValue];
+    
+    [fDisplayedMessages removeAllObjects];
+    for (NSDictionary * message in fMessages)
+        if ([[message objectForKey: @"Level"] integerValue] <= level
+            && [self shouldIncludeMessageForFilter: filterString message: message])
+            [fDisplayedMessages addObject: message];
+    
+    [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
+    
+    [fMessageTable reloadData];
+    
+    if ([fDisplayedMessages count] > 0)
+    {
+        [fMessageTable deselectAll: self];
+        [fMessageTable scrollRowToVisible: [fMessageTable numberOfRows]-1];
+    }
+    
+    [fLock unlock];
+}
+
+- (void) changeFilter: (id) sender
+{
+    [fLock lock];
+    
+    const NSInteger level = [[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"];
+    NSString * filterString = [fFilterField stringValue];
+    
+    [fDisplayedMessages removeAllObjects];
+    for (NSDictionary * message in fMessages)
+        if ([[message objectForKey: @"Level"] integerValue] <= level
+            && [self shouldIncludeMessageForFilter: filterString message: message])
+            [fDisplayedMessages addObject: message];
+    
+    [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
+    
+    [fMessageTable reloadData];
+    
+    if ([fDisplayedMessages count] > 0)
+    {
+        [fMessageTable deselectAll: self];
+        [fMessageTable scrollRowToVisible: [fMessageTable numberOfRows]-1];
+    }
+    
+    [fLock unlock];
 }
 
 - (void) clearLog: (id) sender
 {
+    [fLock lock];
+    
     [fMessages removeAllObjects];
+    [fDisplayedMessages removeAllObjects];
     [fMessageTable reloadData];
+    
+    [fLock unlock];
 }
 
 - (void) writeToFile: (id) sender
@@ -307,38 +413,37 @@
     //make the array sorted by date
     NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
     NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
-    NSArray * sortedMessages = [fMessages sortedArrayUsingDescriptors: descriptors];
+    NSArray * sortedMessages = [[fDisplayedMessages sortedArrayUsingDescriptors: descriptors] retain];
     [descriptors release];
     
-    //create the text to output
-    NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [fMessages count]];
-    for (NSDictionary * message in sortedMessages)
-        [messageStrings addObject: [self stringForMessage: message]];
-    
-    NSString * fileString = [[messageStrings componentsJoinedByString: @"\n"] retain];
-    
     NSSavePanel * panel = [NSSavePanel savePanel];
     [panel setRequiredFileType: @"txt"];
     [panel setCanSelectHiddenExtension: YES];
     
     [panel beginSheetForDirectory: nil file: NSLocalizedString(@"untitled", "Save log panel -> default file name")
             modalForWindow: [self window] modalDelegate: self
-            didEndSelector: @selector(writeToFileSheetClosed:returnCode:contextInfo:) contextInfo: fileString];
+            didEndSelector: @selector(writeToFileSheetClosed:returnCode:contextInfo:) contextInfo: sortedMessages];
 }
 
-- (void) writeToFileSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (NSString *) string
+- (void) writeToFileSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (NSArray *) messages
 {
     if (code == NSOKButton)
     {
-        if (![string writeToFile: [panel filename] atomically: YES encoding: NSUTF8StringEncoding error: nil])
+        //create the text to output
+        NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [messages count]];
+        for (NSDictionary * message in messages)
+            [messageStrings addObject: [self stringForMessage: message]];
+    
+        NSString * fileString = [messageStrings componentsJoinedByString: @"\n"];
+        
+        if (![fileString writeToFile: [panel filename] atomically: YES encoding: NSUTF8StringEncoding error: nil])
         {
             NSAlert * alert = [[NSAlert alloc] init];
             [alert addButtonWithTitle: NSLocalizedString(@"OK", "Save log alert panel -> button")];
-            [alert setMessageText: [NSString stringWithFormat: NSLocalizedString(@"Log Could Not Be Saved",
-                                    "Save log alert panel -> title")]];
-            [alert setInformativeText: [NSString stringWithFormat: 
+            [alert setMessageText: NSLocalizedString(@"Log Could Not Be Saved", "Save log alert panel -> title")];
+            [alert setInformativeText: [NSString stringWithFormat:
                     NSLocalizedString(@"There was a problem creating the file \"%@\".",
-                                        "Save log alert panel -> message"), [[panel filename] lastPathComponent]]];
+                    "Save log alert panel -> message"), [[panel filename] lastPathComponent]]];
             [alert setAlertStyle: NSWarningAlertStyle];
             
             [alert runModal];
@@ -346,7 +451,7 @@
         }
     }
     
-    [string release];
+    [messages release];
 }
 
 @end
@@ -359,26 +464,37 @@
                     NSMakeRange(0, [fMessageTable numberOfRows])]];
 }
 
+- (BOOL) shouldIncludeMessageForFilter: (NSString *) filterString message: (NSDictionary *) message
+{
+    if ([filterString isEqualToString: @""])
+        return YES;
+    
+    const NSStringCompareOptions searchOptions = NSCaseInsensitiveSearch | NSDiacriticInsensitiveSearch;
+    return [[message objectForKey: @"Name"] rangeOfString: filterString options: searchOptions].location != NSNotFound
+            || [[message objectForKey: @"Message"] rangeOfString: filterString options: searchOptions].location != NSNotFound;
+}
+
 - (NSString *) stringForMessage: (NSDictionary *) message
 {
-    NSString * level;
-    switch ([[message objectForKey: @"Level"] intValue])
+    NSString * levelString;
+    const NSInteger level = [[message objectForKey: @"Level"] integerValue];
+    switch (level)
     {
         case TR_MSG_ERR:
-            level = NSLocalizedString(@"Error", "Message window -> level");
+            levelString = NSLocalizedString(@"Error", "Message window -> level");
             break;
         case TR_MSG_INF:
-            level = NSLocalizedString(@"Info", "Message window -> level");
+            levelString = NSLocalizedString(@"Info", "Message window -> level");
             break;
         case TR_MSG_DBG:
-            level = NSLocalizedString(@"Debug", "Message window -> level");
+            levelString = NSLocalizedString(@"Debug", "Message window -> level");
             break;
         default:
-            level = @"";
+            NSAssert1(NO, @"Unknown message log level: %d", level);
     }
     
-    return [NSString stringWithFormat: @"%@ %@:%@ [%@] %@: %@", [message objectForKey: @"Date"],
-            [[message objectForKey: @"File"] lastPathComponent], [message objectForKey: @"Line"], level,
+    return [NSString stringWithFormat: @"%@ %@ [%@] %@: %@", [message objectForKey: @"Date"],
+            [message objectForKey: @"File"], levelString,
             [message objectForKey: @"Name"], [message objectForKey: @"Message"], nil];
 }
 
