@@ -1,7 +1,7 @@
 /******************************************************************************
  * $Id: StringAdditions.h 2869 2007-08-19 03:03:28Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,10 +31,14 @@
 
 + (NSString *) stringForFileSize: (uint64_t) size;
 
-+ (NSString *) stringForSpeed: (float) speed;
-+ (NSString *) stringForSpeedAbbrev: (float) speed;
-+ (NSString *) stringForRatio: (float) ratio;
++ (NSString *) stringForSpeed: (CGFloat) speed;
++ (NSString *) stringForSpeedAbbrev: (CGFloat) speed;
++ (NSString *) stringForRatio: (CGFloat) ratio;
 
-- (NSComparisonResult) compareIP: (NSString *) string;
++ (NSString *) timeString: (uint64_t) seconds showSeconds: (BOOL) showSeconds;
++ (NSString *) timeString: (uint64_t) seconds showSeconds: (BOOL) showSeconds maxFields: (NSUInteger) max;
+
+- (NSComparisonResult) compareFinder: (NSString *) string; //how the Finder compares strings
+- (NSComparisonResult) compareNumeric: (NSString *) string; //simple compare method for strings with numbers (works for IP addresses)
 
 @end
