@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.m 8461 2009-05-21 04:35:40Z livings124 $
+ * $Id: PrefsController.m 10119 2010-02-06 18:44:13Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,7 @@
 #import "BlocklistScheduler.h"
 #import "PortChecker.h"
 #import "BonjourController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
 #import "utils.h"
@@ -206,7 +207,7 @@ tr_session * fHandle;
     
     //set proxy type
     [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
-    int proxyType;
+    NSInteger proxyType;
     switch (tr_sessionGetProxyType(fHandle))
     {
         case TR_PROXY_SOCKS4:
@@ -217,6 +218,9 @@ tr_session * fHandle;
             break;
         case TR_PROXY_HTTP:
             proxyType = PROXY_HTTP;
+            break;
+        default:
+            NSAssert(NO, @"Unknown proxy type received");
     }
     [fProxyTypePopUp selectItemAtIndex: proxyType];
     
@@ -308,9 +312,15 @@ tr_session * fHandle;
     return [item autorelease];
 }
 
+- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
+{
+    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_GROUPS, TOOLBAR_BANDWIDTH,
+                                        TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
+}
+
 - (NSArray *) toolbarSelectableItemIdentifiers: (NSToolbar *) toolbar
 {
-    return [self toolbarDefaultItemIdentifiers: toolbar];
+    return [self toolbarAllowedItemIdentifiers: toolbar];
 }
 
 - (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar
@@ -318,12 +328,6 @@ tr_session * fHandle;
     return [self toolbarAllowedItemIdentifiers: toolbar];
 }
 
-- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
-{
-    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_GROUPS, TOOLBAR_BANDWIDTH,
-                                        TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
-}
-
 //for a beta release, always use the beta appcast
 #if defined(TR_BETA_RELEASE)
 #define SPARKLE_TAG YES
@@ -337,7 +341,7 @@ tr_session * fHandle;
 
 - (void) setPort: (id) sender
 {
-    const int port = [sender intValue];
+    const tr_port port = [sender intValue];
     [fDefaults setInteger: port forKey: @"BindPort"];
     tr_sessionSetPeerPort(fHandle, port);
     
@@ -348,9 +352,11 @@ tr_session * fHandle;
 - (void) randomPort: (id) sender
 {
     const tr_port port = tr_sessionSetPeerPortRandom(fHandle);
-    
+    [fDefaults setInteger: port forKey: @"BindPort"];
     [fPortField setIntValue: port];
-    [self setPort: fPortField];
+    
+    fPeerPort = -1;
+    [self updatePortStatus];
 }
 
 - (void) setRandomPortOnStart: (id) sender
@@ -418,14 +424,17 @@ tr_session * fHandle;
 {
     NSMutableArray * sounds = [NSMutableArray array];
     
-    NSArray * directories = [NSArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds", @"Library/Sounds", nil];
+    NSArray * directories = NSSearchPathForDirectoriesInDomains(NSAllLibrariesDirectory,
+                                NSUserDomainMask | NSLocalDomainMask | NSSystemDomainMask, YES);
     
     for (NSString * directory in directories)
     {
+        directory = [directory stringByAppendingPathComponent: @"Sounds"];
+        
         BOOL isDirectory;
         if ([[NSFileManager defaultManager] fileExistsAtPath: directory isDirectory: &isDirectory] && isDirectory)
         {
-            NSArray * directoryContents = [[NSFileManager defaultManager] directoryContentsAtPath: directory];
+            NSArray * directoryContents = [[NSFileManager defaultManager] contentsOfDirectoryAtPath: directory error: NULL];
             for (NSString * sound in directoryContents)
             {
                 sound = [sound stringByDeletingPathExtension];
@@ -457,6 +466,7 @@ tr_session * fHandle;
 {
     const int count = [sender intValue];
     [fDefaults setInteger: count forKey: @"PeersTorrent"];
+    tr_sessionSetPeerLimitPerTorrent(fHandle, count);
 }
 
 - (void) setPEX: (id) sender
@@ -523,12 +533,18 @@ tr_session * fHandle;
         NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistLastUpdate"];
         if (updatedDate)
         {
-            NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
-            [dateFormatter setDateStyle: NSDateFormatterFullStyle];
-            [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
-            
-            updatedDateString = [dateFormatter stringFromDate: updatedDate];
-            [dateFormatter release];
+            if ([NSApp isOnSnowLeopardOrBetter])
+                updatedDateString = [NSDateFormatter localizedStringFromDate: updatedDate dateStyle: NSDateFormatterFullStyle
+                                        timeStyle: NSDateFormatterShortStyle];
+            else
+            {
+                NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
+                [dateFormatter setDateStyle: NSDateFormatterFullStyle];
+                [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
+                
+                updatedDateString = [dateFormatter stringFromDate: updatedDate];
+                [dateFormatter release];
+            }
         }
         else
             updatedDateString = NSLocalizedString(@"N/A", "Prefs -> blocklist -> message");
@@ -667,8 +683,19 @@ tr_session * fHandle;
     [fDefaults removeObjectForKey: @"WarningFolderDataSameName"];
     [fDefaults removeObjectForKey: @"WarningResetStats"];
     [fDefaults removeObjectForKey: @"WarningCreatorBlankAddress"];
-    [fDefaults removeObjectForKey: @"WarningRemoveBuiltInTracker"];
+    [fDefaults removeObjectForKey: @"WarningCreatorPrivateBlankAddress"];
+    [fDefaults removeObjectForKey: @"WarningRemoveTrackers"];
     [fDefaults removeObjectForKey: @"WarningInvalidOpen"];
+    [fDefaults removeObjectForKey: @"WarningDonate"];
+    //[fDefaults removeObjectForKey: @"WarningLegal"];
+}
+
+- (void) setDefaultForMagnets: (id) sender
+{
+    NSString * bundleID = [[NSBundle mainBundle] bundleIdentifier];
+    const OSStatus result = LSSetDefaultHandlerForURLScheme((CFStringRef)@"magnet", (CFStringRef)bundleID);
+    if (result != noErr)
+        NSLog(@"Failed setting default magnet link handler");
 }
 
 - (void) setQueue: (id) sender
@@ -728,6 +755,16 @@ tr_session * fHandle;
         @selector(incompleteFolderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
 }
 
+- (void) setUseIncompleteFolder: (id) sender
+{
+    tr_sessionSetIncompleteDirEnabled(fHandle, [fDefaults boolForKey: @"UseIncompleteDownloadFolder"]);
+}
+
+- (void) setRenamePartialFiles: (id) sender
+{
+    tr_sessionSetIncompleteFileNamingEnabled(fHandle, [fDefaults boolForKey: @"RenamePartialFiles"]);
+}
+
 - (void) setAutoImport: (id) sender
 {
     NSString * path;
@@ -797,6 +834,9 @@ tr_session * fHandle;
             break;
         case PROXY_SOCKS5:
             type = @"SOCKS5";
+            break;
+        default:
+            NSAssert1(NO, @"Unknown index %d received for proxy type", [sender indexOfSelectedItem]);
     }
     
     [fDefaults setObject: type forKey: @"ProxyType"];
@@ -970,8 +1010,8 @@ tr_session * fHandle;
         [fRPCWhitelistArray addObject: @""];
         [fRPCWhitelistTable reloadData];
         
-        int row = [fRPCWhitelistArray count] - 1;
-        [fRPCWhitelistTable selectRow: row byExtendingSelection: NO];
+        const int row = [fRPCWhitelistArray count] - 1;
+        [fRPCWhitelistTable selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
         [fRPCWhitelistTable editColumn: 0 row: row withEvent: nil select: YES];
     }
 }
@@ -1078,12 +1118,21 @@ tr_session * fHandle;
     NSString * downloadLocation = [[NSString stringWithUTF8String: tr_sessionGetDownloadDir(fHandle)] stringByStandardizingPath];
     [fDefaults setObject: downloadLocation forKey: @"DownloadFolder"];
     
+    NSString * incompleteLocation = [[NSString stringWithUTF8String: tr_sessionGetIncompleteDir(fHandle)] stringByStandardizingPath];
+    [fDefaults setObject: incompleteLocation forKey: @"IncompleteDownloadFolder"];
+    
+    const BOOL useIncomplete = tr_sessionIsIncompleteDirEnabled(fHandle);
+    [fDefaults setBool: useIncomplete forKey: @"UseIncompleteDownloadFolder"];
+    
+    const BOOL usePartialFileRanaming = tr_sessionIsIncompleteFileNamingEnabled(fHandle);
+    [fDefaults setBool: usePartialFileRanaming forKey: @"RenamePartialFiles"];
+    
     //peers
     const uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
     [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
     
     const uint16_t peersTorrent = tr_sessionGetPeerLimitPerTorrent(fHandle);
-    [fDefaults setInteger: peersTotal forKey: @"PeersTorrent"];
+    [fDefaults setInteger: peersTorrent forKey: @"PeersTorrent"];
     
     //pex
     const BOOL pex = tr_sessionIsPexEnabled(fHandle);
@@ -1127,11 +1176,11 @@ tr_session * fHandle;
     
     //alt speed limit - down
     const int downLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_DOWN);
-    [fDefaults setInteger: downLimit forKey: @"SpeedLimitDownloadLimit"];
+    [fDefaults setInteger: downLimitAlt forKey: @"SpeedLimitDownloadLimit"];
     
     //alt speed limit - up
     const int upLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_UP);
-    [fDefaults setInteger: upLimitAlt forKey: @"SpeedLimitDownloadLimit"];
+    [fDefaults setInteger: upLimitAlt forKey: @"SpeedLimitUploadLimit"];
     
     //alt speed limit schedule
     const BOOL useAltSpeedSched = tr_sessionUsesAltSpeedTime(fHandle);
@@ -1286,7 +1335,12 @@ tr_session * fHandle;
 - (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
 {
     if (code == NSOKButton)
-        [fDefaults setObject: [[openPanel filenames] objectAtIndex: 0] forKey: @"IncompleteDownloadFolder"];
+    {
+        NSString * folder = [[openPanel filenames] objectAtIndex: 0];
+        [fDefaults setObject: folder forKey: @"IncompleteDownloadFolder"];
+        
+        tr_sessionSetIncompleteDir(fHandle, [folder UTF8String]);
+    }
     [fIncompleteFolderPopUp selectItemAtIndex: 0];
 }
 
