@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: details.cc 8570 2009-05-31 19:33:48Z charles $
+ * $Id: details.cc 10077 2010-02-02 05:34:26Z charles $
  */
 
 #include <cassert>
@@ -16,29 +16,28 @@
 
 #include <QCheckBox>
 #include <QComboBox>
-#include <QEvent>
-#include <QHeaderView>
-#include <QResizeEvent>
+#include <QDateTime>
 #include <QDialogButtonBox>
 #include <QDoubleSpinBox>
+#include <QEvent>
 #include <QFont>
 #include <QFontMetrics>
 #include <QHBoxLayout>
-#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QHeaderView>
 #include <QLabel>
 #include <QLocale>
 #include <QPushButton>
-#include <QSpinBox>
 #include <QRadioButton>
+#include <QResizeEvent>
+#include <QSpinBox>
 #include <QStyle>
 #include <QTabWidget>
-#include <QTreeView>
 #include <QTextBrowser>
-#include <QDateTime>
+#include <QTreeView>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 #include <QVBoxLayout>
-#include <QHBoxLayout>
 
 #include <libtransmission/transmission.h>
 
@@ -130,14 +129,12 @@ Details :: Details( Session& session, TorrentModel& model, QWidget * parent ):
 
     QTabWidget * t = new QTabWidget( this );
     QWidget * w;
-    t->addTab( w = createActivityTab( ),  tr( "Activity" ) );
+    t->addTab( w = createInfoTab( ),      tr( "Information" ) );
     myWidgets << w;
     t->addTab( w = createPeersTab( ),     tr( "Peers" ) );
     myWidgets << w;
     t->addTab( w = createTrackerTab( ),   tr( "Tracker" ) );
     myWidgets << w;
-    t->addTab( w = createInfoTab( ),      tr( "Information" ) );
-    myWidgets << w;
     t->addTab( w = createFilesTab( ),     tr( "Files" ) );
     myWidgets << w;
     t->addTab( w = createOptionsTab( ),   tr( "Options" ) );
@@ -154,7 +151,7 @@ Details :: Details( Session& session, TorrentModel& model, QWidget * parent ):
     myTimer.setSingleShot( false );
     myTimer.start( REFRESH_INTERVAL_MSEC );
 }
-    
+
 Details :: ~Details( )
 {
 }
@@ -185,7 +182,7 @@ Details :: setIds( const QSet<int>& ids )
 
     foreach( QWidget * w, myWidgets )
         w->setEnabled( false );
-    
+
     onTimer( );
 }
 
@@ -213,7 +210,6 @@ Details :: onTorrentChanged( )
 void
 Details :: refresh( )
 {
-    int i;
     QLocale locale;
     const int n = myIds.size( );
     const bool single = n == 1;
@@ -233,7 +229,7 @@ Details :: refresh( )
     }
 
     ///
-    ///  activity tab 
+    ///  activity tab
     ///
 
     // myStateLabel
@@ -250,69 +246,123 @@ Details :: refresh( )
     }
     myStateLabel->setText( string );
 
-    // myProgressLabel
-    if( torrents.empty( ) )
-        string = tr( "None" );
-    else {
-        double sizeWhenDone = 0;
-        double leftUntilDone = 0;
-        foreach( const Torrent * t, torrents ) {
-            sizeWhenDone += t->sizeWhenDone( );
-            leftUntilDone += t->leftUntilDone( );
-        }
-        string = locale.toString( 100.0*((sizeWhenDone-leftUntilDone)/sizeWhenDone), 'f', 2 );
-    }
-    myProgressLabel->setText( string );
-
     // myHaveLabel
+    double sizeWhenDone = 0;
+    double leftUntilDone = 0;
+    double available = 0;
     int64_t haveTotal = 0;
     int64_t haveVerified = 0;
+    int64_t haveUnverified = 0;
     int64_t verifiedPieces = 0;
     foreach( const Torrent * t, torrents ) {
-        haveTotal += t->haveTotal( );
-        const uint64_t v = t->haveVerified( );
-        haveVerified += v;
-        verifiedPieces += v / t->pieceSize( );
+        if( t->hasMetadata( ) ) {
+            haveTotal += t->haveTotal( );
+            haveUnverified += t->haveUnverified( );
+            const uint64_t v = t->haveVerified( );
+            haveVerified += v;
+            verifiedPieces += v / t->pieceSize( );
+            sizeWhenDone += t->sizeWhenDone( );
+            leftUntilDone += t->leftUntilDone( );
+            available += t->sizeWhenDone() - t->leftUntilDone() + t->desiredAvailable();
+        }
     }
-    myHaveLabel->setText( tr( "%1 (%2 verified in %L3 pieces)" )
-                            .arg( Utils::sizeToString( haveTotal ) )
-                            .arg( Utils::sizeToString( haveVerified ) )
-                            .arg( verifiedPieces ) );
+    if( !haveVerified && !haveUnverified )
+        string = none;
+    else {
+        const double d = 100.0 * ( sizeWhenDone ? ( sizeWhenDone - leftUntilDone ) / sizeWhenDone : 1 );
+        QString pct = locale.toString( d, 'f', 2 );
+        if( !haveUnverified )
+            string = tr( "%1 (%2%)" )
+                         .arg( Utils :: sizeToString( haveVerified + haveUnverified ) )
+                         .arg( pct );
+        else
+            string = tr( "%1 (%2%); %3 Unverified" )
+                         .arg( Utils :: sizeToString( haveVerified + haveUnverified ) )
+                         .arg( pct )
+                         .arg( Utils :: sizeToString( haveUnverified ) );
+    }
+    myHaveLabel->setText( string );
 
-    int64_t sum = 0;
-    foreach( const Torrent * t, torrents ) sum += t->downloadedEver( );
-    myDownloadedLabel->setText( Utils::sizeToString( sum ) );
+    // myAvailabilityLabel
+    if( sizeWhenDone < 1 )
+        string = none;
+    else
+        string.sprintf( "%'.1f%%", ( 100.0 * available ) / sizeWhenDone );
+    myAvailabilityLabel->setText( string );
 
-    sum = 0;
+    // myDownloadedLabel
+    if( torrents.empty( ) )
+        string = none;
+    else {
+        uint64_t d=0, f=0;
+        foreach( const Torrent * t, torrents ) {
+            d += t->downloadedEver( );
+            f += t->failedEver( );
+        }
+        const QString dstr = Utils::sizeToString( d );
+        const QString fstr = Utils::sizeToString( f );
+        if( f )
+            string = tr( "%1 (+%2 corrupt)" ).arg( dstr ).arg( fstr );
+        else
+            string = dstr;
+    }
+    myDownloadedLabel->setText( string );
+
+    uint64_t sum = 0;
     foreach( const Torrent * t, torrents ) sum += t->uploadedEver( );
     myUploadedLabel->setText( Utils::sizeToString( sum ) );
 
-    sum = 0;
-    foreach( const Torrent *t, torrents ) sum += t->failedEver( );
-    myFailedLabel->setText( Utils::sizeToString( sum ) );
-
     double d = 0;
     foreach( const Torrent *t, torrents ) d += t->ratio( );
     myRatioLabel->setText( Utils :: ratioToString( d / n ) );
 
-    Speed speed;
-    foreach( const Torrent *t, torrents ) speed += t->swarmSpeed( );
-    mySwarmSpeedLabel->setText( Utils::speedToString( speed ) );
+    const QDateTime qdt_now = QDateTime::currentDateTime( );
 
+    // myRunTimeLabel
     if( torrents.empty( ) )
         string = none;
     else {
-        string = torrents[0]->dateAdded().toString();
+        bool allPaused = true;
+        QDateTime baseline = torrents[0]->lastStarted( );
         foreach( const Torrent * t, torrents ) {
-            if( string != t->dateAdded().toString() ) {
+            if( baseline != t->lastStarted( ) )
+                baseline = QDateTime( );
+            if( !t->isPaused( ) )
+                allPaused = false;
+        }
+        if( allPaused )
+            string = tr( "Stopped" );
+        else if( baseline.isNull( ) )
+            string = mixed;
+        else
+            string = Utils::timeToString( baseline.secsTo( qdt_now ) );
+    }
+    myRunTimeLabel->setText( string );
+
+
+    // myETALabel
+    string.clear( );
+    if( torrents.empty( ) )
+        string = none;
+    else {
+        int baseline = torrents[0]->getETA( );
+        foreach( const Torrent * t, torrents ) {
+            if( baseline != t->getETA( ) ) {
                 string = mixed;
                 break;
             }
         }
+        if( string.isEmpty( ) ) {
+            if( baseline < 0 )
+                string = tr( "Unknown" );
+            else
+                string = Utils::timeToString( baseline );
+       }
     }
-    myAddedDateLabel->setText( string );
+    myETALabel->setText( string );
 
 
+    // myLastActivityLabel
     if( torrents.empty( ) )
         string = none;
     else {
@@ -322,9 +372,13 @@ Details :: refresh( )
             if( latest < dt )
                 latest = dt;
         }
-        string = latest.toString( );
+        const int seconds = latest.secsTo( qdt_now );
+        if( seconds < 5 )
+            string = tr( "Active now" );
+        else
+            string = tr( "%1 ago" ).arg( Utils::timeToString( seconds ) );
     }
-    myActivityLabel->setText( string );
+    myLastActivityLabel->setText( string );
 
 
     if( torrents.empty( ) )
@@ -338,6 +392,8 @@ Details :: refresh( )
             }
         }
     }
+    if( string.isEmpty( ) )
+        string = none;
     myErrorLabel->setText( string );
 
 
@@ -345,24 +401,30 @@ Details :: refresh( )
     /// information tab
     ///
 
-    // myPiecesLabel
+    // mySizeLabel
     if( torrents.empty( ) )
         string = none;
     else {
-        int64_t pieceCount = 0;
-        uint64_t baseSize = torrents[0]->pieceSize( );
+        int pieces = 0;
+        uint64_t size = 0;
+        uint32_t pieceSize = torrents[0]->pieceSize( );
         foreach( const Torrent * t, torrents ) {
-            pieceCount += t->pieceCount( );
-            if( baseSize != t->pieceSize( ) )
-                baseSize = 0;
+            pieces += t->pieceCount( );
+            size += t->totalSize( );
+            if( pieceSize != t->pieceSize( ) )
+                pieceSize = 0;
         }
-        if( !baseSize ) // mixed piece size
-            string = tr( "%L1 Pieces" ).arg( pieceCount );
+        if( !size )
+            string = none;
+        else if( pieceSize > 0 )
+            string = tr( "%1 (%Ln pieces @ %2)", "", pieces )
+                     .arg( Utils::sizeToString( size ) )
+                     .arg( Utils::sizeToString( pieceSize ) );
         else
-            string = tr( "%L1 Pieces @ %2" ).arg( pieceCount )
-                                            .arg( Utils::sizeToString( baseSize ) );
+            string = tr( "%1 (%Ln pieces)", "", pieces )
+                     .arg( Utils::sizeToString( size ) );
     }
-    myPiecesLabel->setText( string );
+    mySizeLabel->setText( string );
 
     // myHashLabel
     if( torrents.empty( ) )
@@ -383,7 +445,7 @@ Details :: refresh( )
         string = none;
     else {
         bool b = torrents[0]->isPrivate( );
-        string = b ? tr( "Private to this tracker -- PEX disabled" )
+        string = b ? tr( "Private to this tracker -- DHT and PEX disabled" )
                    : tr( "Public torrent" );
         foreach( const Torrent * t, torrents ) {
             if( b != t->isPrivate( ) ) {
@@ -408,35 +470,29 @@ Details :: refresh( )
     }
     myCommentBrowser->setText( string );
 
-    // myCreatorLabel
+    // myOriginLabel
     if( torrents.empty( ) )
         string = none;
     else {
-        string = torrents[0]->creator( );
+        bool mixed_creator=false, mixed_date=false;
+        const QString creator = torrents[0]->creator();
+        const QString date = torrents[0]->dateCreated().toString();
         foreach( const Torrent * t, torrents ) {
-            if( string != t->creator( ) ) {
-                string = mixed;
-                break;
-            }
+            mixed_creator |= ( creator != t->creator() );
+            mixed_date |=  ( date != t->dateCreated().toString() );
         }
+        if( mixed_creator && mixed_date )
+            string = mixed;
+        else if( mixed_date )
+            string = tr( "Created by %1" ).arg( creator );
+        else if( mixed_creator || creator.isEmpty( ) )
+            string = tr( "Created on %1" ).arg( date );
+        else
+            string = tr( "Created by %1 on %2" ).arg( creator ).arg( date );
     }
-    myCreatorLabel->setText( string.isEmpty() ? unknown : string );
+    myOriginLabel->setText( string );
 
-    // myDateCreatedLabel
-    if( torrents.empty( ) )
-        string = none;
-    else {
-        string = torrents[0]->dateCreated().toString();
-        foreach( const Torrent * t, torrents ) {
-            if( string != t->dateCreated().toString() ) {
-                string = mixed;
-                break;
-            }
-        }
-    }
-    myDateCreatedLabel->setText( string );
-
-    // myDestinationLabel
+    // myLocationLabel
     if( torrents.empty( ) )
         string = none;
     else {
@@ -448,21 +504,8 @@ Details :: refresh( )
             }
         }
     }
-    myDestinationLabel->setText( string );
+    myLocationLabel->setText( string );
 
-    // myTorrentFileLabel
-    if( torrents.empty( ) )
-        string = none;
-    else {
-        string = torrents[0]->torrentFile( );
-        foreach( const Torrent * t, torrents ) {
-            if( string != t->torrentFile( ) ) {
-                string = mixed;
-                break;
-            }
-        }
-    }
-    myTorrentFileLabel->setText( string );
 
     ///
     ///  Options Tab
@@ -665,18 +708,6 @@ Details :: refresh( )
     ///  Peers tab
     ///
 
-    i = 0;
-    foreach( const Torrent * t, torrents ) i += t->seeders( );
-    mySeedersLabel->setText( locale.toString( i ) );
-
-    i = 0;
-    foreach( const Torrent * t, torrents ) i += t->leechers( );
-    myLeechersLabel->setText( locale.toString( i ) );
-
-    i = 0;
-    foreach( const Torrent * t, torrents ) i += t->timesCompleted( );
-    myTimesCompletedLabel->setText( locale.toString( i ) );
-
     QMap<QString,QTreeWidgetItem*> peers2;
     QList<QTreeWidgetItem*> newItems;
     foreach( const Torrent * t, torrents )
@@ -759,7 +790,7 @@ Details :: refresh( )
 
     if( single )
         myFileTreeView->update( torrents[0]->files( ) );
-    else 
+    else
         myFileTreeView->clear( );
 
     myHavePendingRefresh = false;
@@ -780,25 +811,35 @@ Details :: enableWhenChecked( QCheckBox * box, QWidget * w )
 ***/
 
 QWidget *
-Details :: createActivityTab( )
+Details :: createInfoTab( )
 {
     HIG * hig = new HIG( this );
 
-    hig->addSectionTitle( tr( "Transfer" ) );
-    hig->addRow( tr( "State:" ), myStateLabel = new SqueezeLabel );
-    hig->addRow( tr( "Progress:" ), myProgressLabel = new SqueezeLabel );
+    hig->addSectionTitle( tr( "Activity" ) );
+    hig->addRow( tr( "Torrent size:" ), mySizeLabel = new SqueezeLabel );
     hig->addRow( tr( "Have:" ), myHaveLabel = new SqueezeLabel );
+    hig->addRow( tr( "Availability:" ), myAvailabilityLabel = new SqueezeLabel );
     hig->addRow( tr( "Downloaded:" ), myDownloadedLabel = new SqueezeLabel );
     hig->addRow( tr( "Uploaded:" ), myUploadedLabel = new SqueezeLabel );
-    hig->addRow( tr( "Failed DL:" ), myFailedLabel = new SqueezeLabel );
     hig->addRow( tr( "Ratio:" ), myRatioLabel = new SqueezeLabel );
-    hig->addRow( tr( "Swarm Rate:" ), mySwarmSpeedLabel = new SqueezeLabel );
+    hig->addRow( tr( "State:" ), myStateLabel = new SqueezeLabel );
+    hig->addRow( tr( "Running time:" ), myRunTimeLabel = new SqueezeLabel );
+    hig->addRow( tr( "Remaining time:" ), myETALabel = new SqueezeLabel );
+    hig->addRow( tr( "Last activity:" ), myLastActivityLabel = new SqueezeLabel );
     hig->addRow( tr( "Error:" ), myErrorLabel = new SqueezeLabel );
     hig->addSectionDivider( );
 
-    hig->addSectionTitle( tr( "Dates" ) );
-    hig->addRow( tr( "Added on:" ), myAddedDateLabel = new SqueezeLabel );
-    hig->addRow( tr( "Last activity on:" ), myActivityLabel = new SqueezeLabel );
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Details" ) );
+    hig->addRow( tr( "Location:" ), myLocationLabel = new SqueezeLabel );
+    hig->addRow( tr( "Hash:" ), myHashLabel = new SqueezeLabel );
+    hig->addRow( tr( "Privacy:" ), myPrivacyLabel = new SqueezeLabel );
+    hig->addRow( tr( "Origin:" ), myOriginLabel = new SqueezeLabel );
+    hig->addRow( tr( "Comment:" ), myCommentBrowser = new QTextBrowser );
+    const int h = QFontMetrics(myCommentBrowser->font()).lineSpacing() * 4;
+    myCommentBrowser->setMinimumHeight( h );
+    myCommentBrowser->setMaximumHeight( h );
+
     hig->finish( );
 
     return hig;
@@ -893,7 +934,7 @@ Details :: createOptionsTab( )
     hig->addWideControl( c );
     connect( c, SIGNAL(clicked(bool)), this, SLOT(onHonorsSessionLimitsToggled(bool)) );
 
-    c = new QCheckBox( tr( "Limit &download speed (KB/s)" ) );
+    c = new QCheckBox( tr( "Limit &download speed (KB/s):" ) );
     mySingleDownCheck = c;
     s = new QSpinBox( );
     mySingleDownSpin = s;
@@ -903,7 +944,7 @@ Details :: createOptionsTab( )
     connect( c, SIGNAL(clicked(bool)), this, SLOT(onDownloadLimitedToggled(bool)) );
     connect( s, SIGNAL(valueChanged(int)), this, SLOT(onDownloadLimitChanged(int)));
 
-    c = new QCheckBox( tr( "Limit &upload speed (KB/s)" ) );
+    c = new QCheckBox( tr( "Limit &upload speed (KB/s):" ) );
     mySingleUpCheck = c;
     s = new QSpinBox( );
     mySingleUpSpin = s;
@@ -914,18 +955,17 @@ Details :: createOptionsTab( )
     connect( s, SIGNAL(valueChanged(int)), this, SLOT(onUploadLimitChanged(int)));
 
     m = new QComboBox;
-    m->addItem( tr( "Low" ),    TR_PRI_LOW );
-    m->addItem( tr( "Normal" ), TR_PRI_NORMAL );
     m->addItem( tr( "High" ),   TR_PRI_HIGH );
+    m->addItem( tr( "Normal" ), TR_PRI_NORMAL );
+    m->addItem( tr( "Low" ),    TR_PRI_LOW );
     connect( m, SIGNAL(currentIndexChanged(int)), this, SLOT(onBandwidthPriorityChanged(int)));
-    hig->addRow( tr( "&Bandwidth priority:" ), m );
+    hig->addRow( tr( "Torrent &priority:" ), m );
     myBandwidthPriorityCombo = m;
-    
 
     hig->addSectionDivider( );
     hig->addSectionTitle( tr( "Seed-Until Ratio" ) );
 
-    r = new QRadioButton( tr( "Use &global setting" ) );
+    r = new QRadioButton( tr( "Use &global settings" ) );
     r->setProperty( RATIO_KEY, TR_RATIOLIMIT_GLOBAL );
     connect( r, SIGNAL(clicked(bool)), this, SLOT(onSeedUntilChanged(bool)));
     mySeedGlobalRadio = r;
@@ -939,7 +979,7 @@ Details :: createOptionsTab( )
 
     h = new QHBoxLayout( );
     h->setSpacing( HIG :: PAD );
-    r = new QRadioButton( tr( "&Stop seeding when a torrent's ratio reaches" ) );
+    r = new QRadioButton( tr( "&Seed torrent until its ratio reaches:" ) );
     r->setProperty( RATIO_KEY, TR_RATIOLIMIT_SINGLE );
     connect( r, SIGNAL(clicked(bool)), this, SLOT(onSeedUntilChanged(bool)));
     mySeedCustomRadio = r;
@@ -958,7 +998,7 @@ Details :: createOptionsTab( )
     s->setRange( 1, 300 );
     connect( s, SIGNAL(valueChanged(int)), this, SLOT(onMaxPeersChanged(int)));
     myPeerLimitSpin = s;
-    hig->addRow( tr( "&Maximum Peers" ), s );
+    hig->addRow( tr( "&Maximum peers:" ), s );
 
     hig->finish( );
 
@@ -969,38 +1009,6 @@ Details :: createOptionsTab( )
 ****
 ***/
 
-QWidget *
-Details :: createInfoTab( )
-{
-    HIG * hig = new HIG( );
-    hig->addSectionTitle( tr( "Details" ) );
-    hig->addRow( tr( "Pieces:" ), myPiecesLabel = new SqueezeLabel );
-    hig->addRow( tr( "Hash:" ), myHashLabel = new SqueezeLabel );
-    hig->addRow( tr( "Privacy:" ), myPrivacyLabel = new SqueezeLabel );
-    hig->addRow( tr( "Comment:" ), myCommentBrowser = new QTextBrowser );
-    hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Origins" ) );
-    hig->addRow( tr( "Creator:" ), myCreatorLabel = new SqueezeLabel );
-    hig->addRow( tr( "Date:" ), myDateCreatedLabel = new SqueezeLabel );
-    hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Origins" ) );
-    hig->addRow( tr( "Destination folder:" ), myDestinationLabel = new SqueezeLabel );
-    hig->addRow( tr( "Torrent file:" ), myTorrentFileLabel = new SqueezeLabel );
-    const int h = QFontMetrics(myCommentBrowser->font()).lineSpacing() * 4;
-    myTorrentFileLabel->setMinimumWidth( 300 );
-    myTorrentFileLabel->setSizePolicy ( QSizePolicy::Expanding, QSizePolicy::Preferred );
-
-    myCommentBrowser->setMinimumHeight( h );
-    myCommentBrowser->setMaximumHeight( h );
-
-    hig->finish( );
-    return hig;
-}
-
-/***
-****
-***/
-
 QWidget *
 Details :: createTrackerTab( )
 {
@@ -1061,30 +1069,6 @@ Details :: createPeersTab( )
     myPeerTree->setColumnWidth( COL_CLIENT, size.width( ) );
     myPeerTree->setAlternatingRowColors( true );
 
-    QHBoxLayout * h = new QHBoxLayout;
-    h->setSpacing( HIG :: PAD );
-    v->addLayout( h );
-
-    QLabel * l = new QLabel( "Seeders:" );
-    l->setStyleSheet( "font: bold" );
-    h->addWidget( l );
-    l = mySeedersLabel = new QLabel( "a" );
-    h->addWidget( l );
-    h->addStretch( 1 );
-    
-    l = new QLabel( "Leechers:" );
-    l->setStyleSheet( "font: bold" );
-    h->addWidget( l );
-    l = myLeechersLabel = new QLabel( "b" );
-    h->addWidget( l );
-    h->addStretch( 1 );
-    
-    l = new QLabel( "Times Completed:" );
-    l->setStyleSheet( "font: bold" );
-    h->addWidget( l );
-    l = myTimesCompletedLabel = new QLabel( "c" );
-    h->addWidget( l );
-
     return top;
 }
 
