@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,25 +7,28 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: prefs-dialog.cc 8570 2009-05-31 19:33:48Z charles $
+ * $Id: prefs-dialog.cc 10065 2010-02-01 04:43:10Z charles $
  */
 
 #include <cassert>
 #include <iostream>
 
-#include <QCoreApplication>
 #include <QCheckBox>
-#include <QFileDialog>
 #include <QComboBox>
+#include <QCoreApplication>
 #include <QDialogButtonBox>
 #include <QDoubleSpinBox>
+#include <QFileDialog>
+#include <QFileIconProvider>
+#include <QFileInfo>
 #include <QHBoxLayout>
+#include <QHttp>
+#include <QIcon>
 #include <QLabel>
 #include <QLineEdit>
-#include <QFileInfo>
 #include <QList>
+#include <QMessageBox>
 #include <QPushButton>
-#include <QIcon>
 #include <QSpinBox>
 #include <QStyle>
 #include <QTabWidget>
@@ -33,8 +36,6 @@
 #include <QTimeEdit>
 #include <QTimer>
 #include <QVBoxLayout>
-#include <QHttp>
-#include <QMessageBox>
 
 #include "hig.h"
 #include "prefs.h"
@@ -201,7 +202,7 @@ PrefsDialog :: createTrackerTab( )
     myProxyWidgets << l << r;
     l = hig->addRow( tr( "Proxy &port:" ), r = spinBoxNew( Prefs::PROXY_PORT, 1, 65535, 1 ) );
     myProxyWidgets << l << r;
-    hig->addWideControl( l = checkBoxNew( tr( "Require &authentication" ), Prefs::PROXY_AUTH_ENABLED ) );
+    hig->addWideControl( l = checkBoxNew( tr( "Use &authentication" ), Prefs::PROXY_AUTH_ENABLED ) );
     myProxyWidgets << l;
     l = hig->addRow( tr( "&Username:" ), r = lineEditNew( Prefs::PROXY_USERNAME ) );
     myProxyAuthWidgets << l << r;
@@ -220,25 +221,25 @@ QWidget *
 PrefsDialog :: createWebTab( Session& session )
 {
     HIG * hig = new HIG( this );
-    hig->addSectionTitle( tr( "Web Interface" ) );
+    hig->addSectionTitle( tr( "Web Client" ) );
     QWidget * w;
     QHBoxLayout * h = new QHBoxLayout( );
     QIcon i( style()->standardIcon( QStyle::StandardPixmap( QStyle::SP_DirOpenIcon ) ) );
-    QPushButton * b = new QPushButton( i, tr( "&Open web interface" ) );
+    QPushButton * b = new QPushButton( i, tr( "&Open web client" ) );
     connect( b, SIGNAL(clicked()), &session, SLOT(launchWebInterface()) );
     h->addWidget( b, 0, Qt::AlignRight );
-    QWidget * l = checkBoxNew( tr( "&Enable web interface" ), Prefs::RPC_ENABLED );
+    QWidget * l = checkBoxNew( tr( "&Enable web client" ), Prefs::RPC_ENABLED );
     myUnsupportedWhenRemote << l;
     hig->addRow( l, h, 0 );
     l = hig->addRow( tr( "Listening &port:" ), w = spinBoxNew( Prefs::RPC_PORT, 0, 65535, 1 ) );
     myWebWidgets << l << w;
-    hig->addWideControl( w = checkBoxNew( tr( "&Require username" ), Prefs::RPC_AUTH_REQUIRED ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Use &authentication" ), Prefs::RPC_AUTH_REQUIRED ) );
     myWebWidgets << w;
     l = hig->addRow( tr( "&Username:" ), w = lineEditNew( Prefs::RPC_USERNAME ) );
     myWebAuthWidgets << l << w;
     l = hig->addRow( tr( "Pass&word:" ), w = lineEditNew( Prefs::RPC_PASSWORD, QLineEdit::Password ) );
     myWebAuthWidgets << l << w;
-    hig->addWideControl( w = checkBoxNew( tr( "Only allow the following IP &addresses to connect:" ), Prefs::RPC_WHITELIST_ENABLED ) );
+    hig->addWideControl( w = checkBoxNew( tr( "Only allow these IP a&ddresses to connect:" ), Prefs::RPC_WHITELIST_ENABLED ) );
     myWebWidgets << w;
     l = hig->addRow( tr( "Addresses:" ), w = lineEditNew( Prefs::RPC_WHITELIST ) );
     myWebWhitelistWidgets << l << w;
@@ -260,11 +261,11 @@ PrefsDialog :: altSpeedDaysEdited( int i )
 
 
 QWidget *
-PrefsDialog :: createBandwidthTab( )
+PrefsDialog :: createSpeedTab( )
 {
     QWidget *l, *r;
     HIG * hig = new HIG( this );
-    hig->addSectionTitle( tr( "Global Bandwidth Limits" ) );
+    hig->addSectionTitle( tr( "Speed Limits" ) );
 
         l = checkBoxNew( tr( "Limit &download speed (KB/s):" ), Prefs::DSPEED_ENABLED );
         r = spinBoxNew( Prefs::DSPEED, 0, INT_MAX, 5 );
@@ -283,13 +284,16 @@ PrefsDialog :: createBandwidthTab( )
     label->setPixmap( QPixmap( ":/icons/alt-limit-off.png" ) );
     label->setAlignment( Qt::AlignLeft|Qt::AlignVCenter );
     h->addWidget( label );
-    label = new QLabel( tr( "Speed Limit Mode" ) );
+    label = new QLabel( tr( "Temporary Speed Limits" ) );
     label->setStyleSheet( "font: bold" );
     label->setAlignment( Qt::AlignLeft|Qt::AlignVCenter );
     h->addWidget( label );
     hig->addSectionTitle( h );
 
-        QString s = tr( "Limit d&ownload speed (KB/s):" );
+        QString s = tr( "<small>Override normal speed limits manually or at scheduled times</small>" );
+        hig->addWideControl( new QLabel( s ) );
+
+        s = tr( "Limit d&ownload speed (KB/s):" );
         r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_DOWN, 0, INT_MAX, 5 );
         hig->addRow( s, r );
 
@@ -297,19 +301,17 @@ PrefsDialog :: createBandwidthTab( )
         r = spinBoxNew( Prefs :: ALT_SPEED_LIMIT_UP, 0, INT_MAX, 5 );
         hig->addRow( s, r );
 
-        s = tr( "<small>When enabled, Speed Limit Mode overrides the Global Bandwidth Limits</small>" );
-        hig->addWideControl( new QLabel( s ) );
-
-        QCheckBox * c = checkBoxNew( tr( "Use Speed Limit Mode &between" ), Prefs::ALT_SPEED_LIMIT_TIME_ENABLED );
+        QCheckBox * c = checkBoxNew( tr( "&Scheduled times:" ), Prefs::ALT_SPEED_LIMIT_TIME_ENABLED );
         h = new QHBoxLayout( );
         h->setSpacing( HIG::PAD );
         QWidget * w = timeEditNew( Prefs :: ALT_SPEED_LIMIT_TIME_BEGIN );
         h->addWidget( w, 1 );
         mySchedWidgets << w;
-        w = new QLabel( "and" );
-        h->addWidget( w );
-        mySchedWidgets << w;
+        QLabel * nd = new QLabel( "&to" );
+        h->addWidget( nd );
+        mySchedWidgets << nd;
         w = timeEditNew( Prefs :: ALT_SPEED_LIMIT_TIME_END );
+        nd->setBuddy( w );
         h->addWidget( w, 1 );
         mySchedWidgets << w;
         hig->addRow( c, h, 0 );
@@ -364,21 +366,26 @@ PrefsDialog :: createNetworkTab( )
     HIG * hig = new HIG( this );
     hig->addSectionTitle( tr( "Incoming Peers" ) );
 
-        QSpinBox * s = spinBoxNew( Prefs::PEER_PORT, 1, 65535, 1 );
-        QHBoxLayout * h = new QHBoxLayout( );
-        QPushButton * b = myPortButton = new QPushButton( tr( "&Test Port" ) );
-        QLabel * l = myPortLabel = new QLabel( tr( "Status unknown" ) );
-        h->addWidget( l );
-        h->addSpacing( HIG :: PAD_BIG );
-        h->addWidget( b );
-        h->setStretchFactor( l, 1 );
-        connect( b, SIGNAL(clicked(bool)), this, SLOT(onPortTest()));
-        connect( &mySession, SIGNAL(portTested(bool)), this, SLOT(onPortTested(bool)));
+    QSpinBox * s = spinBoxNew( Prefs::PEER_PORT, 1, 65535, 1 );
+    QHBoxLayout * h = new QHBoxLayout( );
+    QPushButton * b = myPortButton = new QPushButton( tr( "&Test Port" ) );
+    QLabel * l = myPortLabel = new QLabel( tr( "Status unknown" ) );
+    h->addWidget( l );
+    h->addSpacing( HIG :: PAD_BIG );
+    h->addWidget( b );
+    h->setStretchFactor( l, 1 );
+    connect( b, SIGNAL(clicked(bool)), this, SLOT(onPortTest()));
+    connect( &mySession, SIGNAL(portTested(bool)), this, SLOT(onPortTested(bool)));
 
-        hig->addRow( tr( "&Port for incoming connections:" ), s );
-        hig->addRow( "", h, 0 );
-        hig->addWideControl( checkBoxNew( tr( "&Randomize the port every launch" ), Prefs :: PEER_PORT_RANDOM_ON_START ) );
-        hig->addWideControl( checkBoxNew( tr( "Use UPnP or NAT-PMP port &forwarding from my router" ), Prefs::PORT_FORWARDING ) );
+    hig->addRow( tr( "&Port for incoming connections:" ), s );
+    hig->addRow( "", h, 0 );
+    hig->addWideControl( checkBoxNew( tr( "Use UPnP or NAT-PMP port &forwarding from my router" ), Prefs::PORT_FORWARDING ) );
+    hig->addWideControl( checkBoxNew( tr( "Pick a &random port every time Transmission is started" ), Prefs :: PEER_PORT_RANDOM_ON_START ) );
+
+    hig->addSectionDivider( );
+    hig->addSectionTitle( tr( "Limits" ) );
+    hig->addRow( tr( "Maximum peers per &torrent:" ), spinBoxNew( Prefs::PEER_LIMIT_TORRENT, 1, 300, 5 ) );
+    hig->addRow( tr( "Maximum peers &overall:" ), spinBoxNew( Prefs::PEER_LIMIT_GLOBAL, 1, 3000, 5 ) );
 
     hig->finish( );
     return hig;
@@ -429,7 +436,7 @@ PrefsDialog :: onUpdateBlocklistClicked( )
     myBlocklistDialog->show( );
     mySession.updateBlocklist( );
 }
-    
+
 void
 PrefsDialog :: encryptionEdited( int i )
 {
@@ -438,7 +445,7 @@ PrefsDialog :: encryptionEdited( int i )
 }
 
 QWidget *
-PrefsDialog :: createPeersTab( )
+PrefsDialog :: createPrivacyTab( )
 {
     HIG * hig = new HIG( this );
     hig->addSectionTitle( tr( "Blocklist" ) );
@@ -456,23 +463,18 @@ PrefsDialog :: createPeersTab( )
     myBlockWidgets << l;
     hig->addWideControl( l );
 
-    hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Limits" ) );
-    hig->addRow( tr( "Maximum peers &overall:" ), spinBoxNew( Prefs::PEER_LIMIT_GLOBAL, 1, 3000, 5 ) );
-    hig->addRow( tr( "Maximum peers per &torrent:" ), spinBoxNew( Prefs::PEER_LIMIT_TORRENT, 1, 300, 5 ) );
-
     QComboBox * box = new QComboBox( );
-    box->addItem( tr( "Plaintext Preferred" ), 0 );
-    box->addItem( tr( "Encryption Preferred" ), 1 );
-    box->addItem( tr( "Encryption Required" ), 2 );
+    box->addItem( tr( "Allow encryption" ), 0 );
+    box->addItem( tr( "Prefer encryption" ), 1 );
+    box->addItem( tr( "Require encryption" ), 2 );
     myWidgets.insert( Prefs :: ENCRYPTION, box );
     connect( box, SIGNAL(activated(int)), this, SLOT(encryptionEdited(int)));
 
     hig->addSectionDivider( );
     hig->addSectionTitle( tr( "Privacy" ) );
     hig->addRow( tr( "&Encryption mode:" ), box );
-    hig->addWideControl( checkBoxNew( tr( "Use peer e&xchange (PEX)" ), Prefs::PEX_ENABLED ) );
-    hig->addWideControl( checkBoxNew( tr( "Use &distributed hash table (DHT)" ), Prefs::PEX_ENABLED ) );
+    hig->addWideControl( checkBoxNew( tr( "Use PE&X to find more peers" ), Prefs::PEX_ENABLED ) );
+    hig->addWideControl( checkBoxNew( tr( "Use &DHT to find more peers" ), Prefs::DHT_ENABLED ) );
 
     hig->finish( );
     updateBlocklistCheckBox( );
@@ -483,14 +485,35 @@ PrefsDialog :: createPeersTab( )
 ****
 ***/
 
+void
+PrefsDialog :: onIncompleteClicked( void )
+{
+    const QString title = tr( "Select Incomplete Directory" );
+    const QString path = myPrefs.getString( Prefs::INCOMPLETE_DIR );
+    QFileDialog * d = new QFileDialog( this, title, path );
+    d->setFileMode( QFileDialog::Directory );
+    connect( d, SIGNAL(filesSelected(const QStringList&)),
+             this, SLOT(onIncompleteSelected(const QStringList&)) );
+    d->show( );
+}
+
+void
+PrefsDialog :: onIncompleteSelected( const QStringList& list )
+{
+    if( list.size() == 1 )
+        myPrefs.set( Prefs::INCOMPLETE_DIR, list.first( ) );
+}
+
+
 void
 PrefsDialog :: onWatchClicked( void )
 {
-    QFileDialog * d = new QFileDialog( this,
-                                       tr( "Select Watch Directory" ),
-                                       myPrefs.getString( Prefs::DIR_WATCH ) );
+    const QString title = tr( "Select Watch Directory" );
+    const QString path = myPrefs.getString( Prefs::DIR_WATCH );
+    QFileDialog * d = new QFileDialog( this, title, path );
     d->setFileMode( QFileDialog::Directory );
-    connect( d, SIGNAL(filesSelected(const QStringList&)), this, SLOT(onWatchSelected(const QStringList&)) );
+    connect( d, SIGNAL(filesSelected(const QStringList&)),
+             this, SLOT(onWatchSelected(const QStringList&)) );
     d->show( );
 }
 
@@ -504,11 +527,12 @@ PrefsDialog :: onWatchSelected( const QStringList& list )
 void
 PrefsDialog :: onDestinationClicked( void )
 {
-    QFileDialog * d = new QFileDialog( this,
-                                       tr( "Select Destination" ),
-                                       myPrefs.getString( Prefs::DOWNLOAD_DIR ) );
+    const QString title = tr( "Select Destination" );
+    const QString path = myPrefs.getString( Prefs::DOWNLOAD_DIR );
+    QFileDialog * d = new QFileDialog( this, title, path );
     d->setFileMode( QFileDialog::Directory );
-    connect( d, SIGNAL(filesSelected(const QStringList&)), this, SLOT(onDestinationSelected(const QStringList&)) );
+    connect( d, SIGNAL(filesSelected(const QStringList&)),
+             this, SLOT(onDestinationSelected(const QStringList&)) );
     d->show( );
 }
 
@@ -523,7 +547,8 @@ QWidget *
 PrefsDialog :: createTorrentsTab( )
 {
     const int iconSize( style( )->pixelMetric( QStyle :: PM_SmallIconSize ) );
-    const QIcon folderIcon = QtIconLoader :: icon( "folder", style()->standardIcon( QStyle::SP_DirIcon ) );
+    const QFileIconProvider iconProvider;
+    const QIcon folderIcon = iconProvider.icon( QFileIconProvider::Folder );
     const QPixmap folderPixmap = folderIcon.pixmap( iconSize );
 
     QWidget *l, *r;
@@ -538,20 +563,27 @@ PrefsDialog :: createTorrentsTab( )
         hig->addRow( l, b );
         enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), b );
 
-        hig->addWideControl( checkBoxNew( tr( "Display &options dialog" ), Prefs::OPTIONS_PROMPT ) );
+        hig->addWideControl( checkBoxNew( tr( "Show &options dialog" ), Prefs::OPTIONS_PROMPT ) );
         hig->addWideControl( checkBoxNew( tr( "&Start when added" ), Prefs::START ) );
-        hig->addWideControl( checkBoxNew( tr( "&Delete source files" ), Prefs::TRASH_ORIGINAL ) );
+        hig->addWideControl( checkBoxNew( tr( "Mo&ve .torrent file to the trash" ), Prefs::TRASH_ORIGINAL ) );
+        hig->addWideControl( checkBoxNew( tr( "Append \".&part\" to incomplete files' names" ), Prefs::RENAME_PARTIAL_FILES ) );
+
+        b = myIncompleteButton = new QPushButton;
+        b->setIcon( folderPixmap );
+        b->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        connect( b, SIGNAL(clicked(bool)), this, SLOT(onDestinationClicked(void)) );
+        hig->addRow( tr( "Keep &incomplete files in:" ), b );
 
         b = myDestinationButton = new QPushButton;
         b->setIcon( folderPixmap );
         b->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
         connect( b, SIGNAL(clicked(bool)), this, SLOT(onDestinationClicked(void)) );
-        hig->addRow( tr( "&Destination folder:" ), b );
+        hig->addRow( tr( "Save to &Location:" ), b );
 
     hig->addSectionDivider( );
     hig->addSectionTitle( tr( "Limits" ) );
 
-        l = checkBoxNew( tr( "&Stop seeding torrents at ratio:" ), Prefs::RATIO_ENABLED );
+        l = checkBoxNew( tr( "&Seed torrent until its ratio reaches:" ), Prefs::RATIO_ENABLED );
         r = doubleSpinBoxNew( Prefs::RATIO, 0, INT_MAX, 0.5, 2 );
         hig->addRow( l, r );
         enableBuddyWhenChecked( qobject_cast<QCheckBox*>(l), r );
@@ -574,12 +606,12 @@ PrefsDialog :: PrefsDialog( Session& session, Prefs& prefs, QWidget * parent ):
     setWindowTitle( tr( "Transmission Preferences" ) );
 
     QTabWidget * t = new QTabWidget( this );
-    t->addTab( createTorrentsTab( ),       tr( "Torrents" ) );
-    t->addTab( createPeersTab( ),          tr( "Peers" ) );
-    t->addTab( createBandwidthTab( ),      tr( "Speed" ) );
-    t->addTab( createNetworkTab( ),        tr( "Network" ) );
-    t->addTab( createWebTab( session ),    tr( "Web" ) );
-    //t->addTab( createTrackerTab( ),        tr( "Trackers" ) );
+    t->addTab( createTorrentsTab( ),     tr( "Torrents" ) );
+    t->addTab( createSpeedTab( ),        tr( "Speed" ) );
+    t->addTab( createPrivacyTab( ),      tr( "Privacy" ) );
+    t->addTab( createNetworkTab( ),      tr( "Network" ) );
+    t->addTab( createWebTab( session ),  tr( "Web" ) );
+    //t->addTab( createTrackerTab( ),    tr( "Trackers" ) );
     myLayout->addWidget( t );
 
     QDialogButtonBox * buttons = new QDialogButtonBox( QDialogButtonBox::Close, Qt::Horizontal, this );
@@ -597,7 +629,9 @@ PrefsDialog :: PrefsDialog( Session& session, Prefs& prefs, QWidget * parent ):
          << Prefs :: ENCRYPTION
          << Prefs :: BLOCKLIST_ENABLED
          << Prefs :: DIR_WATCH
-         << Prefs :: DOWNLOAD_DIR;
+         << Prefs :: DOWNLOAD_DIR
+         << Prefs :: INCOMPLETE_DIR
+         << Prefs :: INCOMPLETE_DIR_ENABLED;
     foreach( int key, keys )
         updatePref( key );
 
@@ -647,8 +681,8 @@ PrefsDialog :: updatePref( int key )
             const bool enabled( myPrefs.getBool( Prefs::RPC_ENABLED ) );
             const bool whitelist( myPrefs.getBool( Prefs::RPC_WHITELIST_ENABLED ) );
             const bool auth( myPrefs.getBool( Prefs::RPC_AUTH_REQUIRED ) );
-            foreach( QWidget * w, myWebWhitelistWidgets ) w->setEnabled( enabled && whitelist ); 
-            foreach( QWidget * w, myWebAuthWidgets ) w->setEnabled( enabled && auth ); 
+            foreach( QWidget * w, myWebWhitelistWidgets ) w->setEnabled( enabled && whitelist );
+            foreach( QWidget * w, myWebAuthWidgets ) w->setEnabled( enabled && auth );
             foreach( QWidget * w, myWebWidgets ) w->setEnabled( enabled );
             break;
         }
@@ -657,7 +691,7 @@ PrefsDialog :: updatePref( int key )
         case Prefs :: PROXY_AUTH_ENABLED: {
             const bool enabled( myPrefs.getBool( Prefs::PROXY_ENABLED ) );
             const bool auth( myPrefs.getBool( Prefs::PROXY_AUTH_ENABLED ) );
-            foreach( QWidget * w, myProxyAuthWidgets ) w->setEnabled( enabled && auth ); 
+            foreach( QWidget * w, myProxyAuthWidgets ) w->setEnabled( enabled && auth );
             foreach( QWidget * w, myProxyWidgets ) w->setEnabled( enabled );
             break;
         }
@@ -684,11 +718,23 @@ PrefsDialog :: updatePref( int key )
             break;
 
         case Prefs :: DOWNLOAD_DIR: {
-            QString path( myPrefs.getString( Prefs :: DOWNLOAD_DIR ) );
+            QString path( myPrefs.getString( key ) );
             myDestinationButton->setText( QFileInfo(path).fileName() );
             break;
         }
 
+        case Prefs :: INCOMPLETE_DIR: {
+            QString path( myPrefs.getString( key ) );
+            myIncompleteButton->setText( QFileInfo(path).fileName() );
+            break;
+        }
+
+        case Prefs :: INCOMPLETE_DIR_ENABLED: {
+            const bool enabled = myPrefs.getBool( key );
+            myIncompleteButton->setEnabled( enabled );
+            break;
+        }
+
         default:
             break;
     }
