@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,17 +7,18 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.cc 8570 2009-05-31 19:33:48Z charles $
+ * $Id: torrent.cc 10077 2010-02-02 05:34:26Z charles $
  */
 
 #include <cassert>
 #include <iostream>
 
 #include <QApplication>
-#include <QStyle>
+#include <QFileIconProvider>
+#include <QFileInfo>
 #include <QSet>
 #include <QString>
-#include <QFileInfo>
+#include <QStyle>
 #include <QVariant>
 
 #include <libtransmission/transmission.h>
@@ -26,7 +27,6 @@
 
 #include "app.h"
 #include "prefs.h"
-#include "qticonloader.h"
 #include "torrent.h"
 #include "utils.h"
 
@@ -55,15 +55,16 @@ Torrent :: myProperties[] =
     { ID, "id", QVariant::Int, INFO, },
     { UPLOAD_SPEED, "rateUpload", QVariant::Int, STAT } /* B/s */,
     { DOWNLOAD_SPEED, "rateDownload", QVariant::Int, STAT }, /* B/s */
-    { SWARM_SPEED, "swarmSpeed", QVariant::Int, STAT_EXTRA },/* KB/s */
     { DOWNLOAD_DIR, "downloadDir", QVariant::String, STAT },
     { ACTIVITY, "status", QVariant::Int, STAT },
     { NAME, "name", QVariant::String, INFO },
-    { ERROR, "errorString", QVariant::String, STAT },
+    { ERROR, "error", QVariant::Int, STAT },
+    { ERROR_STRING, "errorString", QVariant::String, STAT },
     { SIZE_WHEN_DONE, "sizeWhenDone", QVariant::ULongLong, STAT },
     { LEFT_UNTIL_DONE, "leftUntilDone", QVariant::ULongLong, STAT },
     { HAVE_UNCHECKED, "haveUnchecked", QVariant::ULongLong, STAT },
     { HAVE_VERIFIED, "haveValid", QVariant::ULongLong, STAT },
+    { DESIRED_AVAILABLE, "desiredAvailable", QVariant::ULongLong, STAT },
     { TOTAL_SIZE, "totalSize", QVariant::ULongLong, INFO },
     { PIECE_SIZE, "pieceSize", QVariant::ULongLong, INFO },
     { PIECE_COUNT, "pieceCount", QVariant::Int, INFO },
@@ -71,6 +72,7 @@ Torrent :: myProperties[] =
     { PEERS_SENDING_TO_US, "peersSendingToUs", QVariant::Int, STAT },
     { WEBSEEDS_SENDING_TO_US, "webseedsSendingToUs", QVariant::Int, STAT_EXTRA },
     { PERCENT_DONE, "percentDone", QVariant::Double, STAT },
+    { METADATA_PERCENT_DONE, "metadataPercentComplete", QVariant::Double, STAT },
     { PERCENT_VERIFIED, "recheckProgress", QVariant::Double, STAT },
     { DATE_ACTIVITY, "activityDate", QVariant::DateTime, STAT_EXTRA },
     { DATE_ADDED, "addedDate", QVariant::DateTime, INFO },
@@ -104,9 +106,6 @@ Torrent :: myProperties[] =
     { SCRAPE_RESPONSE, "scrapeResponse", QVariant::String, STAT_EXTRA },
     { ANNOUNCE_RESPONSE, "announceResponse", QVariant::String, STAT_EXTRA },
     { ANNOUNCE_URL, "announceURL", QVariant::String, STAT_EXTRA },
-    { SEEDERS, "seeders", QVariant::Int, STAT_EXTRA },
-    { LEECHERS, "leechers", QVariant::Int, STAT_EXTRA },
-    { TIMES_COMPLETED, "timesCompleted", QVariant::Int, STAT_EXTRA },
     { PEERS, "peers", TrTypes::PeerList, STAT_EXTRA },
     { TORRENT_FILE, "torrentFile", QVariant::String, STAT_EXTRA },
     { BANDWIDTH_PRIORITY, "bandwidthPriority", QVariant::Int, STAT_EXTRA }
@@ -218,7 +217,7 @@ Torrent :: setDateTime( int i, const QDateTime& value )
 
     return changed;
 }
-  
+
 bool
 Torrent :: setSize( int i, qulonglong value )
 {
@@ -415,9 +414,11 @@ Torrent :: updateMimeIcon( )
     QIcon icon;
 
     if( files.size( ) > 1 )
-        icon = QtIconLoader :: icon( "folder", QApplication::style()->standardIcon( QStyle::SP_DirIcon ) );
-    else
+        icon = QFileIconProvider().icon( QFileIconProvider::Folder );
+    else if( files.size( ) == 1 )
         icon = Utils :: guessMimeIcon( files.at(0).filename );
+    else
+        icon = QIcon( );
 
     setIcon( MIME_ICON, icon );
 }
@@ -498,7 +499,6 @@ Torrent :: update( tr_benc * d )
             default:
                 assert( 0 && "unhandled type" );
         }
-        
     }
 
     tr_benc * files;
@@ -620,3 +620,19 @@ Torrent :: activityString( ) const
 
     return str;
 }
+
+QString
+Torrent :: getError( ) const
+{
+    QString s = getString( ERROR_STRING );
+
+    switch( getInt( ERROR ) )
+    {
+        case TR_STAT_TRACKER_WARNING: s = tr( "Tracker gave a warning: %1" ).arg( s ); break;
+        case TR_STAT_TRACKER_ERROR: s = tr( "Tracker gave an error: %1" ).arg( s ); break;
+        case TR_STAT_LOCAL_ERROR: s = tr( "Error: %1" ).arg( s ); break;
+        default: s.clear(); break;
+    }
+
+    return s;
+}
